@@ -2,8 +2,6 @@ from __future__ import annotations
 
 import os
 
-from daft.logging import setup_logger
-
 ###
 # Set up code coverage for when running code coverage with ray
 ###
@@ -20,12 +18,6 @@ if "COV_CORE_SOURCE" in os.environ:
             "Environ: {!r} "
             "Exception: {!r}\n".format({k: v for k, v in os.environ.items() if k.startswith("COV_CORE")}, exc)
         )
-###
-# Setup logging
-###
-
-
-setup_logger()
 
 ###
 # Get build constants from Rust .so
@@ -62,6 +54,7 @@ if not dev_build and not user_opted_out:
 # Daft top-level imports
 ###
 
+from daft.context import set_execution_config, set_planning_config
 from daft.convert import (
     from_arrow,
     from_dask_dataframe,
@@ -74,7 +67,7 @@ from daft.daft import ImageFormat, ImageMode, ResourceRequest
 from daft.dataframe import DataFrame
 from daft.datatype import DataType, TimeUnit
 from daft.expressions import col, lit
-from daft.io import from_glob_path, read_csv, read_json, read_parquet
+from daft.io import from_glob_path, read_csv, read_iceberg, read_json, read_parquet
 from daft.series import Series
 from daft.udf import udf
 from daft.viz import register_viz_hook
@@ -90,6 +83,7 @@ __all__ = [
     "read_csv",
     "read_json",
     "read_parquet",
+    "read_iceberg",
     "DataFrame",
     "col",
     "DataType",
@@ -101,4 +95,6 @@ __all__ = [
     "register_viz_hook",
     "udf",
     "ResourceRequest",
+    "set_planning_config",
+    "set_execution_config",
 ]
