@@ -5,6 +5,7 @@
 # For technical details, see https://github.com/Eventual-Inc/Daft/pull/630
 
 import pathlib
+import warnings
 from dataclasses import dataclass
 from functools import reduce
 from typing import (
@@ -24,13 +25,7 @@ from typing import (
 from daft.api_annotations import DataframePublicAPI
 from daft.context import get_context
 from daft.convert import InputListType
-from daft.daft import (
-    FileFormat,
-    JoinType,
-    PartitionScheme,
-    PartitionSpec,
-    ResourceRequest,
-)
+from daft.daft import FileFormat, IOConfig, JoinType, PartitionScheme, ResourceRequest
 from daft.dataframe.preview import DataFramePreview
 from daft.datatype import DataType
 from daft.errors import ExpressionTypeError
@@ -38,7 +33,7 @@ from daft.expressions import Expression, ExpressionsProjection, col, lit
 from daft.logical.builder import LogicalPlanBuilder
 from daft.runners.partitioning import PartitionCacheEntry, PartitionSet
 from daft.runners.pyrunner import LocalPartitionSet
-from daft.table import Table
+from daft.table import MicroPartition
 from daft.viz import DataFrameDisplay
 
 if TYPE_CHECKING:
@@ -89,8 +84,17 @@ class DataFrame:
         if self._result_cache is None:
             return self.__builder
         else:
+            num_partitions = self._result_cache.num_partitions()
+            size_bytes = self._result_cache.size_bytes()
+            # Partition set should always be set on cache entry.
+            assert (
+                num_partitions is not None and size_bytes is not None
+            ), "Partition set should always be set on cache entry"
             return self.__builder.from_in_memory_scan(
-                self._result_cache, self.__builder.schema(), self.__builder.partition_spec()
+                self._result_cache,
+                self.__builder.schema(),
+                num_partitions=num_partitions,
+                size_bytes=size_bytes,
             )
 
     def _get_current_builder(self) -> LogicalPlanBuilder:
@@ -127,7 +131,9 @@ class DataFrame:
         print(builder.pretty_print(simple))
 
     def num_partitions(self) -> int:
-        return self.__builder.num_partitions()
+        daft_execution_config = get_context().daft_execution_config
+        # We need to run the optimizer since that could change the number of partitions
+        return self.__builder.optimize().to_physical_plan_scheduler(daft_execution_config).num_partitions()
 
     @DataframePublicAPI
     def schema(self) -> Schema:
@@ -156,34 +162,6 @@ class DataFrame:
         """
         return [col(field.name) for field in self.__builder.schema()]
 
-    @DataframePublicAPI
-    def show(self, n: int = 8) -> "DataFrameDisplay":
-        """Executes enough of the DataFrame in order to display the first ``n`` rows
-
-        .. NOTE::
-            This call is **blocking** and will execute the DataFrame when called
-
-        Args:
-            n: number of rows to show. Defaults to 8.
-
-        Returns:
-            DataFrameDisplay: object that has a rich tabular display
-        """
-        df = self
-        df = df.limit(n)
-        df.collect(num_preview_rows=None)
-        collected_preview = df._preview
-        assert collected_preview is not None
-
-        preview = DataFramePreview(
-            preview_partition=collected_preview.preview_partition,
-            # Override dataframe_num_rows=None, because we do not know
-            # the size of the entire (un-limited) dataframe when showing
-            dataframe_num_rows=None,
-        )
-
-        return DataFrameDisplay(preview, self.schema(), num_rows=n)
-
     @DataframePublicAPI
     def __iter__(self) -> Iterator[Dict[str, Any]]:
         """Return an iterator of rows for this dataframe.
@@ -214,7 +192,7 @@ class DataFrame:
                     yield row
 
     @DataframePublicAPI
-    def iter_partitions(self) -> Iterator[Union[Table, "RayObjectRef"]]:
+    def iter_partitions(self) -> Iterator[Union[MicroPartition, "RayObjectRef"]]:
         """Begin executing this dataframe and return an iterator over the partitions.
 
         Each partition will be returned as a daft.Table object (if using Python runner backend)
@@ -228,8 +206,9 @@ class DataFrame:
         else:
             # Execute the dataframe in a streaming fashion.
             context = get_context()
-            partitions_iter = context.runner().run_iter(self._builder)
-            yield from partitions_iter
+            results_iter = context.runner().run_iter(self._builder)
+            for result in results_iter:
+                yield result.partition()
 
     @DataframePublicAPI
     def __repr__(self) -> str:
@@ -265,7 +244,7 @@ class DataFrame:
                 f"Expected all columns to be of the same length, but received columns with lengths: {column_lengths}"
             )
 
-        data_vpartition = Table.from_pydict(data)
+        data_vpartition = MicroPartition.from_pydict(data)
         return cls._from_tables(data_vpartition)
 
     @classmethod
@@ -273,7 +252,7 @@ class DataFrame:
         """Creates a DataFrame from a pyarrow Table."""
         if not isinstance(data, list):
             data = [data]
-        data_vpartitions = [Table.from_arrow(table) for table in data]
+        data_vpartitions = [MicroPartition.from_arrow(table) for table in data]
         return cls._from_tables(*data_vpartitions)
 
     @classmethod
@@ -281,11 +260,11 @@ class DataFrame:
         """Creates a Daft DataFrame from a pandas DataFrame."""
         if not isinstance(data, list):
             data = [data]
-        data_vpartitions = [Table.from_pandas(df) for df in data]
+        data_vpartitions = [MicroPartition.from_pandas(df) for df in data]
         return cls._from_tables(*data_vpartitions)
 
     @classmethod
-    def _from_tables(cls, *parts: Table) -> "DataFrame":
+    def _from_tables(cls, *parts: MicroPartition) -> "DataFrame":
         """Creates a Daft DataFrame from a single Table.
 
         Args:
@@ -301,7 +280,11 @@ class DataFrame:
 
         context = get_context()
         cache_entry = context.runner().put_partition_set_into_cache(result_pset)
-        builder = context.logical_plan_builder_class().from_in_memory_scan(cache_entry, parts[0].schema())
+        size_bytes = result_pset.size_bytes()
+        assert size_bytes is not None, "In-memory data should always have non-None size in bytes"
+        builder = LogicalPlanBuilder.from_in_memory_scan(
+            cache_entry, parts[0].schema(), result_pset.num_partitions(), size_bytes
+        )
         return cls(builder)
 
     ###
@@ -314,6 +297,7 @@ class DataFrame:
         root_dir: Union[str, pathlib.Path],
         compression: str = "snappy",
         partition_cols: Optional[List[ColumnInputType]] = None,
+        io_config: Optional[IOConfig] = None,
     ) -> "DataFrame":
         """Writes the DataFrame as parquet files, returning a new DataFrame with paths to the files that were written
 
@@ -328,6 +312,7 @@ class DataFrame:
             root_dir (str): root file path to write parquet files to.
             compression (str, optional): compression algorithm. Defaults to "snappy".
             partition_cols (Optional[List[ColumnInputType]], optional): How to subpartition each partition further. Currently only supports Column Expressions with any calls. Defaults to None.
+            io_config (Optional[IOConfig], optional): configurations to use when interacting with remote storage.
 
         Returns:
             DataFrame: The filenames that were written out as strings.
@@ -335,12 +320,14 @@ class DataFrame:
             .. NOTE::
                 This call is **blocking** and will execute the DataFrame when called
         """
+        io_config = get_context().daft_planning_config.default_io_config if io_config is None else io_config
+
         cols: Optional[List[Expression]] = None
         if partition_cols is not None:
             cols = self.__column_input_to_expression(tuple(partition_cols))
             for c in cols:
                 assert c._is_column(), "we cant support non Column Expressions for partition writing"
-            self.repartition(self.num_partitions(), *cols)
+            self.repartition(None, *cols)
         else:
             pass
         builder = self._builder.write_tabular(
@@ -348,6 +335,7 @@ class DataFrame:
             partition_cols=cols,
             file_format=FileFormat.Parquet,
             compression=compression,
+            io_config=io_config,
         )
         # Block and write, then retrieve data and return a new disconnected DataFrame
         write_df = DataFrame(builder)
@@ -357,7 +345,10 @@ class DataFrame:
 
     @DataframePublicAPI
     def write_csv(
-        self, root_dir: Union[str, pathlib.Path], partition_cols: Optional[List[ColumnInputType]] = None
+        self,
+        root_dir: Union[str, pathlib.Path],
+        partition_cols: Optional[List[ColumnInputType]] = None,
+        io_config: Optional[IOConfig] = None,
     ) -> "DataFrame":
         """Writes the DataFrame as CSV files, returning a new DataFrame with paths to the files that were written
 
@@ -372,22 +363,26 @@ class DataFrame:
             root_dir (str): root file path to write parquet files to.
             compression (str, optional): compression algorithm. Defaults to "snappy".
             partition_cols (Optional[List[ColumnInputType]], optional): How to subpartition each partition further. Currently only supports Column Expressions with any calls. Defaults to None.
+            io_config (Optional[IOConfig], optional): configurations to use when interacting with remote storage.
 
         Returns:
             DataFrame: The filenames that were written out as strings.
         """
+        io_config = get_context().daft_planning_config.default_io_config if io_config is None else io_config
+
         cols: Optional[List[Expression]] = None
         if partition_cols is not None:
             cols = self.__column_input_to_expression(tuple(partition_cols))
             for c in cols:
                 assert c._is_column(), "we cant support non Column Expressions for partition writing"
-            self.repartition(self.num_partitions(), *cols)
+            self.repartition(None, *cols)
         else:
             pass
         builder = self._builder.write_tabular(
             root_dir=root_dir,
             partition_cols=cols,
             file_format=FileFormat.Csv,
+            io_config=io_config,
         )
 
         # Block and write, then retrieve data and return a new disconnected DataFrame
@@ -484,6 +479,27 @@ class DataFrame:
         builder = self._builder.distinct()
         return DataFrame(builder)
 
+    @DataframePublicAPI
+    def sample(self, fraction: float, with_replacement: bool = False, seed: Optional[int] = None) -> "DataFrame":
+        """Samples a fraction of rows from the DataFrame
+
+        Example:
+            >>> sampled_df = df.sample(0.5)
+
+        Args:
+            fraction (float): fraction of rows to sample.
+            with_replacement (bool, optional): whether to sample with replacement. Defaults to False.
+            seed (Optional[int], optional): random seed. Defaults to None.
+
+        Returns:
+            DataFrame: DataFrame with a fraction of rows.
+        """
+        if fraction < 0.0 or fraction > 1.0:
+            raise ValueError(f"fraction should be between 0.0 and 1.0, but got {fraction}")
+
+        builder = self._builder.sample(fraction, with_replacement, seed)
+        return DataFrame(builder)
+
     @DataframePublicAPI
     def exclude(self, *names: str) -> "DataFrame":
         """Drops columns from the current DataFrame by name
@@ -586,11 +602,13 @@ class DataFrame:
 
         Args:
             num (int): maximum rows to allow.
+            eager (bool): whether to maximize for latency (time to first result) by eagerly executing
+                only one partition at a time, or throughput by executing multiple limits at a time
 
         Returns:
             DataFrame: Limited DataFrame
         """
-        builder = self._builder.limit(num)
+        builder = self._builder.limit(num, eager=False)
         return DataFrame(builder)
 
     @DataframePublicAPI
@@ -607,7 +625,7 @@ class DataFrame:
         return count_df.to_pydict()["count"][0]
 
     @DataframePublicAPI
-    def repartition(self, num: int, *partition_by: ColumnInputType) -> "DataFrame":
+    def repartition(self, num: Optional[int], *partition_by: ColumnInputType) -> "DataFrame":
         """Repartitions DataFrame to ``num`` partitions
 
         If columns are passed in, then DataFrame will be repartitioned by those, otherwise
@@ -618,8 +636,8 @@ class DataFrame:
             >>> part_by_df = df.repartition(4, 'x', col('y') + 1)
 
         Args:
-            num (int): number of target partitions.
-            *partition_by (Union[str, Expression]): optional columns to partition by.
+            num (Optional[int]): Number of target partitions; if None, the number of partitions will not be changed.
+            *partition_by (Union[str, Expression]): Optional columns to partition by.
 
         Returns:
             DataFrame: Repartitioned DataFrame.
@@ -650,24 +668,12 @@ class DataFrame:
         Returns:
             DataFrame: Dataframe with ``num`` partitions.
         """
-        current_partitions = self._builder.num_partitions()
-
-        if num > current_partitions:
-            # Do a split (increase the number of partitions).
-            builder = self._builder.repartition(
-                num_partitions=num,
-                partition_by=[],
-                scheme=PartitionScheme.Unknown,
-            )
-            return DataFrame(builder)
-
-        elif num < current_partitions:
-            # Do a coalese (decrease the number of partitions).
-            builder = self._builder.coalesce(num)
-            return DataFrame(builder)
-
-        else:
-            return self
+        builder = self._builder.repartition(
+            num_partitions=num,
+            partition_by=[],
+            scheme=PartitionScheme.Unknown,
+        )
+        return DataFrame(builder)
 
     @DataframePublicAPI
     def join(
@@ -858,7 +864,11 @@ class DataFrame:
         Returns:
             DataFrame: Globally aggregated sums. Should be a single row.
         """
-        assert len(cols) > 0, "no columns were passed in"
+        if len(cols) == 0:
+            warnings.warn(
+                "No columns specified; performing sum on all columns. Specify columns using df.sum('col1', 'col2', ...)."
+            )
+            cols = tuple(self.columns)
         return self._agg([(c, "sum") for c in cols])
 
     @DataframePublicAPI
@@ -870,7 +880,11 @@ class DataFrame:
         Returns:
             DataFrame: Globally aggregated mean. Should be a single row.
         """
-        assert len(cols) > 0, "no columns were passed in"
+        if len(cols) == 0:
+            warnings.warn(
+                "No columns specified; performing mean on all columns. Specify columns using df.mean('col1', 'col2', ...)."
+            )
+            cols = tuple(self.columns)
         return self._agg([(c, "mean") for c in cols])
 
     @DataframePublicAPI
@@ -882,7 +896,11 @@ class DataFrame:
         Returns:
             DataFrame: Globally aggregated min. Should be a single row.
         """
-        assert len(cols) > 0, "no columns were passed in"
+        if len(cols) == 0:
+            warnings.warn(
+                "No columns specified; performing min on all columns. Specify columns using df.min('col1', 'col2', ...)."
+            )
+            cols = tuple(self.columns)
         return self._agg([(c, "min") for c in cols])
 
     @DataframePublicAPI
@@ -894,7 +912,11 @@ class DataFrame:
         Returns:
             DataFrame: Globally aggregated max. Should be a single row.
         """
-        assert len(cols) > 0, "no columns were passed in"
+        if len(cols) == 0:
+            warnings.warn(
+                "No columns specified; performing max on all columns. Specify columns using df.max('col1', 'col2', ...)."
+            )
+            cols = tuple(self.columns)
         return self._agg([(c, "max") for c in cols])
 
     @DataframePublicAPI
@@ -906,7 +928,11 @@ class DataFrame:
         Returns:
             DataFrame: Globally aggregated count. Should be a single row.
         """
-        assert len(cols) > 0, "no columns were passed in"
+        if len(cols) == 0:
+            warnings.warn(
+                "No columns specified; performing count on all columns. Specify columns using df.count('col1', 'col2', ...) or use df.count_rows() for row counts."
+            )
+            cols = tuple(self.columns)
         return self._agg([(c, "count") for c in cols])
 
     @DataframePublicAPI
@@ -918,7 +944,11 @@ class DataFrame:
         Returns:
             DataFrame: Globally aggregated list. Should be a single row.
         """
-        assert len(cols) > 0, "no columns were passed in"
+        if len(cols) == 0:
+            warnings.warn(
+                "No columns specified; performing agg_list on all columns. Specify columns using df.agg_list('col1', 'col2', ...)."
+            )
+            cols = tuple(self.columns)
         return self._agg([(c, "list") for c in cols])
 
     @DataframePublicAPI
@@ -930,7 +960,11 @@ class DataFrame:
         Returns:
             DataFrame: Globally aggregated list. Should be a single row.
         """
-        assert len(cols) > 0, "no columns were passed in"
+        if len(cols) == 0:
+            warnings.warn(
+                "No columns specified; performing agg_concat on all columns. Specify columns using df.agg_concat('col1', 'col2', ...)."
+            )
+            cols = tuple(self.columns)
         return self._agg([(c, "concat") for c in cols])
 
     @DataframePublicAPI
@@ -1015,6 +1049,73 @@ class DataFrame:
 
         return self
 
+    def _construct_show_display(self, n: int) -> "DataFrameDisplay":
+        """Helper for .show() which will construct the underlying DataFrameDisplay object"""
+        preview_partition = self._preview.preview_partition
+        total_rows = self._preview.dataframe_num_rows
+
+        # Truncate n to the length of the DataFrame, if we have it.
+        if total_rows is not None and n > total_rows:
+            n = total_rows
+
+        # Construct the PreviewPartition
+        if preview_partition is None or len(preview_partition) < n:
+            # Preview partition doesn't exist or doesn't contain enough rows, so we need to compute a
+            # new one from scratch.
+            builder = self._builder.limit(n, eager=True)
+
+            # Iteratively retrieve partitions until enough data has been materialized
+            tables = []
+            seen = 0
+            for table in get_context().runner().run_iter_tables(builder, results_buffer_size=1):
+                tables.append(table)
+                seen += len(table)
+                if seen >= n:
+                    break
+
+            preview_partition = MicroPartition.concat(tables)
+            if len(preview_partition) > n:
+                preview_partition = preview_partition.slice(0, n)
+            elif len(preview_partition) < n:
+                # Iterator short-circuited before reaching n, so we know that we have the full DataFrame.
+                total_rows = n = len(preview_partition)
+            preview = DataFramePreview(
+                preview_partition=preview_partition,
+                dataframe_num_rows=total_rows,
+            )
+        elif len(preview_partition) > n:
+            # Preview partition is cached but has more rows that we need, so use the appropriate slice.
+            truncated_preview_partition = preview_partition.slice(0, n)
+            preview = DataFramePreview(preview_partition=truncated_preview_partition, dataframe_num_rows=total_rows)
+        else:
+            assert len(preview_partition) == n
+            # Preview partition is cached and has exactly the number of rows that we need, so use it directly.
+            preview = self._preview
+
+        return DataFrameDisplay(preview, self.schema(), num_rows=n)
+
+    @DataframePublicAPI
+    def show(self, n: int = 8) -> None:
+        """Executes enough of the DataFrame in order to display the first ``n`` rows
+
+        If IPython is installed, this will use IPython's `display` utility to pretty-print in a
+        notebook/REPL environment. Otherwise, this will fall back onto a naive Python `print`.
+
+        .. NOTE::
+            This call is **blocking** and will execute the DataFrame when called
+
+        Args:
+            n: number of rows to show. Defaults to 8.
+        """
+        dataframe_display = self._construct_show_display(n)
+        try:
+            from IPython.display import display
+
+            display(dataframe_display, clear=True)
+        except ImportError:
+            print(dataframe_display)
+        return None
+
     def __len__(self):
         """Returns the count of rows when dataframe is materialized.
         If dataframe is not materialized yet, raises a runtime error.
@@ -1168,10 +1269,13 @@ class DataFrame:
 
         partition_set, schema = ray_runner_io.partition_set_from_ray_dataset(ds)
         cache_entry = context.runner().put_partition_set_into_cache(partition_set)
-        builder = context.logical_plan_builder_class().from_in_memory_scan(
+        size_bytes = partition_set.size_bytes()
+        assert size_bytes is not None, "In-memory data should always have non-None size in bytes"
+        builder = LogicalPlanBuilder.from_in_memory_scan(
             cache_entry,
             schema=schema,
-            partition_spec=PartitionSpec(PartitionScheme.Unknown, partition_set.num_partitions()),
+            num_partitions=partition_set.num_partitions(),
+            size_bytes=size_bytes,
         )
         return cls(builder)
 
@@ -1237,10 +1341,13 @@ class DataFrame:
 
         partition_set, schema = ray_runner_io.partition_set_from_dask_dataframe(ddf)
         cache_entry = context.runner().put_partition_set_into_cache(partition_set)
-        builder = context.logical_plan_builder_class().from_in_memory_scan(
+        size_bytes = partition_set.size_bytes()
+        assert size_bytes is not None, "In-memory data should always have non-None size in bytes"
+        builder = LogicalPlanBuilder.from_in_memory_scan(
             cache_entry,
             schema=schema,
-            partition_spec=PartitionSpec(PartitionScheme.Unknown, partition_set.num_partitions()),
+            num_partitions=partition_set.num_partitions(),
+            size_bytes=size_bytes,
         )
         return cls(builder)
 
