@@ -18,7 +18,7 @@ try:
 except ImportError:
     _RAY_DATA_EXTENSIONS_AVAILABLE = False
 else:
-    _RAY_VERSION = tuple(int(s) for s in ray.__version__.split("."))
+    _RAY_VERSION = tuple(int(s) for s in ray.__version__.split(".")[0:3])
     try:
         # Variable-shaped tensor column support was added in Ray 2.1.0.
         if _RAY_VERSION >= (2, 2, 0):
@@ -194,13 +194,17 @@ class DataType:
         return cls._from_pydatatype(PyDataType.date())
 
     @classmethod
-    def timestamp(cls, timeunit: TimeUnit, timezone: str | None = None) -> DataType:
+    def timestamp(cls, timeunit: TimeUnit | str, timezone: str | None = None) -> DataType:
         """Timestamp DataType."""
+        if isinstance(timeunit, str):
+            timeunit = TimeUnit.from_str(timeunit)
         return cls._from_pydatatype(PyDataType.timestamp(timeunit._timeunit, timezone))
 
     @classmethod
-    def duration(cls, timeunit: TimeUnit) -> DataType:
+    def duration(cls, timeunit: TimeUnit | str) -> DataType:
         """Duration DataType."""
+        if isinstance(timeunit, str):
+            timeunit = TimeUnit.from_str(timeunit)
         return cls._from_pydatatype(PyDataType.duration(timeunit._timeunit))
 
     @classmethod
@@ -341,7 +345,11 @@ class DataType:
             return cls.float64()
         elif pa.types.is_string(arrow_type) or pa.types.is_large_string(arrow_type):
             return cls.string()
-        elif pa.types.is_binary(arrow_type) or pa.types.is_large_binary(arrow_type):
+        elif (
+            pa.types.is_binary(arrow_type)
+            or pa.types.is_large_binary(arrow_type)
+            or pa.types.is_fixed_size_binary(arrow_type)
+        ):
             return cls.binary()
         elif pa.types.is_boolean(arrow_type):
             return cls.bool()
