@@ -4,7 +4,7 @@ import itertools
 import pathlib
 import sys
 from dataclasses import dataclass, field
-from typing import Generic, TypeVar
+from typing import Generic
 
 if sys.version_info < (3, 8):
     from typing_extensions import Protocol
@@ -15,7 +15,9 @@ from daft.daft import (
     CsvSourceConfig,
     FileFormat,
     FileFormatConfig,
+    IOConfig,
     JoinType,
+    JsonReadOptions,
     JsonSourceConfig,
     ParquetSourceConfig,
     ResourceRequest,
@@ -25,14 +27,15 @@ from daft.expressions import Expression, ExpressionsProjection, col
 from daft.logical.map_partition_ops import MapPartitionOp
 from daft.logical.schema import Schema
 from daft.runners.partitioning import (
+    MaterializedResult,
     PartialPartitionMetadata,
     PartitionMetadata,
+    PartitionT,
     TableParseCSVOptions,
     TableReadOptions,
 )
-from daft.table import Table, table_io
+from daft.table import MicroPartition, table_io
 
-PartitionT = TypeVar("PartitionT")
 ID_GEN = itertools.count()
 
 
@@ -50,6 +53,7 @@ class PartitionTask(Generic[PartitionT]):
     instructions: list[Instruction]
     resource_request: ResourceRequest
     num_results: int
+    stage_id: int
     _id: int = field(default_factory=lambda: next(ID_GEN))
 
     def id(self) -> str:
@@ -70,7 +74,7 @@ class PartitionTask(Generic[PartitionT]):
     def __str__(self) -> str:
         return (
             f"{self.id()}\n"
-            f"  Inputs: {self.inputs}\n"
+            f"  Inputs: <{len(self.inputs)} partitions>\n"
             f"  Resource Request: {self.resource_request}\n"
             f"  Instructions: {[i.__class__.__name__ for i in self.instructions]}"
         )
@@ -78,6 +82,9 @@ class PartitionTask(Generic[PartitionT]):
     def __repr__(self) -> str:
         return self.__str__()
 
+    def name(self) -> str:
+        return f"{'-'.join(i.__class__.__name__ for i in self.instructions)} [Stage:{self.stage_id}]"
+
 
 class PartitionTaskBuilder(Generic[PartitionT]):
     """Builds a PartitionTask by adding instructions to its pipeline."""
@@ -109,7 +116,7 @@ class PartitionTaskBuilder(Generic[PartitionT]):
         self.num_results = instruction.num_outputs()
         return self
 
-    def finalize_partition_task_single_output(self) -> SingleOutputPartitionTask[PartitionT]:
+    def finalize_partition_task_single_output(self, stage_id: int) -> SingleOutputPartitionTask[PartitionT]:
         """Create a SingleOutputPartitionTask from this PartitionTaskBuilder.
 
         Returns a "frozen" version of this PartitionTask that cannot have instructions added.
@@ -124,12 +131,13 @@ class PartitionTaskBuilder(Generic[PartitionT]):
 
         return SingleOutputPartitionTask[PartitionT](
             inputs=self.inputs,
+            stage_id=stage_id,
             instructions=self.instructions,
             num_results=1,
             resource_request=resource_request_final_cpu,
         )
 
-    def finalize_partition_task_multi_output(self) -> MultiOutputPartitionTask[PartitionT]:
+    def finalize_partition_task_multi_output(self, stage_id: int) -> MultiOutputPartitionTask[PartitionT]:
         """Create a MultiOutputPartitionTask from this PartitionTaskBuilder.
 
         Same as finalize_partition_task_single_output, except the output of this PartitionTask is a list of partitions.
@@ -142,6 +150,7 @@ class PartitionTaskBuilder(Generic[PartitionT]):
         )
         return MultiOutputPartitionTask[PartitionT](
             inputs=self.inputs,
+            stage_id=stage_id,
             instructions=self.instructions,
             num_results=self.num_results,
             resource_request=resource_request_final_cpu,
@@ -150,7 +159,7 @@ class PartitionTaskBuilder(Generic[PartitionT]):
     def __str__(self) -> str:
         return (
             f"PartitionTaskBuilder\n"
-            f"  Inputs: {self.inputs}\n"
+            f"  Inputs: <{len(self.inputs)} partitions>\n"
             f"  Resource Request: {self.resource_request}\n"
             f"  Instructions: {[i.__class__.__name__ for i in self.instructions]}"
         )
@@ -171,28 +180,29 @@ class SingleOutputPartitionTask(PartitionTask[PartitionT]):
     def done(self) -> bool:
         return self._result is not None
 
+    def result(self) -> MaterializedResult[PartitionT]:
+        assert self._result is not None, "Cannot call .result() on a PartitionTask that is not done"
+        return self._result
+
     def cancel(self) -> None:
         # Currently only implemented for Ray tasks.
-        if self._result is not None:
-            self._result.cancel()
+        if self.done():
+            self.result().cancel()
 
     def partition(self) -> PartitionT:
         """Get the PartitionT resulting from running this PartitionTask."""
-        assert self._result is not None
-        return self._result.partition()
+        return self.result().partition()
 
     def partition_metadata(self) -> PartitionMetadata:
         """Get the metadata of the result partition.
 
         (Avoids retrieving the actual partition itself if possible.)
         """
-        assert self._result is not None
-        return self._result.metadata()
+        return self.result().metadata()
 
-    def vpartition(self) -> Table:
+    def vpartition(self) -> MicroPartition:
         """Get the raw vPartition of the result."""
-        assert self._result is not None
-        return self._result.vpartition()
+        return self.result().vpartition()
 
     def __str__(self) -> str:
         return super().__str__()
@@ -235,7 +245,7 @@ class MultiOutputPartitionTask(PartitionTask[PartitionT]):
         assert self._results is not None
         return [result.metadata() for result in self._results]
 
-    def vpartition(self, index: int) -> Table:
+    def vpartition(self, index: int) -> MicroPartition:
         """Get the raw vPartition of the result."""
         assert self._results is not None
         return self._results[index].vpartition()
@@ -247,35 +257,6 @@ class MultiOutputPartitionTask(PartitionTask[PartitionT]):
         return super().__str__()
 
 
-class MaterializedResult(Protocol[PartitionT]):
-    """A protocol for accessing the result partition of a PartitionTask.
-
-    Different Runners can fill in their own implementation here.
-    """
-
-    def partition(self) -> PartitionT:
-        """Get the partition of this result."""
-        ...
-
-    def vpartition(self) -> Table:
-        """Get the vPartition of this result."""
-        ...
-
-    def metadata(self) -> PartitionMetadata:
-        """Get the metadata of the partition in this result."""
-        ...
-
-    def cancel(self) -> None:
-        """If possible, cancel execution of this PartitionTask."""
-        ...
-
-    def _noop(self, _: PartitionT) -> None:
-        """Implement this as a no-op.
-        https://peps.python.org/pep-0544/#overriding-inferred-variance-of-protocol-classes
-        """
-        ...
-
-
 class Instruction(Protocol):
     """An instruction is a function to run over a list of partitions.
 
@@ -285,7 +266,7 @@ class Instruction(Protocol):
     To accomodate these, instructions are typed as list[Table] -> list[Table].
     """
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         """Run the Instruction over the input partitions.
 
         Note: Dispatching a descriptively named helper here will aid profiling.
@@ -318,10 +299,10 @@ class ReadFile(SingleOutputInstruction):
     columns_to_read: list[str] | None
     file_format_config: FileFormatConfig
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._read_file(inputs)
 
-    def _read_file(self, inputs: list[Table]) -> list[Table]:
+    def _read_file(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         assert len(inputs) == 1
         [filepaths_partition] = inputs
         partition = self._handle_tabular_files_scan(
@@ -346,8 +327,8 @@ class ReadFile(SingleOutputInstruction):
 
     def _handle_tabular_files_scan(
         self,
-        filepaths_partition: Table,
-    ) -> Table:
+        filepaths_partition: MicroPartition,
+    ) -> MicroPartition:
         data = filepaths_partition.to_pydict()
         filepaths = data["path"]
 
@@ -364,7 +345,7 @@ class ReadFile(SingleOutputInstruction):
         format_config = self.file_format_config.config
         if file_format == FileFormat.Csv:
             assert isinstance(format_config, CsvSourceConfig)
-            table = Table.concat(
+            table = MicroPartition.concat(
                 [
                     table_io.read_csv(
                         file=fp,
@@ -373,6 +354,12 @@ class ReadFile(SingleOutputInstruction):
                         csv_options=TableParseCSVOptions(
                             delimiter=format_config.delimiter,
                             header_index=0 if format_config.has_headers else None,
+                            double_quote=format_config.double_quote,
+                            quote=format_config.quote,
+                            escape_char=format_config.escape_char,
+                            comment=format_config.comment,
+                            buffer_size=format_config.buffer_size,
+                            chunk_size=format_config.chunk_size,
                         ),
                         read_options=read_options,
                     )
@@ -381,12 +368,15 @@ class ReadFile(SingleOutputInstruction):
             )
         elif file_format == FileFormat.Json:
             assert isinstance(format_config, JsonSourceConfig)
-            table = Table.concat(
+            table = MicroPartition.concat(
                 [
                     table_io.read_json(
                         file=fp,
                         schema=self.schema,
                         storage_config=self.storage_config,
+                        json_read_options=JsonReadOptions(
+                            buffer_size=format_config.buffer_size, chunk_size=format_config.chunk_size
+                        ),
                         read_options=read_options,
                     )
                     for fp in filepaths
@@ -394,7 +384,7 @@ class ReadFile(SingleOutputInstruction):
             )
         elif file_format == FileFormat.Parquet:
             assert isinstance(format_config, ParquetSourceConfig)
-            table = Table.concat(
+            table = MicroPartition.concat(
                 [
                     table_io.read_parquet(
                         file=fp,
@@ -426,11 +416,12 @@ class WriteFile(SingleOutputInstruction):
     root_dir: str | pathlib.Path
     compression: str | None
     partition_cols: ExpressionsProjection | None
+    io_config: IOConfig | None
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._write_file(inputs)
 
-    def _write_file(self, inputs: list[Table]) -> list[Table]:
+    def _write_file(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         [input] = inputs
         partition = self._handle_file_write(
             input=input,
@@ -446,13 +437,14 @@ class WriteFile(SingleOutputInstruction):
             )
         ]
 
-    def _handle_file_write(self, input: Table) -> Table:
+    def _handle_file_write(self, input: MicroPartition) -> MicroPartition:
         if self.file_format == FileFormat.Parquet:
             file_names = table_io.write_parquet(
                 input,
                 path=self.root_dir,
                 compression=self.compression,
                 partition_cols=self.partition_cols,
+                io_config=self.io_config,
             )
         elif self.file_format == FileFormat.Csv:
             file_names = table_io.write_csv(
@@ -460,6 +452,7 @@ class WriteFile(SingleOutputInstruction):
                 path=self.root_dir,
                 compression=self.compression,
                 partition_cols=self.partition_cols,
+                io_config=self.io_config,
             )
         else:
             raise ValueError(
@@ -467,7 +460,7 @@ class WriteFile(SingleOutputInstruction):
             )
 
         assert len(self.schema) == 1
-        return Table.from_pydict(
+        return MicroPartition.from_pydict(
             {
                 self.schema.column_names()[0]: file_names,
             }
@@ -478,10 +471,10 @@ class WriteFile(SingleOutputInstruction):
 class Filter(SingleOutputInstruction):
     predicate: ExpressionsProjection
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._filter(inputs)
 
-    def _filter(self, inputs: list[Table]) -> list[Table]:
+    def _filter(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         [input] = inputs
         return [input.filter(self.predicate)]
 
@@ -499,10 +492,10 @@ class Filter(SingleOutputInstruction):
 class Project(SingleOutputInstruction):
     projection: ExpressionsProjection
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._project(inputs)
 
-    def _project(self, inputs: list[Table]) -> list[Table]:
+    def _project(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         [input] = inputs
         return [input.eval_expression_list(self.projection)]
 
@@ -520,12 +513,12 @@ class Project(SingleOutputInstruction):
 class LocalCount(SingleOutputInstruction):
     schema: Schema
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._count(inputs)
 
-    def _count(self, inputs: list[Table]) -> list[Table]:
+    def _count(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         [input] = inputs
-        partition = Table.from_pydict({"count": [len(input)]})
+        partition = MicroPartition.from_pydict({"count": [len(input)]})
         assert partition.schema() == self.schema
         return [partition]
 
@@ -542,10 +535,10 @@ class LocalCount(SingleOutputInstruction):
 class LocalLimit(SingleOutputInstruction):
     limit: int
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._limit(inputs)
 
-    def _limit(self, inputs: list[Table]) -> list[Table]:
+    def _limit(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         [input] = inputs
         return [input.head(self.limit)]
 
@@ -559,14 +552,19 @@ class LocalLimit(SingleOutputInstruction):
         ]
 
 
+@dataclass(frozen=True)
+class GlobalLimit(LocalLimit):
+    pass
+
+
 @dataclass(frozen=True)
 class MapPartition(SingleOutputInstruction):
     map_op: MapPartitionOp
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._map_partition(inputs)
 
-    def _map_partition(self, inputs: list[Table]) -> list[Table]:
+    def _map_partition(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         [input] = inputs
         return [self.map_op.run(input)]
 
@@ -582,19 +580,25 @@ class MapPartition(SingleOutputInstruction):
 
 @dataclass(frozen=True)
 class Sample(SingleOutputInstruction):
-    sort_by: ExpressionsProjection
-    num_samples: int = 20
+    fraction: float | None = None
+    size: int | None = None
+    with_replacement: bool = False
+    seed: int | None = None
+    sort_by: ExpressionsProjection | None = None
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._sample(inputs)
 
-    def _sample(self, inputs: list[Table]) -> list[Table]:
+    def _sample(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         [input] = inputs
-        result = (
-            input.sample(self.num_samples)
-            .eval_expression_list(self.sort_by)
-            .filter(ExpressionsProjection([~col(e.name()).is_null() for e in self.sort_by]))
-        )
+        if self.sort_by:
+            result = (
+                input.sample(self.fraction, self.size, self.with_replacement, self.seed)
+                .eval_expression_list(self.sort_by)
+                .filter(ExpressionsProjection([~col(e.name()).is_null() for e in self.sort_by]))
+            )
+        else:
+            result = input.sample(self.fraction, self.size, self.with_replacement, self.seed)
         return [result]
 
     def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
@@ -612,10 +616,10 @@ class Aggregate(SingleOutputInstruction):
     to_agg: list[Expression]
     group_by: ExpressionsProjection | None
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._aggregate(inputs)
 
-    def _aggregate(self, inputs: list[Table]) -> list[Table]:
+    def _aggregate(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         [input] = inputs
         return [input.agg(self.to_agg, self.group_by)]
 
@@ -634,12 +638,23 @@ class Join(SingleOutputInstruction):
     left_on: ExpressionsProjection
     right_on: ExpressionsProjection
     how: JoinType
+    is_swapped: bool
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._join(inputs)
 
-    def _join(self, inputs: list[Table]) -> list[Table]:
-        [left, right] = inputs
+    def _join(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        # All inputs except for the last are the left side of the join, in order to support left-broadcasted joins.
+        *lefts, right = inputs
+        if len(lefts) > 1:
+            # NOTE: MicroPartition concats don't concatenate the underlying column arrays, since MicroPartitions are chunked.
+            left = MicroPartition.concat(lefts)
+        else:
+            left = lefts[0]
+        if self.is_swapped:
+            # Swap left/right back.
+            # We don't need to swap left_on and right_on since those were never swapped in the first place.
+            left, right = right, left
         result = left.join(
             right,
             left_on=self.left_on,
@@ -664,11 +679,11 @@ class ReduceInstruction(SingleOutputInstruction):
 
 @dataclass(frozen=True)
 class ReduceMerge(ReduceInstruction):
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._reduce_merge(inputs)
 
-    def _reduce_merge(self, inputs: list[Table]) -> list[Table]:
-        return [Table.concat(inputs)]
+    def _reduce_merge(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        return [MicroPartition.concat(inputs)]
 
     def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
         input_rows = [_.num_rows for _ in input_metadatas]
@@ -686,11 +701,11 @@ class ReduceMergeAndSort(ReduceInstruction):
     sort_by: ExpressionsProjection
     descending: list[bool]
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._reduce_merge_and_sort(inputs)
 
-    def _reduce_merge_and_sort(self, inputs: list[Table]) -> list[Table]:
-        partition = Table.concat(inputs).sort(self.sort_by, descending=self.descending)
+    def _reduce_merge_and_sort(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        partition = MicroPartition.concat(inputs).sort(self.sort_by, descending=self.descending)
         return [partition]
 
     def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
@@ -710,11 +725,11 @@ class ReduceToQuantiles(ReduceInstruction):
     sort_by: ExpressionsProjection
     descending: list[bool]
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._reduce_to_quantiles(inputs)
 
-    def _reduce_to_quantiles(self, inputs: list[Table]) -> list[Table]:
-        merged = Table.concat(inputs)
+    def _reduce_to_quantiles(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        merged = MicroPartition.concat(inputs)
 
         # Skip evaluation of expressions by converting to Column Expression, since evaluation was done in Sample
         merged_sorted = merged.sort(self.sort_by.to_column_expressions(), descending=self.descending)
@@ -753,10 +768,10 @@ class FanoutInstruction(Instruction):
 class FanoutRandom(FanoutInstruction):
     seed: int
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._fanout_random(inputs)
 
-    def _fanout_random(self, inputs: list[Table]) -> list[Table]:
+    def _fanout_random(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         [input] = inputs
         return input.partition_by_random(num_partitions=self._num_outputs, seed=self.seed)
 
@@ -765,10 +780,10 @@ class FanoutRandom(FanoutInstruction):
 class FanoutHash(FanoutInstruction):
     partition_by: ExpressionsProjection
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._fanout_hash(inputs)
 
-    def _fanout_hash(self, inputs: list[Table]) -> list[Table]:
+    def _fanout_hash(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         [input] = inputs
         return input.partition_by_hash(self.partition_by, num_partitions=self._num_outputs)
 
@@ -778,21 +793,23 @@ class FanoutRange(FanoutInstruction, Generic[PartitionT]):
     sort_by: ExpressionsProjection
     descending: list[bool]
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._fanout_range(inputs)
 
-    def _fanout_range(self, inputs: list[Table]) -> list[Table]:
+    def _fanout_range(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         [boundaries, input] = inputs
         if self._num_outputs == 1:
             return [input]
-        partitioned_tables = input.partition_by_range(self.sort_by, boundaries, self.descending)
+
+        table_boundaries = boundaries.to_table()
+        partitioned_tables = input.partition_by_range(self.sort_by, table_boundaries, self.descending)
 
         # Pad the partitioned_tables with empty tables if fewer than self._num_outputs were returned
         # This can happen when all values are null or empty, which leads to an empty `boundaries` input
         assert len(partitioned_tables) >= 1, "Should have at least one returned table"
         schema = partitioned_tables[0].schema()
         partitioned_tables = partitioned_tables + [
-            Table.empty(schema=schema) for _ in range(self._num_outputs - len(partitioned_tables))
+            MicroPartition.empty(schema=schema) for _ in range(self._num_outputs - len(partitioned_tables))
         ]
 
         return partitioned_tables
@@ -802,10 +819,10 @@ class FanoutRange(FanoutInstruction, Generic[PartitionT]):
 class FanoutSlices(FanoutInstruction):
     slices: list[tuple[int, int]]  # start inclusive, end exclusive
 
-    def run(self, inputs: list[Table]) -> list[Table]:
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._multislice(inputs)
 
-    def _multislice(self, inputs: list[Table]) -> list[Table]:
+    def _multislice(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         [input] = inputs
         results = []
 
