@@ -5,14 +5,15 @@ import sys
 from datetime import date, datetime
 from typing import TYPE_CHECKING, Callable, Iterable, Iterator, TypeVar, overload
 
-import fsspec
 import pyarrow as pa
 
 from daft import context
 from daft.daft import CountMode, ImageFormat
 from daft.daft import PyExpr as _PyExpr
 from daft.daft import col as _col
+from daft.daft import date_lit as _date_lit
 from daft.daft import lit as _lit
+from daft.daft import timestamp_lit as _timestamp_lit
 from daft.daft import udf as _udf
 from daft.datatype import DataType, TimeUnit
 from daft.expressions.testing import expr_structurally_equal
@@ -40,18 +41,16 @@ def lit(value: object) -> Expression:
         Expression: Expression representing the value provided
     """
     if isinstance(value, datetime):
+        # pyo3 datetime (PyDateTime) is not available when running in abi3 mode, workaround
         pa_timestamp = pa.scalar(value)
         i64_value = pa_timestamp.cast(pa.int64()).as_py()
-        return lit(i64_value).cast(
-            DataType.timestamp(
-                TimeUnit.from_str(pa_timestamp.type.unit),
-                pa_timestamp.type.tz,
-            )
-        )
-
+        time_unit = TimeUnit.from_str(pa_timestamp.type.unit)._timeunit
+        tz = pa_timestamp.type.tz
+        lit_value = _timestamp_lit(i64_value, time_unit, tz)
     elif isinstance(value, date):
+        # pyo3 date (PyDate) is not available when running in abi3 mode, workaround
         epoch_time = value - date(1970, 1, 1)
-        return lit(epoch_time.days).cast(DataType.date())
+        lit_value = _date_lit(epoch_time.days)
     else:
         lit_value = _lit(value)
     return Expression._from_pyexpr(lit_value)
@@ -103,6 +102,11 @@ class Expression:
         """Access methods that work on columns of lists"""
         return ExpressionListNamespace.from_expression(self)
 
+    @property
+    def struct(self) -> ExpressionStructNamespace:
+        """Access methods that work on columns of structs"""
+        return ExpressionStructNamespace.from_expression(self)
+
     @property
     def image(self) -> ExpressionImageNamespace:
         """Access methods that work on columns of images"""
@@ -370,6 +374,19 @@ class Expression:
         expr = self._expr.is_null()
         return Expression._from_pyexpr(expr)
 
+    def not_null(self) -> Expression:
+        """Checks if values in the Expression are not Null (a special value indicating missing data)
+
+        Example:
+            >>> # [1., None, NaN] -> [True, False, True]
+            >>> col("x").not_null()
+
+        Returns:
+            Expression: Boolean Expression indicating whether values are not missing
+        """
+        expr = self._expr.not_null()
+        return Expression._from_pyexpr(expr)
+
     def name(self) -> builtins.str:
         return self._expr.name()
 
@@ -399,9 +416,6 @@ class Expression:
     def _is_column(self) -> bool:
         return self._expr._is_column()
 
-    def _replace_column_with_expression(self, column: builtins.str, new_expr: Expression) -> Expression:
-        return Expression._from_pyexpr(self._expr._replace_column_with_expression(column, new_expr._expr))
-
 
 SomeExpressionNamespace = TypeVar("SomeExpressionNamespace", bound="ExpressionNamespace")
 
@@ -424,25 +438,33 @@ class ExpressionUrlNamespace(ExpressionNamespace):
         self,
         max_connections: int = 32,
         on_error: Literal["raise"] | Literal["null"] = "raise",
-        fs: fsspec.AbstractFileSystem | None = None,
         io_config: IOConfig | None = None,
-        use_native_downloader: bool = False,
+        use_native_downloader: bool = True,
     ) -> Expression:
         """Treats each string as a URL, and downloads the bytes contents as a bytes column
 
+        ..NOTE::
+            If you are observing excessive S3 issues (such as timeouts, DNS errors or slowdown errors) during URL downloads,
+            you may wish to reduce the value of ``max_connections`` (defaults to 32) to reduce the amount of load you are placing
+            on your S3 servers.
+
+            Alternatively, if you are running on machines with lower number of cores but very high network bandwidth, you can increase
+            ``max_connections`` to get higher throughput with additional parallelism
+
         Args:
-            max_connections: The maximum number of connections to use per thread to use for downloading URLs, defaults to 32
+            max_connections: The maximum number of connections to use per thread to use for downloading URLs. Defaults to 32.
             on_error: Behavior when a URL download error is encountered - "raise" to raise the error immediately or "null" to log
                 the error but fallback to a Null value. Defaults to "raise".
-            fs (fsspec.AbstractFileSystem): fsspec FileSystem to use for downloading data.
-                By default, Daft will automatically construct a FileSystem instance internally.
+            io_config: IOConfig to use when accessing remote storage. Note that the S3Config's `max_connections` parameter will be overridden
+                with `max_connections` that is passed in as a kwarg.
             use_native_downloader (bool): Use the native downloader rather than python based one.
-                Defaults to False.
+                Defaults to True.
 
         Returns:
             Expression: a Binary expression which is the bytes contents of the URL, or None if an error occured during download
         """
-        if fs is None and use_native_downloader:
+        if use_native_downloader:
+
             raise_on_error = False
             if on_error == "raise":
                 raise_on_error = True
@@ -450,8 +472,17 @@ class ExpressionUrlNamespace(ExpressionNamespace):
                 raise_on_error = False
             else:
                 raise NotImplemented(f"Unimplemented on_error option: {on_error}.")
+
             if not (isinstance(max_connections, int) and max_connections > 0):
                 raise ValueError(f"Invalid value for `max_connections`: {max_connections}")
+
+            # Use the `max_connections` kwarg to override the value in S3Config
+            # This is because the max parallelism is actually `min(S3Config's max_connections, url_download's max_connections)` under the hood.
+            # However, default max_connections on S3Config is only 8, and even if we specify 32 here we are bottlenecked there.
+            # Therefore for S3 downloads, we override `max_connections` kwarg to have the intended effect.
+            io_config = context.get_context().daft_planning_config.default_io_config if io_config is None else io_config
+            io_config = io_config.replace(s3=io_config.s3.replace(max_connections=max_connections))
+
             using_ray_runner = context.get_context().is_ray_runner
             return Expression._from_pyexpr(
                 self._expr.url_download(max_connections, raise_on_error, not using_ray_runner, io_config)
@@ -463,7 +494,6 @@ class ExpressionUrlNamespace(ExpressionNamespace):
                 Expression._from_pyexpr(self._expr),
                 max_worker_threads=max_connections,
                 on_error=on_error,
-                fs=fs,
             )
 
 
@@ -507,6 +537,17 @@ class ExpressionDatetimeNamespace(ExpressionNamespace):
         """
         return Expression._from_pyexpr(self._expr.dt_day())
 
+    def hour(self) -> Expression:
+        """Retrieves the day for a datetime column
+
+        Example:
+            >>> col("x").dt.day()
+
+        Returns:
+            Expression: a UInt32 expression with just the day extracted from a datetime column
+        """
+        return Expression._from_pyexpr(self._expr.dt_hour())
+
     def month(self) -> Expression:
         """Retrieves the month for a datetime column
 
@@ -572,7 +613,7 @@ class ExpressionStringNamespace(ExpressionNamespace):
         suffix_expr = Expression._to_expression(suffix)
         return Expression._from_pyexpr(self._expr.utf8_endswith(suffix_expr._expr))
 
-    def startswith(self, prefix: str) -> Expression:
+    def startswith(self, prefix: str | Expression) -> Expression:
         """Checks whether each string starts with the given pattern in a string column
 
         Example:
@@ -587,6 +628,22 @@ class ExpressionStringNamespace(ExpressionNamespace):
         prefix_expr = Expression._to_expression(prefix)
         return Expression._from_pyexpr(self._expr.utf8_startswith(prefix_expr._expr))
 
+    def split(self, pattern: str | Expression) -> Expression:
+        """Splits each string on the given pattern, into one or more strings.
+
+        Example:
+            >>> col("x").str.split(",")
+            >>> col("x").str.split(col("pattern"))
+
+        Args:
+            pattern: The pattern on which each string should be split, or a column to pick such patterns from.
+
+        Returns:
+            Expression: A List[Utf8] expression containing the string splits for each string in the column.
+        """
+        pattern_expr = Expression._to_expression(pattern)
+        return Expression._from_pyexpr(self._expr.utf8_split(pattern_expr._expr))
+
     def concat(self, other: str) -> Expression:
         """Concatenates two string expressions together
 
@@ -632,6 +689,33 @@ class ExpressionListNamespace(ExpressionNamespace):
         """
         return Expression._from_pyexpr(self._expr.list_lengths())
 
+    def get(self, idx: int | Expression, default: object = None) -> Expression:
+        """Gets the element at an index in each list
+
+        Args:
+            idx: index or indices to retrieve from each list
+            default: the default value if the specified index is out of bounds
+
+        Returns:
+            Expression: an expression with the type of the list values
+        """
+        idx_expr = Expression._to_expression(idx)
+        default_expr = lit(default)
+        return Expression._from_pyexpr(self._expr.list_get(idx_expr._expr, default_expr._expr))
+
+
+class ExpressionStructNamespace(ExpressionNamespace):
+    def get(self, name: str) -> Expression:
+        """Retrieves one field from a struct column
+
+        Args:
+            name: the name of the field to retrieve
+
+        Returns:
+            Expression: the field expression
+        """
+        return Expression._from_pyexpr(self._expr.struct_get(name))
+
 
 class ExpressionsProjection(Iterable[Expression]):
     """A collection of Expressions that can be projected onto a Table to produce another Table
@@ -741,6 +825,9 @@ class ExpressionsProjection(Iterable[Expression]):
         fields = [e._to_field(schema) for e in self]
         return Schema._from_field_name_and_types([(f.name, f.dtype) for f in fields])
 
+    def __repr__(self) -> str:
+        return f"{self._output_name_to_exprs.values()}"
+
 
 class ExpressionImageNamespace(ExpressionNamespace):
     """Expression operations for image columns."""
