@@ -10,6 +10,7 @@ from daft.daft import (
     set_io_pool_num_threads,
 )
 from daft.io._csv import read_csv
+from daft.io._iceberg import read_iceberg
 from daft.io._json import read_json
 from daft.io._parquet import read_parquet
 from daft.io.file_path import from_glob_path
@@ -34,6 +35,7 @@ __all__ = [
     "read_json",
     "from_glob_path",
     "read_parquet",
+    "read_iceberg",
     "IOConfig",
     "S3Config",
     "AzureConfig",
