@@ -3,8 +3,11 @@ from __future__ import annotations
 import sys
 from typing import TYPE_CHECKING, Iterator
 
+from daft.daft import CsvParseOptions, JsonParseOptions
 from daft.daft import PyField as _PyField
 from daft.daft import PySchema as _PySchema
+from daft.daft import read_csv_schema as _read_csv_schema
+from daft.daft import read_json_schema as _read_json_schema
 from daft.daft import read_parquet_schema as _read_parquet_schema
 from daft.datatype import DataType, TimeUnit
 
@@ -32,6 +35,11 @@ class Field:
         f._field = field
         return f
 
+    @staticmethod
+    def create(name: str, dtype: DataType) -> Field:
+        pyfield = _PyField.create(name, dtype._dtype)
+        return Field._from_pyfield(pyfield)
+
     @property
     def name(self):
         return self._field.name()
@@ -134,6 +142,12 @@ class Schema:
 
         return Schema._from_pyschema(self._schema.union(other._schema))
 
+    def apply_hints(self, other: Schema) -> Schema:
+        if not isinstance(other, Schema):
+            raise ValueError(f"Expected Schema, got other: {type(other)}")
+
+        return Schema._from_pyschema(self._schema.apply_hints(other._schema))
+
     def __reduce__(self) -> tuple:
         return Schema._from_pyschema, (self._schema,)
 
@@ -153,3 +167,37 @@ class Schema:
                 coerce_int96_timestamp_unit=coerce_int96_timestamp_unit._timeunit,
             )
         )
+
+    @classmethod
+    def from_csv(
+        cls,
+        path: str,
+        parse_options: CsvParseOptions | None = None,
+        io_config: IOConfig | None = None,
+        multithreaded_io: bool | None = None,
+    ) -> Schema:
+        return Schema._from_pyschema(
+            _read_csv_schema(
+                uri=path,
+                parse_options=parse_options,
+                io_config=io_config,
+                multithreaded_io=multithreaded_io,
+            )
+        )
+
+    @classmethod
+    def from_json(
+        cls,
+        path: str,
+        parse_options: JsonParseOptions | None = None,
+        io_config: IOConfig | None = None,
+        multithreaded_io: bool | None = None,
+    ) -> Schema:
+        return Schema._from_pyschema(
+            _read_json_schema(
+                uri=path,
+                parse_options=parse_options,
+                io_config=io_config,
+                multithreaded_io=multithreaded_io,
+            )
+        )
