@@ -1,17 +1,19 @@
 from __future__ import annotations
 
+import logging
 import os
 from contextlib import contextmanager
 from typing import TYPE_CHECKING
 
-from loguru import logger
-
 if TYPE_CHECKING:
     from viztracer import VizTracer
 
 ACTIVE = False
 
 
+logger = logging.getLogger(__name__)
+
+
 @contextmanager
 def profiler(filename: str) -> VizTracer:
     if int(os.environ.get("DAFT_PROFILING", 0)) == 1:
@@ -47,7 +49,7 @@ def timingcontext(name: str):
     tracer = get_tracer()
 
     try:
-        logger.debug(f"log_event:enter:{name}")
+        logger.debug("log_event:enter:%s", name)
         start = time.time()
         if tracer is not None:
             with tracer.log_event(name) as event:
