@@ -1,59 +1,61 @@
 from __future__ import annotations
 
+import logging
 import multiprocessing
 from concurrent import futures
 from dataclasses import dataclass
-from typing import TYPE_CHECKING, Iterable, Iterator
+from typing import Iterable, Iterator
 
 import psutil
-from loguru import logger
 
+from daft.context import get_context
 from daft.daft import (
     FileFormatConfig,
     FileInfos,
-    PythonStorageConfig,
+    IOConfig,
     ResourceRequest,
     StorageConfig,
 )
 from daft.execution import physical_plan
-from daft.execution.execution_step import Instruction, MaterializedResult, PartitionTask
-from daft.filesystem import get_filesystem_from_path, glob_path_with_stats
+from daft.execution.execution_step import Instruction, PartitionTask
+from daft.filesystem import glob_path_with_stats
 from daft.internal.gpu import cuda_device_count
 from daft.logical.builder import LogicalPlanBuilder
 from daft.logical.schema import Schema
 from daft.runners import runner_io
 from daft.runners.partitioning import (
+    MaterializedResult,
     PartID,
     PartitionCacheEntry,
     PartitionMetadata,
     PartitionSet,
 )
 from daft.runners.profiler import profiler
+from daft.runners.progress_bar import ProgressBar
 from daft.runners.runner import Runner
-from daft.table import Table
+from daft.table import MicroPartition
 
-if TYPE_CHECKING:
-    import fsspec
+logger = logging.getLogger(__name__)
 
 
 @dataclass
-class LocalPartitionSet(PartitionSet[Table]):
-    _partitions: dict[PartID, Table]
+class LocalPartitionSet(PartitionSet[MicroPartition]):
+    _partitions: dict[PartID, MicroPartition]
 
-    def items(self) -> list[tuple[PartID, Table]]:
+    def items(self) -> list[tuple[PartID, MicroPartition]]:
         return sorted(self._partitions.items())
 
-    def _get_merged_vpartition(self) -> Table:
+    def _get_merged_vpartition(self) -> MicroPartition:
         ids_and_partitions = self.items()
         assert ids_and_partitions[0][0] == 0
         assert ids_and_partitions[-1][0] + 1 == len(ids_and_partitions)
-        return Table.concat([part for id, part in ids_and_partitions])
+        return MicroPartition.concat([part for id, part in ids_and_partitions])
 
-    def get_partition(self, idx: PartID) -> Table:
+    def get_partition(self, idx: PartID) -> MicroPartition:
         return self._partitions[idx]
 
-    def set_partition(self, idx: PartID, part: Table) -> None:
-        self._partitions[idx] = part
+    def set_partition(self, idx: PartID, part: MaterializedResult[MicroPartition]) -> None:
+        self._partitions[idx] = part.partition()
 
     def delete_partition(self, idx: PartID) -> None:
         del self._partitions[idx]
@@ -62,11 +64,15 @@ class LocalPartitionSet(PartitionSet[Table]):
         return idx in self._partitions
 
     def __len__(self) -> int:
-        return sum(self.len_of_partitions())
+        return sum(len(partition) for partition in self._partitions.values())
 
-    def len_of_partitions(self) -> list[int]:
-        partition_ids = sorted(list(self._partitions.keys()))
-        return [len(self._partitions[pid]) for pid in partition_ids]
+    def size_bytes(self) -> int | None:
+        size_bytes_ = [partition.size_bytes() for partition in self._partitions.values()]
+        size_bytes: list[int] = [size for size in size_bytes_ if size is not None]
+        if len(size_bytes) != len(size_bytes_):
+            return None
+        else:
+            return sum(size_bytes)
 
     def num_partitions(self) -> int:
         return len(self._partitions)
@@ -80,20 +86,12 @@ class PyRunnerIO(runner_io.RunnerIO):
         self,
         source_paths: list[str],
         file_format_config: FileFormatConfig | None = None,
-        fs: fsspec.AbstractFileSystem | None = None,
-        storage_config: StorageConfig | None = None,
+        io_config: IOConfig | None = None,
     ) -> FileInfos:
-        if fs is None and storage_config is not None:
-            config = storage_config.config
-            if isinstance(config, PythonStorageConfig):
-                fs = config.fs
         file_infos = FileInfos()
         file_format = file_format_config.file_format() if file_format_config is not None else None
         for source_path in source_paths:
-            if fs is None:
-                fs = get_filesystem_from_path(source_path)
-
-            path_file_infos = glob_path_with_stats(source_path, file_format, fs, storage_config)
+            path_file_infos = glob_path_with_stats(source_path, file_format, io_config)
 
             if len(path_file_infos) == 0:
                 raise FileNotFoundError(f"No files found at {source_path}")
@@ -114,7 +112,7 @@ class PyRunnerIO(runner_io.RunnerIO):
         return runner_io.sample_schema(file_infos[0].file_path, file_format_config, storage_config)
 
 
-class PyRunner(Runner[Table]):
+class PyRunner(Runner[MicroPartition]):
     def __init__(self, use_thread_pool: bool | None) -> None:
         super().__init__()
         self._use_thread_pool: bool = use_thread_pool if use_thread_pool is not None else True
@@ -127,21 +125,29 @@ class PyRunner(Runner[Table]):
         return PyRunnerIO()
 
     def run(self, builder: LogicalPlanBuilder) -> PartitionCacheEntry:
-        partitions = list(self.run_iter(builder))
+        results = list(self.run_iter(builder))
 
         result_pset = LocalPartitionSet({})
-        for i, partition in enumerate(partitions):
-            result_pset.set_partition(i, partition)
+        for i, result in enumerate(results):
+            result_pset.set_partition(i, result)
 
         pset_entry = self.put_partition_set_into_cache(result_pset)
         return pset_entry
 
-    def run_iter(self, builder: LogicalPlanBuilder) -> Iterator[Table]:
+    def run_iter(
+        self,
+        builder: LogicalPlanBuilder,
+        # NOTE: PyRunner does not run any async execution, so it ignores `results_buffer_size` which is essentially 0
+        results_buffer_size: int | None = None,
+    ) -> Iterator[PyMaterializedResult]:
+        # NOTE: Freeze and use this same execution config for the entire execution
+        daft_execution_config = get_context().daft_execution_config
+
         # Optimize the logical plan.
         builder = builder.optimize()
         # Finalize the logical plan and get a physical plan scheduler for translating the
         # physical plan to executable tasks.
-        plan_scheduler = builder.to_physical_plan_scheduler()
+        plan_scheduler = builder.to_physical_plan_scheduler(daft_execution_config)
         psets = {
             key: entry.value.values()
             for key, entry in self._part_set_cache._uuid_to_partition_set.items()
@@ -149,19 +155,24 @@ class PyRunner(Runner[Table]):
         }
         # Get executable tasks from planner.
         tasks = plan_scheduler.to_partition_tasks(psets, is_ray_runner=False)
-
         with profiler("profile_PyRunner.run_{datetime.now().isoformat()}.json"):
-            partitions_gen = self._physical_plan_to_partitions(tasks)
-            yield from partitions_gen
+            results_gen = self._physical_plan_to_partitions(tasks)
+            yield from results_gen
 
-    def run_iter_tables(self, builder: LogicalPlanBuilder) -> Iterator[Table]:
-        return self.run_iter(builder)
+    def run_iter_tables(
+        self, builder: LogicalPlanBuilder, results_buffer_size: int | None = None
+    ) -> Iterator[MicroPartition]:
+        for result in self.run_iter(builder, results_buffer_size=results_buffer_size):
+            yield result.partition()
 
-    def _physical_plan_to_partitions(self, plan: physical_plan.MaterializedPhysicalPlan) -> Iterator[Table]:
+    def _physical_plan_to_partitions(
+        self, plan: physical_plan.MaterializedPhysicalPlan[MicroPartition]
+    ) -> Iterator[PyMaterializedResult]:
         inflight_tasks: dict[str, PartitionTask] = dict()
         inflight_tasks_resources: dict[str, ResourceRequest] = dict()
         future_to_task: dict[futures.Future, str] = dict()
 
+        pbar = ProgressBar(use_ray_tqdm=False)
         with futures.ThreadPoolExecutor() as thread_pool:
             try:
                 next_step = next(plan)
@@ -174,7 +185,9 @@ class PyRunner(Runner[Table]):
                             # Blocked on already dispatched tasks; await some tasks.
                             break
 
-                        elif isinstance(next_step, Table):
+                        elif isinstance(next_step, MaterializedResult):
+                            assert isinstance(next_step, PyMaterializedResult)
+
                             # A final result.
                             yield next_step
                             next_step = next(plan)
@@ -200,20 +213,23 @@ class PyRunner(Runner[Table]):
                                     and next_step.resource_request.num_gpus > 0
                                 )
                             ):
-                                logger.debug(
-                                    "Running task synchronously in main thread: {next_step}", next_step=next_step
-                                )
+                                logger.debug("Running task synchronously in main thread: %s", next_step)
                                 partitions = self.build_partitions(next_step.instructions, *next_step.inputs)
                                 next_step.set_result([PyMaterializedResult(partition) for partition in partitions])
 
                             else:
                                 # Submit the task for execution.
-                                logger.debug("Submitting task for execution: {next_step}", next_step=next_step)
+                                logger.debug("Submitting task for execution: %s", next_step)
+
+                                # update progress bar
+                                pbar.mark_task_start(next_step)
+
                                 future = thread_pool.submit(
                                     self.build_partitions, next_step.instructions, *next_step.inputs
                                 )
                                 # Register the inflight task and resources used.
                                 future_to_task[future] = next_step.id()
+
                                 inflight_tasks[next_step.id()] = next_step
                                 inflight_tasks_resources[next_step.id()] = next_step.resource_request
 
@@ -230,15 +246,16 @@ class PyRunner(Runner[Table]):
                         done_task = inflight_tasks.pop(done_id)
                         partitions = done_future.result()
 
-                        logger.debug(
-                            "Task completed: {done_id} -> {partitions}", done_id=done_id, partitions=partitions
-                        )
+                        pbar.mark_task_done(done_task)
+
+                        logger.debug("Task completed: %s -> <%s partitions>", done_id, len(partitions))
                         done_task.set_result([PyMaterializedResult(partition) for partition in partitions])
 
                     if next_step is None:
                         next_step = next(plan)
 
             except StopIteration:
+                pbar.close()
                 return
 
     def _check_resource_requests(self, resource_request: ResourceRequest) -> None:
@@ -266,7 +283,7 @@ class PyRunner(Runner[Table]):
         return all((cpus_okay, gpus_okay, memory_okay))
 
     @staticmethod
-    def build_partitions(instruction_stack: list[Instruction], *inputs: Table) -> list[Table]:
+    def build_partitions(instruction_stack: list[Instruction], *inputs: MicroPartition) -> list[MicroPartition]:
         partitions = list(inputs)
         for instruction in instruction_stack:
             partitions = instruction.run(partitions)
@@ -275,13 +292,13 @@ class PyRunner(Runner[Table]):
 
 
 @dataclass(frozen=True)
-class PyMaterializedResult(MaterializedResult[Table]):
-    _partition: Table
+class PyMaterializedResult(MaterializedResult[MicroPartition]):
+    _partition: MicroPartition
 
-    def partition(self) -> Table:
+    def partition(self) -> MicroPartition:
         return self._partition
 
-    def vpartition(self) -> Table:
+    def vpartition(self) -> MicroPartition:
         return self._partition
 
     def metadata(self) -> PartitionMetadata:
@@ -290,5 +307,5 @@ class PyMaterializedResult(MaterializedResult[Table]):
     def cancel(self) -> None:
         return None
 
-    def _noop(self, _: Table) -> None:
+    def _noop(self, _: MicroPartition) -> None:
         return None
