@@ -1,18 +1,22 @@
 from __future__ import annotations
 
+import logging
 import threading
+import time
 import uuid
-from collections import defaultdict
 from dataclasses import dataclass
 from datetime import datetime
-from queue import Queue
-from typing import TYPE_CHECKING, Any, Iterable, Iterator
+from queue import Full, Queue
+from typing import TYPE_CHECKING, Any, Generator, Iterable, Iterator
 
 import pyarrow as pa
-from loguru import logger
 
+from daft.context import get_context, set_execution_config
 from daft.logical.builder import LogicalPlanBuilder
-from daft.planner import PhysicalPlanScheduler
+from daft.plan_scheduler import PhysicalPlanScheduler
+from daft.runners.progress_bar import ProgressBar
+
+logger = logging.getLogger(__name__)
 
 try:
     import ray
@@ -25,7 +29,8 @@ except ImportError:
 from daft.daft import (
     FileFormatConfig,
     FileInfos,
-    PythonStorageConfig,
+    IOConfig,
+    PyDaftExecutionConfig,
     ResourceRequest,
     StorageConfig,
 )
@@ -33,15 +38,15 @@ from daft.datatype import DataType
 from daft.execution.execution_step import (
     FanoutInstruction,
     Instruction,
-    MaterializedResult,
     MultiOutputPartitionTask,
     PartitionTask,
     ReduceInstruction,
     SingleOutputPartitionTask,
 )
-from daft.filesystem import get_filesystem_from_path, glob_path_with_stats
+from daft.filesystem import glob_path_with_stats
 from daft.runners import runner_io
 from daft.runners.partitioning import (
+    MaterializedResult,
     PartID,
     PartitionCacheEntry,
     PartitionMetadata,
@@ -50,11 +55,10 @@ from daft.runners.partitioning import (
 from daft.runners.profiler import profiler
 from daft.runners.pyrunner import LocalPartitionSet
 from daft.runners.runner import Runner
-from daft.table import Table
+from daft.table import MicroPartition
 
 if TYPE_CHECKING:
     import dask
-    import fsspec
     import pandas as pd
     from ray.data.block import Block as RayDatasetBlock
     from ray.data.dataset import Dataset as RayDataset
@@ -67,36 +71,28 @@ except ImportError:
 
 from daft.logical.schema import Schema
 
-RAY_VERSION = tuple(int(s) for s in ray.__version__.split("."))
+RAY_VERSION = tuple(int(s) for s in ray.__version__.split(".")[0:3])
 
 
 @ray.remote
 def _glob_path_into_file_infos(
     paths: list[str],
     file_format_config: FileFormatConfig | None,
-    fs: fsspec.AbstractFileSystem | None,
-    storage_config: StorageConfig | None,
-) -> Table:
-    if fs is None and storage_config is not None:
-        config = storage_config.config
-        if isinstance(config, PythonStorageConfig):
-            fs = config.fs
+    io_config: IOConfig | None,
+) -> MicroPartition:
     file_infos = FileInfos()
     file_format = file_format_config.file_format() if file_format_config is not None else None
     for path in paths:
-        if fs is None:
-            fs = get_filesystem_from_path(path)
-
-        path_file_infos = glob_path_with_stats(path, file_format, fs, storage_config)
+        path_file_infos = glob_path_with_stats(path, file_format=file_format, io_config=io_config)
         if len(path_file_infos) == 0:
             raise FileNotFoundError(f"No files found at {path}")
         file_infos.extend(path_file_infos)
 
-    return Table._from_pytable(file_infos.to_table())
+    return MicroPartition._from_pytable(file_infos.to_table())
 
 
 @ray.remote
-def _make_ray_block_from_vpartition(partition: Table) -> RayDatasetBlock:
+def _make_ray_block_from_vpartition(partition: MicroPartition) -> RayDatasetBlock:
     try:
         return partition.to_arrow(cast_tensors_to_ray_tensor_dtype=True)
     except pa.ArrowInvalid:
@@ -104,13 +100,17 @@ def _make_ray_block_from_vpartition(partition: Table) -> RayDatasetBlock:
 
 
 @ray.remote
-def _make_daft_partition_from_ray_dataset_blocks(ray_dataset_block: pa.Table, daft_schema: Schema) -> Table:
-    return Table.from_arrow(ray_dataset_block)
+def _make_daft_partition_from_ray_dataset_blocks(
+    ray_dataset_block: pa.MicroPartition, daft_schema: Schema
+) -> MicroPartition:
+    return MicroPartition.from_arrow(ray_dataset_block)
 
 
 @ray.remote(num_returns=2)
-def _make_daft_partition_from_dask_dataframe_partitions(dask_df_partition: pd.DataFrame) -> tuple[Table, pa.Schema]:
-    vpart = Table.from_pandas(dask_df_partition)
+def _make_daft_partition_from_dask_dataframe_partitions(
+    dask_df_partition: pd.DataFrame,
+) -> tuple[MicroPartition, pa.Schema]:
+    vpart = MicroPartition.from_pandas(dask_df_partition)
     return vpart, vpart.schema()
 
 
@@ -126,35 +126,30 @@ def _to_pandas_ref(df: pd.DataFrame | ray.ObjectRef[pd.DataFrame]) -> ray.Object
         raise ValueError("Expected a Ray object ref or a Pandas DataFrame, " f"got {type(df)}")
 
 
-@ray.remote
-def remote_len_partition(p: Table) -> int:
-    return len(p)
-
-
 @ray.remote
 def sample_schema_from_filepath(
     first_file_path: str,
     file_format_config: FileFormatConfig,
     storage_config: StorageConfig,
 ) -> Schema:
-    """Ray remote function to run schema sampling on top of a Table containing a single filepath"""
+    """Ray remote function to run schema sampling on top of a MicroPartition containing a single filepath"""
     # Currently just samples the Schema from the first file
     return runner_io.sample_schema(first_file_path, file_format_config, storage_config)
 
 
 @dataclass
 class RayPartitionSet(PartitionSet[ray.ObjectRef]):
-    _partitions: dict[PartID, ray.ObjectRef]
+    _results: dict[PartID, RayMaterializedResult]
 
     def items(self) -> list[tuple[PartID, ray.ObjectRef]]:
-        return sorted(self._partitions.items())
+        return [(pid, result.partition()) for pid, result in sorted(self._results.items())]
 
-    def _get_merged_vpartition(self) -> Table:
+    def _get_merged_vpartition(self) -> MicroPartition:
         ids_and_partitions = self.items()
         assert ids_and_partitions[0][0] == 0
         assert ids_and_partitions[-1][0] + 1 == len(ids_and_partitions)
         all_partitions = ray.get([part for id, part in ids_and_partitions])
-        return Table.concat(all_partitions)
+        return MicroPartition.concat(all_partitions)
 
     def to_ray_dataset(self) -> RayDataset:
         if not _RAY_FROM_ARROW_REFS_AVAILABLE:
@@ -162,7 +157,7 @@ class RayPartitionSet(PartitionSet[ray.ObjectRef]):
                 "Unable to import `ray.data.from_arrow_refs`. Please ensure that you have a compatible version of Ray >= 1.10 installed."
             )
 
-        blocks = [_make_ray_block_from_vpartition.remote(self._partitions[k]) for k in self._partitions.keys()]
+        blocks = [_make_ray_block_from_vpartition.remote(self._results[k].partition()) for k in self._results.keys()]
         # NOTE: although the Ray method is called `from_arrow_refs`, this method works also when the blocks are List[T] types
         # instead of Arrow tables as the codepath for Dataset creation is the same.
         return from_arrow_refs(blocks)
@@ -178,57 +173,61 @@ class RayPartitionSet(PartitionSet[ray.ObjectRef]):
         dask.config.set(scheduler=ray_dask_get)
 
         @dask.delayed
-        def _make_dask_dataframe_partition_from_vpartition(partition: Table) -> pd.DataFrame:
+        def _make_dask_dataframe_partition_from_vpartition(partition: MicroPartition) -> pd.DataFrame:
             return partition.to_pandas()
 
         ddf_parts = [
-            _make_dask_dataframe_partition_from_vpartition(self._partitions[k]) for k in self._partitions.keys()
+            _make_dask_dataframe_partition_from_vpartition(self._results[k].partition()) for k in self._results.keys()
         ]
         return dd.from_delayed(ddf_parts, meta=meta)
 
     def get_partition(self, idx: PartID) -> ray.ObjectRef:
-        return self._partitions[idx]
+        return self._results[idx].partition()
 
-    def set_partition(self, idx: PartID, part: ray.ObjectRef) -> None:
-        self._partitions[idx] = part
+    def set_partition(self, idx: PartID, result: MaterializedResult[ray.ObjectRef]) -> None:
+        assert isinstance(result, RayMaterializedResult)
+        self._results[idx] = result
 
     def delete_partition(self, idx: PartID) -> None:
-        del self._partitions[idx]
+        del self._results[idx]
 
     def has_partition(self, idx: PartID) -> bool:
-        return idx in self._partitions
+        return idx in self._results
 
     def __len__(self) -> int:
-        return sum(self.len_of_partitions())
+        return sum(result.metadata().num_rows for result in self._results.values())
 
-    def len_of_partitions(self) -> list[int]:
-        partition_ids = sorted(list(self._partitions.keys()))
-
-        result: list[int] = ray.get([remote_len_partition.remote(self._partitions[pid]) for pid in partition_ids])
-        return result
+    def size_bytes(self) -> int | None:
+        size_bytes_ = [result.metadata().size_bytes for result in self._results.values()]
+        size_bytes: list[int] = [size for size in size_bytes_ if size is not None]
+        if len(size_bytes) != len(size_bytes_):
+            return None
+        else:
+            return sum(size_bytes)
 
     def num_partitions(self) -> int:
-        return len(self._partitions)
+        return len(self._results)
 
     def wait(self) -> None:
-        ray.wait([o for o in self._partitions.values()])
+        deduped_object_refs = {r.partition() for r in self._results.values()}
+        ray.wait(list(deduped_object_refs))
 
 
 class RayRunnerIO(runner_io.RunnerIO):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
     def glob_paths_details(
         self,
         source_paths: list[str],
         file_format_config: FileFormatConfig | None = None,
-        fs: fsspec.AbstractFileSystem | None = None,
-        storage_config: StorageConfig | None = None,
+        io_config: IOConfig | None = None,
     ) -> FileInfos:
         # Synchronously fetch the file infos, for now.
         return FileInfos.from_table(
-            ray.get(
-                _glob_path_into_file_infos.remote(
-                    source_paths, file_format_config, fs=fs, storage_config=storage_config
-                )
-            )._table
+            ray.get(_glob_path_into_file_infos.remote(source_paths, file_format_config, io_config=io_config))
+            .to_table()
+            ._table
         )
 
     def get_schema_from_first_filepath(
@@ -282,7 +281,11 @@ class RayRunnerIO(runner_io.RunnerIO):
         daft_vpartitions = [
             _make_daft_partition_from_ray_dataset_blocks.remote(block, daft_schema) for block in block_refs
         ]
-        return RayPartitionSet(dict(enumerate(daft_vpartitions))), daft_schema
+
+        return (
+            RayPartitionSet(_results={i: RayMaterializedResult(obj) for i, obj in enumerate(daft_vpartitions)}),
+            daft_schema,
+        )
 
     def partition_set_from_dask_dataframe(
         self,
@@ -296,7 +299,7 @@ class RayRunnerIO(runner_io.RunnerIO):
             raise ValueError("Can't convert an empty Dask DataFrame (with no partitions) to a Daft DataFrame.")
         persisted_partitions = dask.persist(*partitions, scheduler=ray_dask_get)
         parts = [_to_pandas_ref(next(iter(part.dask.values()))) for part in persisted_partitions]
-        daft_vpartitions, schemas = zip(*map(_make_daft_partition_from_dask_dataframe_partitions.remote, parts))
+        daft_vpartitions, schemas = zip(*(_make_daft_partition_from_dask_dataframe_partitions.remote(p) for p in parts))
         schemas = ray.get(list(schemas))
         # Dask shouldn't allow inconsistent schemas across partitions, but we double-check here.
         if not all(schemas[0] == schema for schema in schemas[1:]):
@@ -304,7 +307,10 @@ class RayRunnerIO(runner_io.RunnerIO):
                 "Can't convert a Dask DataFrame with inconsistent schemas across partitions to a Daft DataFrame:",
                 schemas,
             )
-        return RayPartitionSet(dict(enumerate(daft_vpartitions))), schemas[0]
+        return (
+            RayPartitionSet(_results={i: RayMaterializedResult(obj) for i, obj in enumerate(daft_vpartitions)}),
+            schemas[0],
+        )
 
 
 def _get_ray_task_options(resource_request: ResourceRequest) -> dict[str, Any]:
@@ -324,7 +330,9 @@ def _get_ray_task_options(resource_request: ResourceRequest) -> dict[str, Any]:
     return options
 
 
-def build_partitions(instruction_stack: list[Instruction], *inputs: Table) -> list[list[PartitionMetadata] | Table]:
+def build_partitions(
+    instruction_stack: list[Instruction], *inputs: MicroPartition
+) -> list[list[PartitionMetadata] | MicroPartition]:
     partitions = list(inputs)
     for instruction in instruction_stack:
         partitions = instruction.run(partitions)
@@ -339,37 +347,69 @@ def build_partitions(instruction_stack: list[Instruction], *inputs: Table) -> li
 
 @ray.remote
 def single_partition_pipeline(
-    instruction_stack: list[Instruction], *inputs: Table
-) -> list[list[PartitionMetadata] | Table]:
+    daft_execution_config: PyDaftExecutionConfig, instruction_stack: list[Instruction], *inputs: MicroPartition
+) -> list[list[PartitionMetadata] | MicroPartition]:
+    set_execution_config(daft_execution_config)
     return build_partitions(instruction_stack, *inputs)
 
 
 @ray.remote
-def fanout_pipeline(instruction_stack: list[Instruction], *inputs: Table) -> list[list[PartitionMetadata] | Table]:
+def fanout_pipeline(
+    daft_execution_config: PyDaftExecutionConfig, instruction_stack: list[Instruction], *inputs: MicroPartition
+) -> list[list[PartitionMetadata] | MicroPartition]:
+    set_execution_config(daft_execution_config)
     return build_partitions(instruction_stack, *inputs)
 
 
 @ray.remote(scheduling_strategy="SPREAD")
-def reduce_pipeline(instruction_stack: list[Instruction], inputs: list) -> list[list[PartitionMetadata] | Table]:
+def reduce_pipeline(
+    daft_execution_config: PyDaftExecutionConfig, instruction_stack: list[Instruction], inputs: list
+) -> list[list[PartitionMetadata] | MicroPartition]:
     import ray
 
+    set_execution_config(daft_execution_config)
     return build_partitions(instruction_stack, *ray.get(inputs))
 
 
 @ray.remote(scheduling_strategy="SPREAD")
-def reduce_and_fanout(instruction_stack: list[Instruction], inputs: list) -> list[list[PartitionMetadata] | Table]:
+def reduce_and_fanout(
+    daft_execution_config: PyDaftExecutionConfig, instruction_stack: list[Instruction], inputs: list
+) -> list[list[PartitionMetadata] | MicroPartition]:
     import ray
 
+    set_execution_config(daft_execution_config)
     return build_partitions(instruction_stack, *ray.get(inputs))
 
 
 @ray.remote
-def get_meta(partition: Table) -> PartitionMetadata:
-    return PartitionMetadata.from_table(partition)
+def get_metas(*partitions: MicroPartition) -> list[PartitionMetadata]:
+    return [PartitionMetadata.from_table(partition) for partition in partitions]
+
+
+def _ray_num_cpus_provider(ttl_seconds: int = 1) -> Generator[int, None, None]:
+    """Helper that gets the number of CPUs from Ray
+
+    Used as a generator as it provides a guard against calling ray.cluster_resources()
+    more than once per `ttl_seconds`.
+
+    Example:
+    >>> p = _ray_num_cpus_provider()
+    >>> next(p)
+    """
+    last_checked_time = time.time()
+    last_num_cpus_queried = int(ray.cluster_resources().get("CPU", 0))
+    while True:
+        currtime = time.time()
+        if currtime - last_checked_time < ttl_seconds:
+            yield last_num_cpus_queried
+        else:
+            last_checked_time = currtime
+            last_num_cpus_queried = int(ray.cluster_resources().get("CPU", 0))
+            yield last_num_cpus_queried
 
 
 class Scheduler:
-    def __init__(self, max_task_backlog: int | None) -> None:
+    def __init__(self, max_task_backlog: int | None, use_ray_tqdm: bool) -> None:
         """
         max_task_backlog: Max number of inflight tasks waiting for cores.
         """
@@ -386,31 +426,40 @@ class Scheduler:
 
         self.reserved_cores = 0
 
+        self.execution_configs_objref_by_df: dict[str, ray.ObjectRef] = dict()
         self.threads_by_df: dict[str, threading.Thread] = dict()
-        self.results_by_df: dict[str, Queue] = defaultdict(Queue)
+        self.results_by_df: dict[str, Queue] = {}
+        self.active_by_df: dict[str, bool] = dict()
 
-    def next(self, result_uuid: str) -> ray.ObjectRef | StopIteration:
+        self.use_ray_tqdm = use_ray_tqdm
+
+    def next(self, result_uuid: str) -> RayMaterializedResult | StopIteration:
         # Case: thread is terminated and no longer exists.
         # Should only be hit for repeated calls to next() after StopIteration.
         if result_uuid not in self.threads_by_df:
             return StopIteration()
 
+        # Case: thread needs to be terminated
+        if not self.active_by_df.get(result_uuid, False):
+            return StopIteration()
+
         # Common case: get the next result from the thread.
         result = self.results_by_df[result_uuid].get()
 
-        # If there are no more results, delete the thread.
-        if isinstance(result, StopIteration):
-            self.threads_by_df[result_uuid].join()
-            del self.threads_by_df[result_uuid]
-
         return result
 
-    def run_plan(
+    def start_plan(
         self,
         plan_scheduler: PhysicalPlanScheduler,
         psets: dict[str, ray.ObjectRef],
         result_uuid: str,
+        daft_execution_config: PyDaftExecutionConfig,
+        results_buffer_size: int | None = None,
     ) -> None:
+        self.execution_configs_objref_by_df[result_uuid] = ray.put(daft_execution_config)
+        self.results_by_df[result_uuid] = Queue(maxsize=results_buffer_size or -1)
+        self.active_by_df[result_uuid] = True
+
         t = threading.Thread(
             target=self._run_plan,
             name=result_uuid,
@@ -423,60 +472,81 @@ class Scheduler:
         t.start()
         self.threads_by_df[result_uuid] = t
 
+    def active_plans(self) -> list[str]:
+        return [r_uuid for r_uuid, is_active in self.active_by_df.items() if is_active]
+
+    def stop_plan(self, result_uuid: str) -> None:
+        if result_uuid in self.active_by_df:
+            # Mark df as non-active
+            self.active_by_df[result_uuid] = False
+            # wait till thread gracefully completes
+            self.threads_by_df[result_uuid].join()
+            # remove thread and history of df
+            del self.threads_by_df[result_uuid]
+            del self.active_by_df[result_uuid]
+            del self.results_by_df[result_uuid]
+
     def _run_plan(
         self,
         plan_scheduler: PhysicalPlanScheduler,
         psets: dict[str, ray.ObjectRef],
         result_uuid: str,
     ) -> None:
-        from loguru import logger
-
         # Get executable tasks from plan scheduler.
         tasks = plan_scheduler.to_partition_tasks(psets, is_ray_runner=True)
 
-        # Note: For autoscaling clusters, we will probably want to query cores dynamically.
-        # Keep in mind this call takes about 0.3ms.
-        cores = int(ray.cluster_resources()["CPU"]) - self.reserved_cores
-
-        max_inflight_tasks = cores + self.max_task_backlog
-
+        daft_execution_config = self.execution_configs_objref_by_df[result_uuid]
         inflight_tasks: dict[str, PartitionTask[ray.ObjectRef]] = dict()
         inflight_ref_to_task: dict[ray.ObjectRef, str] = dict()
+        pbar = ProgressBar(use_ray_tqdm=self.use_ray_tqdm)
+        num_cpus_provider = _ray_num_cpus_provider()
 
         start = datetime.now()
         profile_filename = (
             f"profile_RayRunner.run()_"
             f"{datetime.replace(datetime.now(), second=0, microsecond=0).isoformat()[:-3]}.json"
         )
+
+        def is_active():
+            return self.active_by_df.get(result_uuid, False)
+
+        def place_in_queue(item):
+            while is_active():
+                try:
+                    self.results_by_df[result_uuid].put(item, timeout=0.1)
+                    break
+                except Full:
+                    pass
+
         with profiler(profile_filename):
             try:
                 next_step = next(tasks)
 
-                while True:  # Loop: Dispatch -> await.
-                    while True:  # Loop: Dispatch (get tasks -> batch dispatch).
+                while is_active():  # Loop: Dispatch -> await.
+                    while is_active():  # Loop: Dispatch (get tasks -> batch dispatch).
                         tasks_to_dispatch: list[PartitionTask] = []
 
+                        cores: int = max(next(num_cpus_provider) - self.reserved_cores, 0)
+                        max_inflight_tasks = cores + self.max_task_backlog
                         dispatches_allowed = max_inflight_tasks - len(inflight_tasks)
                         dispatches_allowed = min(cores, dispatches_allowed)
 
                         # Loop: Get a batch of tasks.
-                        while len(tasks_to_dispatch) < dispatches_allowed:
+                        while len(tasks_to_dispatch) < dispatches_allowed and is_active():
                             if next_step is None:
                                 # Blocked on already dispatched tasks; await some tasks.
                                 break
 
-                            elif isinstance(next_step, ray.ObjectRef):
+                            elif isinstance(next_step, MaterializedResult):
                                 # A final result.
-                                self.results_by_df[result_uuid].put(next_step)
+                                place_in_queue(next_step)
                                 next_step = next(tasks)
 
                             # next_step is a task.
 
                             # If it is a no-op task, just run it locally immediately.
                             elif len(next_step.instructions) == 0:
-                                logger.debug(
-                                    "Running task synchronously in main thread: {next_step}", next_step=next_step
-                                )
+                                logger.debug("Running task synchronously in main thread: %s", next_step)
                                 assert isinstance(next_step, SingleOutputPartitionTask)
                                 next_step.set_result(
                                     [RayMaterializedResult(partition) for partition in next_step.inputs]
@@ -490,15 +560,23 @@ class Scheduler:
 
                         # Dispatch the batch of tasks.
                         logger.debug(
-                            f"{(datetime.now() - start).total_seconds()}s: RayRunner dispatching {len(tasks_to_dispatch)} tasks:"
+                            "%ss: RayRunner dispatching %s tasks",
+                            (datetime.now() - start).total_seconds(),
+                            len(tasks_to_dispatch),
                         )
+
+                        if not is_active():
+                            break
+
                         for task in tasks_to_dispatch:
-                            results = _build_partitions(task)
-                            logger.debug(f"{task} -> {results}")
+                            results = _build_partitions(daft_execution_config, task)
+                            logger.debug("%s -> %s", task, results)
                             inflight_tasks[task.id()] = task
                             for result in results:
                                 inflight_ref_to_task[result] = task.id()
 
+                            pbar.mark_task_start(task)
+
                         if dispatches_allowed == 0 or next_step is None:
                             break
 
@@ -508,6 +586,9 @@ class Scheduler:
                     dispatch = datetime.now()
                     completed_task_ids = []
                     for wait_for in ("next_one", "next_batch"):
+                        if not is_active():
+                            break
+
                         if wait_for == "next_one":
                             num_returns = 1
                             timeout = None
@@ -537,17 +618,26 @@ class Scheduler:
                                     for partition in task.partitions():
                                         del inflight_ref_to_task[partition]
 
+                                pbar.mark_task_done(task)
                                 del inflight_tasks[task_id]
 
                     logger.debug(
-                        f"+{(datetime.now() - dispatch).total_seconds()}s to await results from {completed_task_ids}"
+                        "%ss to await results from %s", (datetime.now() - dispatch).total_seconds(), completed_task_ids
                     )
 
                     if next_step is None:
                         next_step = next(tasks)
 
             except StopIteration as e:
-                self.results_by_df[result_uuid].put(e)
+                place_in_queue(e)
+
+            # Ensure that all Exceptions are correctly propagated to the consumer before reraising to kill thread
+            except Exception as e:
+                place_in_queue(e)
+                pbar.close()
+                raise
+
+        pbar.close()
 
 
 @ray.remote(num_cpus=1)
@@ -557,11 +647,11 @@ class SchedulerActor(Scheduler):
         self.reserved_cores = 1
 
 
-def _build_partitions(task: PartitionTask[ray.ObjectRef]) -> list[ray.ObjectRef]:
+def _build_partitions(
+    daft_execution_config_objref: ray.ObjectRef, task: PartitionTask[ray.ObjectRef]
+) -> list[ray.ObjectRef]:
     """Run a PartitionTask and return the resulting list of partitions."""
-    ray_options: dict[str, Any] = {
-        "num_returns": task.num_results + 1,
-    }
+    ray_options: dict[str, Any] = {"num_returns": task.num_results + 1, "name": task.name()}
 
     if task.resource_request is not None:
         ray_options = {**ray_options, **_get_ray_task_options(task.resource_request)}
@@ -569,17 +659,28 @@ def _build_partitions(task: PartitionTask[ray.ObjectRef]) -> list[ray.ObjectRef]
     if isinstance(task.instructions[0], ReduceInstruction):
         build_remote = reduce_and_fanout if isinstance(task.instructions[-1], FanoutInstruction) else reduce_pipeline
         build_remote = build_remote.options(**ray_options)
-        [metadatas_ref, *partitions] = build_remote.remote(task.instructions, task.inputs)
+        [metadatas_ref, *partitions] = build_remote.remote(daft_execution_config_objref, task.instructions, task.inputs)
 
     else:
         build_remote = (
             fanout_pipeline if isinstance(task.instructions[-1], FanoutInstruction) else single_partition_pipeline
         )
         build_remote = build_remote.options(**ray_options)
-        [metadatas_ref, *partitions] = build_remote.remote(task.instructions, *task.inputs)
+        [metadatas_ref, *partitions] = build_remote.remote(
+            daft_execution_config_objref, task.instructions, *task.inputs
+        )
 
     metadatas_accessor = PartitionMetadataAccessor(metadatas_ref)
-    task.set_result([RayMaterializedResult(partition, metadatas_accessor, i) for i, partition in enumerate(partitions)])
+    task.set_result(
+        [
+            RayMaterializedResult(
+                partition=partition,
+                metadatas=metadatas_accessor,
+                metadata_idx=i,
+            )
+            for i, partition in enumerate(partitions)
+        ]
+    )
 
     return partitions
 
@@ -599,18 +700,32 @@ class RayRunner(Runner[ray.ObjectRef]):
             # Run scheduler remotely if the cluster is connected remotely.
             self.scheduler_actor = SchedulerActor.remote(  # type: ignore
                 max_task_backlog=max_task_backlog,
+                use_ray_tqdm=True,
             )
         else:
             self.scheduler = Scheduler(
                 max_task_backlog=max_task_backlog,
+                use_ray_tqdm=False,
             )
 
-    def run_iter(self, builder: LogicalPlanBuilder) -> Iterator[ray.ObjectRef]:
+    def active_plans(self) -> list[str]:
+        if isinstance(self.ray_context, ray.client_builder.ClientContext):
+            return ray.get(self.scheduler_actor.active_plans.remote())
+        else:
+            return self.scheduler.active_plans()
+
+    def run_iter(
+        self, builder: LogicalPlanBuilder, results_buffer_size: int | None = None
+    ) -> Iterator[RayMaterializedResult]:
+        # Grab and freeze the current DaftExecutionConfig
+        daft_execution_config = get_context().daft_execution_config
+
         # Optimize the logical plan.
         builder = builder.optimize()
+
         # Finalize the logical plan and get a physical plan scheduler for translating the
         # physical plan to executable tasks.
-        plan_scheduler = builder.to_physical_plan_scheduler()
+        plan_scheduler = builder.to_physical_plan_scheduler(daft_execution_config)
 
         psets = {
             key: entry.value.values()
@@ -620,41 +735,56 @@ class RayRunner(Runner[ray.ObjectRef]):
         result_uuid = str(uuid.uuid4())
         if isinstance(self.ray_context, ray.client_builder.ClientContext):
             ray.get(
-                self.scheduler_actor.run_plan.remote(
+                self.scheduler_actor.start_plan.remote(
+                    daft_execution_config=daft_execution_config,
                     plan_scheduler=plan_scheduler,
                     psets=psets,
                     result_uuid=result_uuid,
+                    results_buffer_size=results_buffer_size,
                 )
             )
-
         else:
-            self.scheduler.run_plan(
+            self.scheduler.start_plan(
+                daft_execution_config=daft_execution_config,
                 plan_scheduler=plan_scheduler,
                 psets=psets,
                 result_uuid=result_uuid,
+                results_buffer_size=results_buffer_size,
             )
 
-        while True:
+        try:
+            while True:
+                if isinstance(self.ray_context, ray.client_builder.ClientContext):
+                    result = ray.get(self.scheduler_actor.next.remote(result_uuid))
+                else:
+                    result = self.scheduler.next(result_uuid)
+
+                if isinstance(result, StopIteration):
+                    break
+                elif isinstance(result, Exception):
+                    raise result
+
+                yield result
+        finally:
+            # Generator is out of scope, ensure that state has been cleaned up
             if isinstance(self.ray_context, ray.client_builder.ClientContext):
-                result = ray.get(self.scheduler_actor.next.remote(result_uuid))
+                ray.get(self.scheduler_actor.stop_plan.remote(result_uuid))
             else:
-                result = self.scheduler.next(result_uuid)
+                self.scheduler.stop_plan(result_uuid)
 
-            if isinstance(result, StopIteration):
-                return
-            yield result
-
-    def run_iter_tables(self, builder: LogicalPlanBuilder) -> Iterator[Table]:
-        for ref in self.run_iter(builder):
-            yield ray.get(ref)
+    def run_iter_tables(
+        self, builder: LogicalPlanBuilder, results_buffer_size: int | None = None
+    ) -> Iterator[MicroPartition]:
+        for result in self.run_iter(builder, results_buffer_size=results_buffer_size):
+            yield ray.get(result.partition())
 
     def run(self, builder: LogicalPlanBuilder) -> PartitionCacheEntry:
-        result_pset = RayPartitionSet({})
+        result_pset = RayPartitionSet(_results={})
 
-        partitions_iter = self.run_iter(builder)
+        results_iter = self.run_iter(builder)
 
-        for i, partition in enumerate(partitions_iter):
-            result_pset.set_partition(i, partition)
+        for i, result in enumerate(results_iter):
+            result_pset.set_partition(i, result)
 
         pset_entry = self._part_set_cache.put_partition_set(result_pset)
 
@@ -662,7 +792,14 @@ class RayRunner(Runner[ray.ObjectRef]):
 
     def put_partition_set_into_cache(self, pset: PartitionSet) -> PartitionCacheEntry:
         if isinstance(pset, LocalPartitionSet):
-            pset = RayPartitionSet({pid: ray.put(val) for pid, val in pset._partitions.items()})
+            pset = RayPartitionSet(
+                _results={
+                    pid: RayMaterializedResult(
+                        ray.put(val),
+                    )
+                    for pid, val in pset._partitions.items()
+                },
+            )
 
         return self._part_set_cache.put_partition_set(pset=pset)
 
@@ -670,23 +807,29 @@ class RayRunner(Runner[ray.ObjectRef]):
         return RayRunnerIO()
 
 
-@dataclass(frozen=True)
 class RayMaterializedResult(MaterializedResult[ray.ObjectRef]):
-    _partition: ray.ObjectRef
-    _metadatas: PartitionMetadataAccessor | None = None
-    _metadata_index: int | None = None
+    def __init__(
+        self,
+        partition: ray.ObjectRef,
+        metadatas: PartitionMetadataAccessor | None = None,
+        metadata_idx: int | None = None,
+    ):
+        self._partition = partition
+        if metadatas is None:
+            assert metadata_idx is None
+            metadatas = PartitionMetadataAccessor(get_metas.remote(self._partition))
+            metadata_idx = 0
+        self._metadatas = metadatas
+        self._metadata_idx = metadata_idx
 
     def partition(self) -> ray.ObjectRef:
         return self._partition
 
-    def vpartition(self) -> Table:
+    def vpartition(self) -> MicroPartition:
         return ray.get(self._partition)
 
     def metadata(self) -> PartitionMetadata:
-        if self._metadatas is not None and self._metadata_index is not None:
-            return self._metadatas.get_index(self._metadata_index)
-        else:
-            return ray.get(get_meta.remote(self._partition))
+        return self._metadatas.get_index(self._metadata_idx)
 
     def cancel(self) -> None:
         return ray.cancel(self._partition)
