@@ -320,6 +320,9 @@ class Series:
 
         return Series._from_pyseries(self._series.hash(seed._series if seed is not None else None))
 
+    def murmur3_32(self) -> Series:
+        return Series._from_pyseries(self._series.murmur3_32())
+
     def __repr__(self) -> str:
         return repr(self._series)
 
@@ -467,6 +470,10 @@ class Series:
         assert self._series is not None
         return Series._from_pyseries(self._series.is_null())
 
+    def not_null(self) -> Series:
+        assert self._series is not None
+        return Series._from_pyseries(self._series.not_null())
+
     @property
     def float(self) -> SeriesFloatNamespace:
         return SeriesFloatNamespace.from_series(self)
@@ -487,12 +494,23 @@ class Series:
     def image(self) -> SeriesImageNamespace:
         return SeriesImageNamespace.from_series(self)
 
+    @property
+    def partitioning(self) -> SeriesPartitioningNamespace:
+        return SeriesPartitioningNamespace.from_series(self)
+
     def __reduce__(self) -> tuple:
         if self.datatype()._is_python_type():
             return (Series.from_pylist, (self.to_pylist(), self.name(), "force"))
         else:
             return (Series.from_arrow, (self.to_arrow(), self.name()))
 
+    def _debug_bincode_serialize(self) -> bytes:
+        return self._series._debug_bincode_serialize()
+
+    @classmethod
+    def _debug_bincode_deserialize(cls, b: bytes) -> Series:
+        return Series._from_pyseries(PySeries._debug_bincode_deserialize(b))
+
 
 SomeSeriesNamespace = TypeVar("SomeSeriesNamespace", bound="SeriesNamespace")
 
@@ -534,6 +552,12 @@ class SeriesStringNamespace(SeriesNamespace):
         assert self._series is not None and pattern._series is not None
         return Series._from_pyseries(self._series.utf8_contains(pattern._series))
 
+    def split(self, pattern: Series) -> Series:
+        if not isinstance(pattern, Series):
+            raise ValueError(f"expected another Series but got {type(pattern)}")
+        assert self._series is not None and pattern._series is not None
+        return Series._from_pyseries(self._series.utf8_split(pattern._series))
+
     def concat(self, other: Series) -> Series:
         if not isinstance(other, Series):
             raise ValueError(f"expected another Series but got {type(other)}")
@@ -552,6 +576,9 @@ class SeriesDateNamespace(SeriesNamespace):
     def day(self) -> Series:
         return Series._from_pyseries(self._series.dt_day())
 
+    def hour(self) -> Series:
+        return Series._from_pyseries(self._series.dt_hour())
+
     def month(self) -> Series:
         return Series._from_pyseries(self._series.dt_month())
 
@@ -562,10 +589,33 @@ class SeriesDateNamespace(SeriesNamespace):
         return Series._from_pyseries(self._series.dt_day_of_week())
 
 
+class SeriesPartitioningNamespace(SeriesNamespace):
+    def days(self) -> Series:
+        return Series._from_pyseries(self._series.partitioning_days())
+
+    def hours(self) -> Series:
+        return Series._from_pyseries(self._series.partitioning_hours())
+
+    def months(self) -> Series:
+        return Series._from_pyseries(self._series.partitioning_months())
+
+    def years(self) -> Series:
+        return Series._from_pyseries(self._series.partitioning_years())
+
+    def iceberg_bucket(self, n: int) -> Series:
+        return Series._from_pyseries(self._series.partitioning_iceberg_bucket(n))
+
+    def iceberg_truncate(self, w: int) -> Series:
+        return Series._from_pyseries(self._series.partitioning_iceberg_truncate(w))
+
+
 class SeriesListNamespace(SeriesNamespace):
     def lengths(self) -> Series:
         return Series._from_pyseries(self._series.list_lengths())
 
+    def get(self, idx: Series, default: Series) -> Series:
+        return Series._from_pyseries(self._series.list_get(idx._series, default._series))
+
 
 class SeriesImageNamespace(SeriesNamespace):
     def decode(self) -> Series:
