@@ -1,14 +1,24 @@
 from __future__ import annotations
 
-import sys
+import logging
 from typing import TYPE_CHECKING, Any
 
 import pyarrow as pa
-from loguru import logger
 
 from daft.arrow_utils import ensure_table
-from daft.daft import JoinType
+from daft.daft import (
+    CsvConvertOptions,
+    CsvParseOptions,
+    CsvReadOptions,
+    JoinType,
+    JsonConvertOptions,
+    JsonParseOptions,
+    JsonReadOptions,
+)
 from daft.daft import PyTable as _PyTable
+from daft.daft import ScanTask as _ScanTask
+from daft.daft import read_csv as _read_csv
+from daft.daft import read_json as _read_json
 from daft.daft import read_parquet as _read_parquet
 from daft.daft import read_parquet_bulk as _read_parquet_bulk
 from daft.daft import read_parquet_into_pyarrow as _read_parquet_into_pyarrow
@@ -19,11 +29,6 @@ from daft.expressions import Expression, ExpressionsProjection
 from daft.logical.schema import Schema
 from daft.series import Series
 
-if sys.version_info < (3, 8):
-    pass
-else:
-    pass
-
 _NUMPY_AVAILABLE = True
 try:
     import numpy as np
@@ -44,6 +49,9 @@ if TYPE_CHECKING:
     from daft.io import IOConfig
 
 
+logger = logging.getLogger(__name__)
+
+
 class Table:
     _table: _PyTable
 
@@ -80,6 +88,10 @@ class Table:
         pyt = _PyTable.empty(None) if schema is None else _PyTable.empty(schema._schema)
         return Table._from_pytable(pyt)
 
+    @staticmethod
+    def _from_scan_task(_: _ScanTask) -> Table:
+        raise NotImplementedError("_from_scan_task is not implemented for legacy Python Table.")
+
     @staticmethod
     def _from_pytable(pyt: _PyTable) -> Table:
         assert isinstance(pyt, _PyTable)
@@ -103,7 +115,7 @@ class Table:
         if non_native_fields:
             # If there are any contained Arrow types that are not natively supported, go through Table.from_pydict()
             # path.
-            logger.debug(f"Unsupported Arrow types detected for columns: {non_native_fields}")
+            logger.debug("Unsupported Arrow types detected for columns: %s", non_native_fields)
             return Table.from_pydict(dict(zip(arrow_table.column_names, arrow_table.columns)))
         else:
             # Otherwise, go through record batch happy path.
@@ -111,6 +123,12 @@ class Table:
             pyt = _PyTable.from_arrow_record_batches(arrow_table.to_batches(), schema._schema)
             return Table._from_pytable(pyt)
 
+    @staticmethod
+    def from_arrow_record_batches(rbs: list[pa.RecordBatch], arrow_schema: pa.Schema) -> Table:
+        schema = Schema._from_field_name_and_types([(f.name, DataType.from_arrow_type(f.type)) for f in arrow_schema])
+        pyt = _PyTable.from_arrow_record_batches(rbs, schema._schema)
+        return Table._from_pytable(pyt)
+
     @staticmethod
     def from_pandas(pd_df: pd.DataFrame) -> Table:
         if not _PANDAS_AVAILABLE:
@@ -165,6 +183,10 @@ class Table:
     # Exporting methods
     ###
 
+    def to_table(self) -> Table:
+        """For compatibility with MicroPartition"""
+        return self
+
     def to_arrow(self, cast_tensors_to_ray_tensor_dtype: bool = False, convert_large_arrays: bool = False) -> pa.Table:
         python_fields = set()
         tensor_fields = set()
@@ -274,8 +296,21 @@ class Table:
             raise TypeError(f"Expected a bool, list[bool] or None for `descending` but got {type(descending)}")
         return Table._from_pytable(self._table.sort(pyexprs, descending))
 
-    def sample(self, num: int) -> Table:
-        return Table._from_pytable(self._table.sample(num))
+    def sample(
+        self,
+        fraction: float | None = None,
+        size: int | None = None,
+        with_replacement: bool = False,
+        seed: int | None = None,
+    ) -> Table:
+        if fraction is not None and size is not None:
+            raise ValueError("Must specify either `fraction` or `size`, but not both")
+        elif fraction is not None:
+            return Table._from_pytable(self._table.sample_by_fraction(fraction, with_replacement, seed))
+        elif size is not None:
+            return Table._from_pytable(self._table.sample_by_size(size, with_replacement, seed))
+        else:
+            raise ValueError("Must specify either `fraction` or `size`")
 
     def agg(self, to_agg: list[Expression], group_by: ExpressionsProjection | None = None) -> Table:
         to_agg_pyexprs = [e._expr for e in to_agg]
@@ -370,6 +405,7 @@ class Table:
         start_offset: int | None = None,
         num_rows: int | None = None,
         row_groups: list[int] | None = None,
+        predicate: Expression | None = None,
         io_config: IOConfig | None = None,
         multithreaded_io: bool | None = None,
         coerce_int96_timestamp_unit: TimeUnit = TimeUnit.ns(),
@@ -381,6 +417,7 @@ class Table:
                 start_offset=start_offset,
                 num_rows=num_rows,
                 row_groups=row_groups,
+                predicate=predicate._expr if predicate is not None else None,
                 io_config=io_config,
                 multithreaded_io=multithreaded_io,
                 coerce_int96_timestamp_unit=coerce_int96_timestamp_unit._timeunit,
@@ -394,7 +431,8 @@ class Table:
         columns: list[str] | None = None,
         start_offset: int | None = None,
         num_rows: int | None = None,
-        row_groups_per_path: list[list[int]] | None = None,
+        row_groups_per_path: list[list[int] | None] | None = None,
+        predicate: Expression | None = None,
         io_config: IOConfig | None = None,
         num_parallel_tasks: int | None = 128,
         multithreaded_io: bool | None = None,
@@ -406,6 +444,7 @@ class Table:
             start_offset=start_offset,
             num_rows=num_rows,
             row_groups=row_groups_per_path,
+            predicate=predicate._expr if predicate is not None else None,
             io_config=io_config,
             num_parallel_tasks=num_parallel_tasks,
             multithreaded_io=multithreaded_io,
@@ -421,7 +460,7 @@ class Table:
         multithreaded_io: bool | None = None,
     ) -> Table:
         if not isinstance(paths, Series):
-            paths = Series.from_pylist(paths, name="uris")
+            paths = Series.from_pylist(paths, name="uris").cast(DataType.string())
         assert paths.name() == "uris", f"Expected input series to have name 'uris', but found: {paths.name()}"
         return Table._from_pytable(
             _read_parquet_statistics(
@@ -431,6 +470,50 @@ class Table:
             )
         )
 
+    @classmethod
+    def read_csv(
+        cls,
+        path: str,
+        convert_options: CsvConvertOptions | None = None,
+        parse_options: CsvParseOptions | None = None,
+        read_options: CsvReadOptions | None = None,
+        io_config: IOConfig | None = None,
+        multithreaded_io: bool | None = None,
+    ) -> Table:
+        return Table._from_pytable(
+            _read_csv(
+                uri=path,
+                convert_options=convert_options,
+                parse_options=parse_options,
+                read_options=read_options,
+                io_config=io_config,
+                multithreaded_io=multithreaded_io,
+            )
+        )
+
+    @classmethod
+    def read_json(
+        cls,
+        path: str,
+        convert_options: JsonConvertOptions | None = None,
+        parse_options: JsonParseOptions | None = None,
+        read_options: JsonReadOptions | None = None,
+        io_config: IOConfig | None = None,
+        multithreaded_io: bool | None = None,
+        max_chunks_in_flight: int | None = None,
+    ) -> Table:
+        return Table._from_pytable(
+            _read_json(
+                uri=path,
+                convert_options=convert_options,
+                parse_options=parse_options,
+                read_options=read_options,
+                io_config=io_config,
+                multithreaded_io=multithreaded_io,
+                max_chunks_in_flight=max_chunks_in_flight,
+            )
+        )
+
 
 def _trim_pyarrow_large_arrays(arr: pa.ChunkedArray) -> pa.ChunkedArray:
     if pa.types.is_large_binary(arr.type) or pa.types.is_large_string(arr.type):
@@ -477,7 +560,7 @@ def read_parquet_into_pyarrow(
         coerce_int96_timestamp_unit=coerce_int96_timestamp_unit._timeunit,
     )
     schema = pa.schema(fields, metadata=metadata)
-    columns = [pa.chunked_array(c) for c in columns]  # type: ignore
+    columns = [pa.chunked_array(c, type=f.type) for f, c in zip(schema, columns)]  # type: ignore
     return pa.table(columns, schema=schema)
 
 
@@ -486,7 +569,7 @@ def read_parquet_into_pyarrow_bulk(
     columns: list[str] | None = None,
     start_offset: int | None = None,
     num_rows: int | None = None,
-    row_groups_per_path: list[list[int]] | None = None,
+    row_groups_per_path: list[list[int] | None] | None = None,
     io_config: IOConfig | None = None,
     num_parallel_tasks: int | None = 128,
     multithreaded_io: bool | None = None,
@@ -504,6 +587,6 @@ def read_parquet_into_pyarrow_bulk(
         coerce_int96_timestamp_unit=coerce_int96_timestamp_unit._timeunit,
     )
     return [
-        pa.table([pa.chunked_array(c) for c in columns], schema=pa.schema(fields, metadata=metadata))  # type: ignore
+        pa.table([pa.chunked_array(c, type=f.type) for f, c in zip(fields, columns)], schema=pa.schema(fields, metadata=metadata))  # type: ignore
         for fields, metadata, columns in bulk_result
     ]
