@@ -1,3 +1,5 @@
+use std::hash::Hasher;
+
 use common_error::DaftError;
 use pyo3::prelude::*;
 
@@ -11,13 +13,13 @@ use crate::config;
 ///     endpoint_url: URL to the S3 endpoint, defaults to endpoints to AWS
 ///     key_id: AWS Access Key ID, defaults to auto-detection from the current environment
 ///     access_key: AWS Secret Access Key, defaults to auto-detection from the current environment
-///     max_connections: Maximum number of connections to S3 at any time, defaults to 1024
+///     max_connections: Maximum number of connections to S3 at any time, defaults to 64
 ///     session_token: AWS Session Token, required only if `key_id` and `access_key` are temporary credentials
 ///     retry_initial_backoff_ms: Initial backoff duration in milliseconds for an S3 retry, defaults to 1000ms
-///     connect_timeout_ms: Timeout duration to wait to make a connection to S3 in milliseconds, defaults to 60 seconds
-///     read_timeout_ms: Timeout duration to wait to read the first byte from S3 in milliseconds, defaults to 60 seconds
+///     connect_timeout_ms: Timeout duration to wait to make a connection to S3 in milliseconds, defaults to 10 seconds
+///     read_timeout_ms: Timeout duration to wait to read the first byte from S3 in milliseconds, defaults to 10 seconds
 ///     num_tries: Number of attempts to make a connection, defaults to 5
-///     retry_mode: Retry Mode when a request fails, current supported values are `standard` and `adaptive`
+///     retry_mode: Retry Mode when a request fails, current supported values are `standard` and `adaptive`, defaults to `adaptive`
 ///     anonymous: Whether or not to use "anonymous mode", which will access S3 without any credentials
 ///     verify_ssl: Whether or not to verify ssl certificates, which will access S3 without checking if the certs are valid, defaults to True
 ///     check_hostname_ssl: Whether or not to verify the hostname when verifying ssl certificates, this was the legacy behavior for openssl, defaults to True
@@ -89,6 +91,23 @@ impl IOConfig {
         }
     }
 
+    pub fn replace(
+        &self,
+        s3: Option<S3Config>,
+        azure: Option<AzureConfig>,
+        gcs: Option<GCSConfig>,
+    ) -> Self {
+        IOConfig {
+            config: config::IOConfig {
+                s3: s3.map(|s3| s3.config).unwrap_or(self.config.s3.clone()),
+                azure: azure
+                    .map(|azure| azure.config)
+                    .unwrap_or(self.config.azure.clone()),
+                gcs: gcs.map(|gcs| gcs.config).unwrap_or(self.config.gcs.clone()),
+            },
+        }
+    }
+
     pub fn __repr__(&self) -> PyResult<String> {
         Ok(format!("{}", self.config))
     }
@@ -133,6 +152,15 @@ impl IOConfig {
             (json_string,),
         ))
     }
+
+    pub fn __hash__(&self) -> PyResult<u64> {
+        use std::collections::hash_map::DefaultHasher;
+        use std::hash::Hash;
+
+        let mut hasher = DefaultHasher::new();
+        self.config.hash(&mut hasher);
+        Ok(hasher.finish())
+    }
 }
 
 #[pymethods]
@@ -163,7 +191,8 @@ impl S3Config {
                 key_id: key_id.or(def.key_id),
                 session_token: session_token.or(def.session_token),
                 access_key: access_key.or(def.access_key),
-                max_connections: max_connections.unwrap_or(def.max_connections),
+                max_connections_per_io_thread: max_connections
+                    .unwrap_or(def.max_connections_per_io_thread),
                 retry_initial_backoff_ms: retry_initial_backoff_ms
                     .unwrap_or(def.retry_initial_backoff_ms),
                 connect_timeout_ms: connect_timeout_ms.unwrap_or(def.connect_timeout_ms),
@@ -177,6 +206,46 @@ impl S3Config {
         }
     }
 
+    #[allow(clippy::too_many_arguments)]
+    pub fn replace(
+        &self,
+        region_name: Option<String>,
+        endpoint_url: Option<String>,
+        key_id: Option<String>,
+        session_token: Option<String>,
+        access_key: Option<String>,
+        max_connections: Option<u32>,
+        retry_initial_backoff_ms: Option<u64>,
+        connect_timeout_ms: Option<u64>,
+        read_timeout_ms: Option<u64>,
+        num_tries: Option<u32>,
+        retry_mode: Option<String>,
+        anonymous: Option<bool>,
+        verify_ssl: Option<bool>,
+        check_hostname_ssl: Option<bool>,
+    ) -> Self {
+        S3Config {
+            config: crate::S3Config {
+                region_name: region_name.or_else(|| self.config.region_name.clone()),
+                endpoint_url: endpoint_url.or_else(|| self.config.endpoint_url.clone()),
+                key_id: key_id.or_else(|| self.config.key_id.clone()),
+                session_token: session_token.or_else(|| self.config.session_token.clone()),
+                access_key: access_key.or_else(|| self.config.access_key.clone()),
+                max_connections_per_io_thread: max_connections
+                    .unwrap_or(self.config.max_connections_per_io_thread),
+                retry_initial_backoff_ms: retry_initial_backoff_ms
+                    .unwrap_or(self.config.retry_initial_backoff_ms),
+                connect_timeout_ms: connect_timeout_ms.unwrap_or(self.config.connect_timeout_ms),
+                read_timeout_ms: read_timeout_ms.unwrap_or(self.config.read_timeout_ms),
+                num_tries: num_tries.unwrap_or(self.config.num_tries),
+                retry_mode: retry_mode.or_else(|| self.config.retry_mode.clone()),
+                anonymous: anonymous.unwrap_or(self.config.anonymous),
+                verify_ssl: verify_ssl.unwrap_or(self.config.verify_ssl),
+                check_hostname_ssl: check_hostname_ssl.unwrap_or(self.config.check_hostname_ssl),
+            },
+        }
+    }
+
     pub fn __repr__(&self) -> PyResult<String> {
         Ok(format!("{}", self.config))
     }
@@ -211,10 +280,10 @@ impl S3Config {
         Ok(self.config.access_key.clone())
     }
 
-    /// AWS max connections
+    /// AWS max connections per IO thread
     #[getter]
     pub fn max_connections(&self) -> PyResult<u32> {
-        Ok(self.config.max_connections)
+        Ok(self.config.max_connections_per_io_thread)
     }
 
     /// AWS Retry Initial Backoff Time in Milliseconds
@@ -246,6 +315,24 @@ impl S3Config {
     pub fn retry_mode(&self) -> PyResult<Option<String>> {
         Ok(self.config.retry_mode.clone())
     }
+
+    /// AWS Anonymous Mode
+    #[getter]
+    pub fn anonymous(&self) -> PyResult<Option<bool>> {
+        Ok(Some(self.config.anonymous))
+    }
+
+    /// AWS Verify SSL
+    #[getter]
+    pub fn verify_ssl(&self) -> PyResult<Option<bool>> {
+        Ok(Some(self.config.verify_ssl))
+    }
+
+    /// AWS Check SSL Hostname
+    #[getter]
+    pub fn check_hostname_ssl(&self) -> PyResult<Option<bool>> {
+        Ok(Some(self.config.check_hostname_ssl))
+    }
 }
 
 #[pymethods]
@@ -267,6 +354,21 @@ impl AzureConfig {
         }
     }
 
+    pub fn replace(
+        &self,
+        storage_account: Option<String>,
+        access_key: Option<String>,
+        anonymous: Option<bool>,
+    ) -> Self {
+        AzureConfig {
+            config: crate::AzureConfig {
+                storage_account: storage_account.or_else(|| self.config.storage_account.clone()),
+                access_key: access_key.or_else(|| self.config.access_key.clone()),
+                anonymous: anonymous.unwrap_or(self.config.anonymous),
+            },
+        }
+    }
+
     pub fn __repr__(&self) -> PyResult<String> {
         Ok(format!("{}", self.config))
     }
@@ -298,6 +400,15 @@ impl GCSConfig {
         }
     }
 
+    pub fn replace(&self, project_id: Option<String>, anonymous: Option<bool>) -> Self {
+        GCSConfig {
+            config: crate::GCSConfig {
+                project_id: project_id.or_else(|| self.config.project_id.clone()),
+                anonymous: anonymous.unwrap_or(self.config.anonymous),
+            },
+        }
+    }
+
     pub fn __repr__(&self) -> PyResult<String> {
         Ok(format!("{}", self.config))
     }
@@ -307,6 +418,12 @@ impl GCSConfig {
     pub fn project_id(&self) -> PyResult<Option<String>> {
         Ok(self.config.project_id.clone())
     }
+
+    /// Whether to use anonymous mode
+    #[getter]
+    pub fn anonymous(&self) -> PyResult<bool> {
+        Ok(self.config.anonymous)
+    }
 }
 
 impl From<config::IOConfig> for IOConfig {
