@@ -11,7 +11,7 @@ pub struct S3Config {
     pub key_id: Option<String>,
     pub session_token: Option<String>,
     pub access_key: Option<String>,
-    pub max_connections: u32,
+    pub max_connections_per_io_thread: u32,
     pub retry_initial_backoff_ms: u64,
     pub connect_timeout_ms: u64,
     pub read_timeout_ms: u64,
@@ -30,12 +30,14 @@ impl Default for S3Config {
             key_id: None,
             session_token: None,
             access_key: None,
-            max_connections: 1024,
+            max_connections_per_io_thread: 8,
             retry_initial_backoff_ms: 1000,
-            connect_timeout_ms: 60_000,
-            read_timeout_ms: 60_000,
-            num_tries: 5,
-            retry_mode: Some("standard".to_string()),
+            connect_timeout_ms: 10_000,
+            read_timeout_ms: 10_000,
+            // AWS EMR actually does 100 tries by default for AIMD retries
+            // (See [Advanced AIMD retry settings]: https://docs.aws.amazon.com/emr/latest/ReleaseGuide/emr-spark-emrfs-retry.html)
+            num_tries: 25,
+            retry_mode: Some("adaptive".to_string()),
             anonymous: false,
             verify_ssl: true,
             check_hostname_ssl: true,
@@ -67,8 +69,8 @@ impl Display for S3Config {
             self.key_id,
             self.session_token,
             self.access_key,
+            self.max_connections_per_io_thread,
             self.retry_initial_backoff_ms,
-            self.max_connections,
             self.connect_timeout_ms,
             self.read_timeout_ms,
             self.num_tries,
