@@ -1,15 +1,19 @@
+mod fixed_size_list_array;
 pub mod from;
 pub mod growable;
 pub mod iterator;
+mod list_array;
 pub mod ops;
 pub mod pseudo_arrow;
-
-mod fixed_size_list_array;
-mod list_array;
+mod serdes;
 mod struct_array;
+use arrow2::bitmap::Bitmap;
 pub use fixed_size_list_array::FixedSizeListArray;
 pub use list_array::ListArray;
+
 pub use struct_array::StructArray;
+mod boolean;
+mod from_iter;
 
 use std::{marker::PhantomData, sync::Arc};
 
@@ -72,7 +76,7 @@ where
         self.len() == 0
     }
 
-    pub fn with_validity(&self, validity: &[bool]) -> DaftResult<Self> {
+    pub fn with_validity_slice(&self, validity: &[bool]) -> DaftResult<Self> {
         if validity.len() != self.data.len() {
             return Err(DaftError::ValueError(format!(
                 "validity mask length does not match DataArray length, {} vs {}",
@@ -80,11 +84,26 @@ where
                 self.data.len()
             )));
         }
-        use arrow2::bitmap::Bitmap;
         let with_bitmap = self.data.with_validity(Some(Bitmap::from(validity)));
         DataArray::new(self.field.clone(), with_bitmap)
     }
 
+    pub fn with_validity(&self, validity: Option<Bitmap>) -> DaftResult<Self> {
+        if let Some(v) = &validity && v.len() != self.data.len() {
+            return Err(DaftError::ValueError(format!(
+                "validity mask length does not match DataArray length, {} vs {}",
+                v.len(),
+                self.data.len()
+            )));
+        }
+        let with_bitmap = self.data.with_validity(validity);
+        DataArray::new(self.field.clone(), with_bitmap)
+    }
+
+    pub fn validity(&self) -> Option<&Bitmap> {
+        self.data.validity()
+    }
+
     pub fn slice(&self, start: usize, end: usize) -> DaftResult<Self> {
         if start > end {
             return Err(DaftError::ValueError(format!(
