@@ -4,7 +4,7 @@ use arrow2::{array::PrimitiveArray, compute::arithmetics::basic};
 
 use crate::{
     array::DataArray,
-    datatypes::{DaftNumericType, Float64Array, Utf8Array},
+    datatypes::{DaftNumericType, Float64Array, Int64Array, Utf8Array},
     kernels::utf8::add_utf8_arrays,
 };
 
@@ -121,6 +121,13 @@ impl Div for &Float64Array {
     }
 }
 
+impl Div for &Int64Array {
+    type Output = DaftResult<Int64Array>;
+    fn div(self, rhs: Self) -> Self::Output {
+        arithmetic_helper(self, rhs, basic::div, |l, r| l / r)
+    }
+}
+
 pub fn binary_with_nulls<T, F>(
     lhs: &PrimitiveArray<T>,
     rhs: &PrimitiveArray<T>,
