@@ -1,4 +1,4 @@
-use std::iter::repeat;
+use std::{iter::repeat, sync::Arc};
 
 use super::as_arrow::AsArrow;
 use crate::{
@@ -14,8 +14,8 @@ use crate::{
             FixedShapeTensorArray, ImageArray, LogicalArray, LogicalArrayImpl, TensorArray,
             TimestampArray,
         },
-        DaftArrowBackedType, DaftLogicalType, DataType, Field, ImageMode, TimeUnit, UInt64Array,
-        Utf8Array,
+        DaftArrowBackedType, DaftLogicalType, DataType, Field, ImageMode, Int64Array, TimeUnit,
+        UInt64Array, Utf8Array,
     },
     series::{IntoSeries, Series},
     with_match_daft_logical_primitive_types,
@@ -141,8 +141,8 @@ where
         }
     };
 
-    let new_field = Field::new(to_cast.name(), dtype.clone());
-    Series::from_arrow(&new_field, result_arrow_physical_array)
+    let new_field = Arc::new(Field::new(to_cast.name(), dtype.clone()));
+    Series::from_arrow(new_field, result_arrow_physical_array)
 }
 
 fn arrow_cast<T>(to_cast: &DataArray<T>, dtype: &DataType) -> DaftResult<Series>
@@ -186,21 +186,13 @@ where
         )?
     } else if can_cast_types(&self_arrow_type, &target_arrow_type) {
         // Cast from logical Arrow2 type to logical Arrow2 type.
-        let arrow_logical = cast(
+        cast(
             to_cast.data(),
             &target_arrow_type,
             CastOptions {
                 wrapped: true,
                 partial: false,
             },
-        )?;
-        cast(
-            arrow_logical.as_ref(),
-            &target_arrow_physical_type,
-            CastOptions {
-                wrapped: true,
-                partial: false,
-            },
         )?
     } else if can_cast_types(&self_physical_arrow_type, &target_arrow_physical_type) {
         // Cast from physical Arrow2 type to physical Arrow2 type.
@@ -224,8 +216,8 @@ where
         )));
     };
 
-    let new_field = Field::new(to_cast.name(), dtype.clone());
-    Series::from_arrow(&new_field, result_array)
+    let new_field = Arc::new(Field::new(to_cast.name(), dtype.clone()));
+    Series::from_arrow(new_field, result_array)
 }
 
 impl<T> DataArray<T>
@@ -263,14 +255,12 @@ where
 
 impl DateArray {
     pub fn cast(&self, dtype: &DataType) -> DaftResult<Series> {
-        // We need to handle casts that Arrow doesn't allow, but our type-system does
-
         let date_array = self
             .as_arrow()
             .clone()
             .to(arrow2::datatypes::DataType::Date32);
-
         match dtype {
+            DataType::Date => Ok(self.clone().into_series()),
             DataType::Utf8 => {
                 // TODO: we should move this into our own strftime kernel
                 let year_array = compute::temporal::year(&date_array)?;
@@ -287,11 +277,27 @@ impl DateArray {
                     .collect();
                 Ok(Utf8Array::from((self.name(), Box::new(date_str))).into_series())
             }
+            DataType::Int32 => Ok(self.physical.clone().into_series()),
             DataType::Float32 => self.cast(&DataType::Int32)?.cast(&DataType::Float32),
             DataType::Float64 => self.cast(&DataType::Int32)?.cast(&DataType::Float64),
+            DataType::Timestamp(tu, _) => {
+                let days_to_unit: i64 = match tu {
+                    TimeUnit::Nanoseconds => 24 * 3_600_000_000_000,
+                    TimeUnit::Microseconds => 24 * 3_600_000_000,
+                    TimeUnit::Milliseconds => 24 * 3_600_000,
+                    TimeUnit::Seconds => 24 * 3_600,
+                };
+
+                let units_per_day = Int64Array::from(("units", vec![days_to_unit])).into_series();
+                let unit_since_epoch = ((&self.physical.clone().into_series()) * &units_per_day)?;
+                unit_since_epoch.cast(dtype)
+            }
             #[cfg(feature = "python")]
             DataType::Python => cast_logical_to_python_array(self, dtype),
-            _ => arrow_cast(&self.physical, dtype),
+            _ => Err(DaftError::TypeError(format!(
+                "Cannot cast Date to {}",
+                dtype
+            ))),
         }
     }
 }
@@ -314,7 +320,7 @@ pub(super) fn decimal128_to_str(val: i128, _precision: u8, scale: i8) -> String
     }
 }
 
-pub(super) fn timestamp_to_str_naive(val: i64, unit: &TimeUnit) -> String {
+pub(crate) fn timestamp_to_str_naive(val: i64, unit: &TimeUnit) -> String {
     let chrono_ts =
         { arrow2::temporal_conversions::timestamp_to_naive_datetime(val, unit.to_arrow()) };
     let format_str = match unit {
@@ -326,7 +332,7 @@ pub(super) fn timestamp_to_str_naive(val: i64, unit: &TimeUnit) -> String {
     chrono_ts.format(format_str).to_string()
 }
 
-pub(super) fn timestamp_to_str_offset(
+pub(crate) fn timestamp_to_str_offset(
     val: i64,
     unit: &TimeUnit,
     offset: &chrono::FixedOffset,
@@ -341,7 +347,7 @@ pub(super) fn timestamp_to_str_offset(
         .to_rfc3339_opts(seconds_format, false)
 }
 
-pub(super) fn timestamp_to_str_tz(val: i64, unit: &TimeUnit, tz: &chrono_tz::Tz) -> String {
+pub(crate) fn timestamp_to_str_tz(val: i64, unit: &TimeUnit, tz: &chrono_tz::Tz) -> String {
     let seconds_format = match unit {
         TimeUnit::Seconds => chrono::SecondsFormat::Secs,
         TimeUnit::Milliseconds => chrono::SecondsFormat::Millis,
@@ -356,9 +362,7 @@ impl TimestampArray {
     pub fn cast(&self, dtype: &DataType) -> DaftResult<Series> {
         match dtype {
             DataType::Timestamp(..) => arrow_logical_cast(self, dtype),
-            DataType::Date => Err(DaftError::TypeError(
-                "Cannot cast Timestamp to Date, use .date() instead.".to_string(),
-            )),
+            DataType::Date => Ok(self.date()?.into_series()),
             DataType::Utf8 => {
                 let DataType::Timestamp(unit, timezone) = self.data_type() else {
                     panic!("Wrong dtype for TimestampArray: {}", self.data_type())
@@ -747,7 +751,7 @@ fn extract_python_like_to_fixed_size_list<
     );
 
     FixedSizeListArray::from_arrow(
-        &Field::new(python_objects.name(), daft_type),
+        Arc::new(Field::new(python_objects.name(), daft_type)),
         Box::new(list_array),
     )
 }
@@ -784,7 +788,7 @@ fn extract_python_like_to_list<
     );
 
     ListArray::from_arrow(
-        &Field::new(python_objects.name(), daft_type),
+        Arc::new(Field::new(python_objects.name(), daft_type)),
         Box::new(list_arrow_array),
     )
 }
@@ -947,12 +951,12 @@ fn extract_python_like_to_tensor_array<
         Field::new(name, physical_type),
         vec![
             ListArray::from_arrow(
-                &Field::new("data", data_array.data_type().into()),
+                Arc::new(Field::new("data", data_array.data_type().into())),
                 data_array,
             )?
             .into_series(),
             ListArray::from_arrow(
-                &Field::new("shape", shapes_array.data_type().into()),
+                Arc::new(Field::new("shape", shapes_array.data_type().into())),
                 shapes_array,
             )?
             .into_series(),
