@@ -3,7 +3,8 @@ use num_traits::{NumCast, ToPrimitive};
 use crate::{
     array::DataArray,
     datatypes::{
-        BooleanArray, DaftArrowBackedType, DaftNumericType, DataType, NullArray, Utf8Array,
+        BinaryArray, BooleanArray, DaftArrowBackedType, DaftNumericType, DataType, NullArray,
+        Utf8Array,
     },
     utils::arrow::arrow_bitmap_and_helper,
 };
@@ -1165,6 +1166,302 @@ impl DaftCompare<&str> for Utf8Array {
     }
 }
 
+impl DaftCompare<&BinaryArray> for BinaryArray {
+    type Output = DaftResult<BooleanArray>;
+
+    fn equal(&self, rhs: &BinaryArray) -> Self::Output {
+        match (self.len(), rhs.len()) {
+            (x, y) if x == y => {
+                let validity =
+                    arrow_bitmap_and_helper(self.as_arrow().validity(), rhs.as_arrow().validity());
+                Ok(BooleanArray::from((
+                    self.name(),
+                    comparison::eq(self.as_arrow(), rhs.as_arrow()).with_validity(validity),
+                )))
+            }
+            (l_size, 1) => {
+                if let Some(value) = rhs.get(0) {
+                    self.equal(value)
+                } else {
+                    Ok(BooleanArray::full_null(
+                        self.name(),
+                        &DataType::Boolean,
+                        l_size,
+                    ))
+                }
+            }
+            (1, r_size) => {
+                if let Some(value) = self.get(0) {
+                    rhs.equal(value)
+                } else {
+                    Ok(BooleanArray::full_null(
+                        self.name(),
+                        &DataType::Boolean,
+                        r_size,
+                    ))
+                }
+            }
+            (l, r) => Err(DaftError::ValueError(format!(
+                "trying to compare different length arrays: {}: {l} vs {}: {r}",
+                self.name(),
+                rhs.name()
+            ))),
+        }
+    }
+
+    fn not_equal(&self, rhs: &BinaryArray) -> Self::Output {
+        match (self.len(), rhs.len()) {
+            (x, y) if x == y => {
+                let validity =
+                    arrow_bitmap_and_helper(self.as_arrow().validity(), rhs.as_arrow().validity());
+                Ok(BooleanArray::from((
+                    self.name(),
+                    comparison::neq(self.as_arrow(), rhs.as_arrow()).with_validity(validity),
+                )))
+            }
+            (l_size, 1) => {
+                if let Some(value) = rhs.get(0) {
+                    self.not_equal(value)
+                } else {
+                    Ok(BooleanArray::full_null(
+                        self.name(),
+                        &DataType::Boolean,
+                        l_size,
+                    ))
+                }
+            }
+            (1, r_size) => {
+                if let Some(value) = self.get(0) {
+                    rhs.not_equal(value)
+                } else {
+                    Ok(BooleanArray::full_null(
+                        self.name(),
+                        &DataType::Boolean,
+                        r_size,
+                    ))
+                }
+            }
+            (l, r) => Err(DaftError::ValueError(format!(
+                "trying to compare different length arrays: {}: {l} vs {}: {r}",
+                self.name(),
+                rhs.name()
+            ))),
+        }
+    }
+
+    fn lt(&self, rhs: &BinaryArray) -> Self::Output {
+        match (self.len(), rhs.len()) {
+            (x, y) if x == y => {
+                let validity =
+                    arrow_bitmap_and_helper(self.as_arrow().validity(), rhs.as_arrow().validity());
+                Ok(BooleanArray::from((
+                    self.name(),
+                    comparison::lt(self.as_arrow(), rhs.as_arrow()).with_validity(validity),
+                )))
+            }
+            (l_size, 1) => {
+                if let Some(value) = rhs.get(0) {
+                    self.lt(value)
+                } else {
+                    Ok(BooleanArray::full_null(
+                        self.name(),
+                        &DataType::Boolean,
+                        l_size,
+                    ))
+                }
+            }
+            (1, r_size) => {
+                if let Some(value) = self.get(0) {
+                    rhs.gt(value)
+                } else {
+                    Ok(BooleanArray::full_null(
+                        self.name(),
+                        &DataType::Boolean,
+                        r_size,
+                    ))
+                }
+            }
+            (l, r) => Err(DaftError::ValueError(format!(
+                "trying to compare different length arrays: {}: {l} vs {}: {r}",
+                self.name(),
+                rhs.name()
+            ))),
+        }
+    }
+
+    fn lte(&self, rhs: &BinaryArray) -> Self::Output {
+        match (self.len(), rhs.len()) {
+            (x, y) if x == y => {
+                let validity =
+                    arrow_bitmap_and_helper(self.as_arrow().validity(), rhs.as_arrow().validity());
+                Ok(BooleanArray::from((
+                    self.name(),
+                    comparison::lt_eq(self.as_arrow(), rhs.as_arrow()).with_validity(validity),
+                )))
+            }
+            (l_size, 1) => {
+                if let Some(value) = rhs.get(0) {
+                    self.lte(value)
+                } else {
+                    Ok(BooleanArray::full_null(
+                        self.name(),
+                        &DataType::Boolean,
+                        l_size,
+                    ))
+                }
+            }
+            (1, r_size) => {
+                if let Some(value) = self.get(0) {
+                    rhs.gte(value)
+                } else {
+                    Ok(BooleanArray::full_null(
+                        self.name(),
+                        &DataType::Boolean,
+                        r_size,
+                    ))
+                }
+            }
+            (l, r) => Err(DaftError::ValueError(format!(
+                "trying to compare different length arrays: {}: {l} vs {}: {r}",
+                self.name(),
+                rhs.name()
+            ))),
+        }
+    }
+
+    fn gt(&self, rhs: &BinaryArray) -> Self::Output {
+        match (self.len(), rhs.len()) {
+            (x, y) if x == y => {
+                let validity =
+                    arrow_bitmap_and_helper(self.as_arrow().validity(), rhs.as_arrow().validity());
+                Ok(BooleanArray::from((
+                    self.name(),
+                    comparison::gt(self.as_arrow(), rhs.as_arrow()).with_validity(validity),
+                )))
+            }
+            (l_size, 1) => {
+                if let Some(value) = rhs.get(0) {
+                    self.gt(value)
+                } else {
+                    Ok(BooleanArray::full_null(
+                        self.name(),
+                        &DataType::Boolean,
+                        l_size,
+                    ))
+                }
+            }
+            (1, r_size) => {
+                if let Some(value) = self.get(0) {
+                    rhs.lt(value)
+                } else {
+                    Ok(BooleanArray::full_null(
+                        self.name(),
+                        &DataType::Boolean,
+                        r_size,
+                    ))
+                }
+            }
+            (l, r) => Err(DaftError::ValueError(format!(
+                "trying to compare different length arrays: {}: {l} vs {}: {r}",
+                self.name(),
+                rhs.name()
+            ))),
+        }
+    }
+
+    fn gte(&self, rhs: &BinaryArray) -> Self::Output {
+        match (self.len(), rhs.len()) {
+            (x, y) if x == y => {
+                let validity =
+                    arrow_bitmap_and_helper(self.as_arrow().validity(), rhs.as_arrow().validity());
+                Ok(BooleanArray::from((
+                    self.name(),
+                    comparison::gt_eq(self.as_arrow(), rhs.as_arrow()).with_validity(validity),
+                )))
+            }
+            (l_size, 1) => {
+                if let Some(value) = rhs.get(0) {
+                    self.gte(value)
+                } else {
+                    Ok(BooleanArray::full_null(
+                        self.name(),
+                        &DataType::Boolean,
+                        l_size,
+                    ))
+                }
+            }
+            (1, r_size) => {
+                if let Some(value) = self.get(0) {
+                    rhs.lte(value)
+                } else {
+                    Ok(BooleanArray::full_null(
+                        self.name(),
+                        &DataType::Boolean,
+                        r_size,
+                    ))
+                }
+            }
+            (l, r) => Err(DaftError::ValueError(format!(
+                "trying to compare different length arrays: {}: {l} vs {}: {r}",
+                self.name(),
+                rhs.name()
+            ))),
+        }
+    }
+}
+
+impl DaftCompare<&[u8]> for BinaryArray {
+    type Output = DaftResult<BooleanArray>;
+
+    fn equal(&self, rhs: &[u8]) -> Self::Output {
+        let validity = self.as_arrow().validity().cloned();
+        let arrow_result =
+            comparison::binary::eq_scalar(self.as_arrow(), rhs).with_validity(validity);
+
+        Ok(BooleanArray::from((self.name(), arrow_result)))
+    }
+
+    fn not_equal(&self, rhs: &[u8]) -> Self::Output {
+        let validity = self.as_arrow().validity().cloned();
+        let arrow_result =
+            comparison::binary::neq_scalar(self.as_arrow(), rhs).with_validity(validity);
+
+        Ok(BooleanArray::from((self.name(), arrow_result)))
+    }
+
+    fn lt(&self, rhs: &[u8]) -> Self::Output {
+        let validity = self.as_arrow().validity().cloned();
+        let arrow_result =
+            comparison::binary::lt_scalar(self.as_arrow(), rhs).with_validity(validity);
+
+        Ok(BooleanArray::from((self.name(), arrow_result)))
+    }
+
+    fn lte(&self, rhs: &[u8]) -> Self::Output {
+        let validity = self.as_arrow().validity().cloned();
+        let arrow_result =
+            comparison::binary::lt_eq_scalar(self.as_arrow(), rhs).with_validity(validity);
+
+        Ok(BooleanArray::from((self.name(), arrow_result)))
+    }
+
+    fn gt(&self, rhs: &[u8]) -> Self::Output {
+        let validity = self.as_arrow().validity().cloned();
+        let arrow_result =
+            comparison::binary::gt_scalar(self.as_arrow(), rhs).with_validity(validity);
+
+        Ok(BooleanArray::from((self.name(), arrow_result)))
+    }
+
+    fn gte(&self, rhs: &[u8]) -> Self::Output {
+        let validity = self.as_arrow().validity().cloned();
+        let arrow_result =
+            comparison::binary::gt_eq_scalar(self.as_arrow(), rhs).with_validity(validity);
+
+        Ok(BooleanArray::from((self.name(), arrow_result)))
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use crate::{array::ops::DaftCompare, datatypes::Int64Array};
@@ -1177,7 +1474,7 @@ mod tests {
         let result: Vec<_> = array.equal(2).into_iter().collect();
         assert_eq!(result[..], [Some(false), Some(true), Some(false)]);
 
-        let array = array.with_validity(&[true, false, true])?;
+        let array = array.with_validity_slice(&[true, false, true])?;
         let result: Vec<_> = array.equal(2).into_iter().collect();
         assert_eq!(result[..], [Some(false), None, Some(false)]);
         Ok(())
@@ -1190,7 +1487,7 @@ mod tests {
         let result: Vec<_> = array.not_equal(2).into_iter().collect();
         assert_eq!(result[..], [Some(true), Some(false), Some(true)]);
 
-        let array = array.with_validity(&[true, false, true])?;
+        let array = array.with_validity_slice(&[true, false, true])?;
         let result: Vec<_> = array.not_equal(2).into_iter().collect();
         assert_eq!(result[..], [Some(true), None, Some(true)]);
         Ok(())
@@ -1203,7 +1500,7 @@ mod tests {
         let result: Vec<_> = array.lt(2).into_iter().collect();
         assert_eq!(result[..], [Some(true), Some(false), Some(false)]);
 
-        let array = array.with_validity(&[true, false, true])?;
+        let array = array.with_validity_slice(&[true, false, true])?;
         let result: Vec<_> = array.lt(2).into_iter().collect();
         assert_eq!(result[..], [Some(true), None, Some(false)]);
         Ok(())
@@ -1216,7 +1513,7 @@ mod tests {
         let result: Vec<_> = array.lte(2).into_iter().collect();
         assert_eq!(result[..], [Some(true), Some(true), Some(false)]);
 
-        let array = array.with_validity(&[true, false, true])?;
+        let array = array.with_validity_slice(&[true, false, true])?;
         let result: Vec<_> = array.lte(2).into_iter().collect();
         assert_eq!(result[..], [Some(true), None, Some(false)]);
         Ok(())
@@ -1229,7 +1526,7 @@ mod tests {
         let result: Vec<_> = array.gt(2).into_iter().collect();
         assert_eq!(result[..], [Some(false), Some(false), Some(true)]);
 
-        let array = array.with_validity(&[true, false, true])?;
+        let array = array.with_validity_slice(&[true, false, true])?;
         let result: Vec<_> = array.gt(2).into_iter().collect();
         assert_eq!(result[..], [Some(false), None, Some(true)]);
         Ok(())
@@ -1242,7 +1539,7 @@ mod tests {
         let result: Vec<_> = array.gte(2).into_iter().collect();
         assert_eq!(result[..], [Some(false), Some(true), Some(true)]);
 
-        let array = array.with_validity(&[true, false, true])?;
+        let array = array.with_validity_slice(&[true, false, true])?;
         let result: Vec<_> = array.gte(2).into_iter().collect();
         assert_eq!(result[..], [Some(false), None, Some(true)]);
         Ok(())
@@ -1255,7 +1552,7 @@ mod tests {
         let result: Vec<_> = array.equal(&array)?.into_iter().collect();
         assert_eq!(result[..], [Some(true), Some(true), Some(true)]);
 
-        let array = array.with_validity(&[true, false, true])?;
+        let array = array.with_validity_slice(&[true, false, true])?;
         let result: Vec<_> = array.equal(&array)?.into_iter().collect();
         assert_eq!(result[..], [Some(true), None, Some(true)]);
         Ok(())
@@ -1268,7 +1565,7 @@ mod tests {
         let result: Vec<_> = array.not_equal(&array)?.into_iter().collect();
         assert_eq!(result[..], [Some(false), Some(false), Some(false)]);
 
-        let array = array.with_validity(&[true, false, true])?;
+        let array = array.with_validity_slice(&[true, false, true])?;
         let result: Vec<_> = array.not_equal(&array)?.into_iter().collect();
         assert_eq!(result[..], [Some(false), None, Some(false)]);
         Ok(())
@@ -1281,11 +1578,11 @@ mod tests {
         let result: Vec<_> = lhs.lt(&rhs)?.into_iter().collect();
         assert_eq!(result[..], [Some(false), Some(false), Some(true)]);
 
-        let lhs = lhs.with_validity(&[true, false, true])?;
+        let lhs = lhs.with_validity_slice(&[true, false, true])?;
         let result: Vec<_> = lhs.lt(&rhs)?.into_iter().collect();
         assert_eq!(result[..], [Some(false), None, Some(true)]);
 
-        let rhs = rhs.with_validity(&[false, true, true])?;
+        let rhs = rhs.with_validity_slice(&[false, true, true])?;
         let result: Vec<_> = lhs.lt(&rhs)?.into_iter().collect();
         assert_eq!(result[..], [None, None, Some(true)]);
         Ok(())
@@ -1298,11 +1595,11 @@ mod tests {
         let result: Vec<_> = lhs.lte(&rhs)?.into_iter().collect();
         assert_eq!(result[..], [Some(false), Some(true), Some(true)]);
 
-        let lhs = lhs.with_validity(&[true, false, true])?;
+        let lhs = lhs.with_validity_slice(&[true, false, true])?;
         let result: Vec<_> = lhs.lte(&rhs)?.into_iter().collect();
         assert_eq!(result[..], [Some(false), None, Some(true)]);
 
-        let rhs = rhs.with_validity(&[false, true, true])?;
+        let rhs = rhs.with_validity_slice(&[false, true, true])?;
         let result: Vec<_> = lhs.lte(&rhs)?.into_iter().collect();
         assert_eq!(result[..], [None, None, Some(true)]);
         Ok(())
@@ -1315,11 +1612,11 @@ mod tests {
         let result: Vec<_> = lhs.gt(&rhs)?.into_iter().collect();
         assert_eq!(result[..], [Some(true), Some(false), Some(false)]);
 
-        let lhs = lhs.with_validity(&[true, false, true])?;
+        let lhs = lhs.with_validity_slice(&[true, false, true])?;
         let result: Vec<_> = lhs.gt(&rhs)?.into_iter().collect();
         assert_eq!(result[..], [Some(true), None, Some(false)]);
 
-        let rhs = rhs.with_validity(&[false, true, true])?;
+        let rhs = rhs.with_validity_slice(&[false, true, true])?;
         let result: Vec<_> = lhs.gt(&rhs)?.into_iter().collect();
         assert_eq!(result[..], [None, None, Some(false)]);
         Ok(())
@@ -1332,11 +1629,11 @@ mod tests {
         let result: Vec<_> = lhs.gte(&rhs)?.into_iter().collect();
         assert_eq!(result[..], [Some(true), Some(true), Some(false)]);
 
-        let lhs = lhs.with_validity(&[true, false, true])?;
+        let lhs = lhs.with_validity_slice(&[true, false, true])?;
         let result: Vec<_> = lhs.gte(&rhs)?.into_iter().collect();
         assert_eq!(result[..], [Some(true), None, Some(false)]);
 
-        let rhs = rhs.with_validity(&[false, true, true])?;
+        let rhs = rhs.with_validity_slice(&[false, true, true])?;
         let result: Vec<_> = lhs.gte(&rhs)?.into_iter().collect();
         assert_eq!(result[..], [None, None, Some(false)]);
         Ok(())
