@@ -5,7 +5,7 @@ mod arithmetic;
 pub mod arrow2;
 pub mod as_arrow;
 pub(crate) mod broadcast;
-mod cast;
+pub(crate) mod cast;
 mod compare_agg;
 mod comparison;
 mod concat;
@@ -30,9 +30,11 @@ mod pairwise;
 mod repr;
 mod search_sorted;
 mod sort;
+mod struct_;
 mod sum;
 mod take;
 pub(crate) mod tensor;
+mod truncate;
 mod utf8;
 
 pub use sort::{build_multi_array_bicompare, build_multi_array_compare};
@@ -81,6 +83,11 @@ pub trait DaftIsNull {
     fn is_null(&self) -> Self::Output;
 }
 
+pub trait DaftNotNull {
+    type Output;
+    fn not_null(&self) -> Self::Output;
+}
+
 pub trait DaftIsNan {
     type Output;
     fn is_nan(&self) -> Self::Output;
