@@ -2,7 +2,7 @@ use std::ops::{Add, Div, Mul, Rem, Sub};
 
 use common_error::{DaftError, DaftResult};
 
-use crate::impl_binary_trait_by_reference;
+use crate::{impl_binary_trait_by_reference, utils::supertype::try_get_supertype};
 
 use super::DataType;
 
@@ -24,29 +24,42 @@ impl DataType {
             ))
         })
     }
-    pub fn comparison_op(&self, other: &Self) -> DaftResult<(DataType, DataType)> {
+    pub fn comparison_op(
+        &self,
+        other: &Self,
+    ) -> DaftResult<(DataType, Option<DataType>, DataType)> {
         // Whether a comparison op is supported between the two types.
         // Returns:
         // - the output type,
+        // - an optional intermediate type
         // - the type at which the comparison should be performed.
-        use DataType::*;
-        match (self, other) {
-            // TODO: [ISSUE-688] Make Binary type comparable
-            (Binary, _) | (_, Binary) => Err(()),
-            (s, o) if s == o => Ok(s.to_physical()),
-            (s, o) if s.is_physical() && o.is_physical() => {
-                try_physical_supertype(s, o).map_err(|_| ())
+        let evaluator = || {
+            use DataType::*;
+            match (self, other) {
+                (s, o) if s == o => Ok((Boolean, None, s.to_physical())),
+                (s, o) if s.is_physical() && o.is_physical() => {
+                    Ok((Boolean, None, try_physical_supertype(s, o)?))
+                }
+                (Timestamp(..), Timestamp(..)) => {
+                    let intermediate_type = try_get_supertype(self, other)?;
+                    let pt = intermediate_type.to_physical();
+                    Ok((Boolean, Some(intermediate_type), pt))
+                }
+                (Timestamp(..), Date) | (Date, Timestamp(..)) => {
+                    let intermediate_type = Date;
+                    let pt = intermediate_type.to_physical();
+                    Ok((Boolean, Some(intermediate_type), pt))
+                }
+                _ => Err(DaftError::TypeError(format!(
+                    "Cannot perform comparison on types: {}, {}",
+                    self, other
+                ))),
             }
-            // To maintain existing behaviour. TODO: cleanup
-            (Date, o) | (o, Date) if o.is_physical() && o.clone() != Boolean => {
-                try_physical_supertype(&Date.to_physical(), o).map_err(|_| ())
-            }
-            _ => Err(()),
-        }
-        .map(|comp_type| (Boolean, comp_type))
-        .map_err(|()| {
+        };
+
+        evaluator().map_err(|err| {
             DaftError::TypeError(format!(
-                "Cannot perform comparison on types: {}, {}",
+                "Cannot perform comparison on types: {}, {}\nDetails:\n{err}",
                 self, other
             ))
         })
@@ -187,10 +200,10 @@ pub fn try_physical_supertype(l: &DataType, r: &DataType) -> DaftResult<DataType
     use DataType::*;
     try_numeric_supertype(l, r).or(match (l, r) {
         (Null, other) | (other, Null) if other.is_physical() => Ok(other.clone()),
-        (Boolean, other) | (other, Boolean) if other.is_physical() => Ok(other.clone()),
+        (Boolean, other) | (other, Boolean) if other.is_numeric() => Ok(other.clone()),
         #[cfg(feature = "python")]
         (Python, _) | (_, Python) => Ok(Python),
-        (Utf8, o) | (o, Utf8) if o.is_physical() => Ok(Utf8),
+        (Utf8, o) | (o, Utf8) if o.is_physical() && !matches!(o, Binary) => Ok(Utf8),
         _ => Err(DaftError::TypeError(format!(
             "Invalid arguments to try_physical_supertype: {}, {}",
             l, r
