@@ -15,7 +15,7 @@ use super::{
 
 /// A LogicalArray is a wrapper on top of some underlying array, applying the semantic meaning of its
 /// field.datatype() to the underlying array.
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct LogicalArrayImpl<L: DaftLogicalType, PhysicalArray: DaftArrayType> {
     pub field: Arc<Field>,
     pub physical: PhysicalArray,
