@@ -6,8 +6,6 @@ mod image_mode;
 mod matching;
 mod time_unit;
 
-use std::ops::{Add, Div, Mul, Rem, Sub};
-
 pub use crate::array::{DataArray, FixedSizeListArray};
 use crate::array::{ListArray, StructArray};
 use arrow2::{
@@ -18,9 +16,12 @@ pub use binary_ops::try_physical_supertype;
 pub use dtype::DataType;
 pub use field::Field;
 pub use field::FieldID;
+pub use field::FieldRef;
 pub use image_format::ImageFormat;
 pub use image_mode::ImageMode;
 use num_traits::{Bounded, Float, FromPrimitive, Num, NumCast, ToPrimitive, Zero};
+use serde::Serialize;
+use std::ops::{Add, Div, Mul, Rem, Sub};
 pub use time_unit::TimeUnit;
 
 pub mod logical;
@@ -57,7 +58,7 @@ pub trait DaftLogicalType: Send + Sync + DaftDataType + 'static {
 
 macro_rules! impl_daft_arrow_datatype {
     ($ca:ident, $variant:ident) => {
-        #[derive(Clone)]
+        #[derive(Clone, Debug)]
         pub struct $ca {}
 
         impl DaftDataType for $ca {
@@ -76,7 +77,7 @@ macro_rules! impl_daft_arrow_datatype {
 
 macro_rules! impl_daft_non_arrow_datatype {
     ($ca:ident, $variant:ident) => {
-        #[derive(Clone)]
+        #[derive(Clone, Debug)]
         pub struct $ca {}
 
         impl DaftDataType for $ca {
@@ -93,7 +94,7 @@ macro_rules! impl_daft_non_arrow_datatype {
 
 macro_rules! impl_daft_logical_data_array_datatype {
     ($ca:ident, $variant:ident, $physical_type:ident) => {
-        #[derive(Clone)]
+        #[derive(Clone, Debug)]
         pub struct $ca {}
 
         impl DaftDataType for $ca {
@@ -113,7 +114,7 @@ macro_rules! impl_daft_logical_data_array_datatype {
 
 macro_rules! impl_daft_logical_fixed_size_list_datatype {
     ($ca:ident, $variant:ident) => {
-        #[derive(Clone)]
+        #[derive(Clone, Debug)]
         pub struct $ca {}
 
         impl DaftDataType for $ca {
@@ -133,7 +134,7 @@ macro_rules! impl_daft_logical_fixed_size_list_datatype {
 
 macro_rules! impl_nested_datatype {
     ($ca:ident, $array_type:ident) => {
-        #[derive(Clone)]
+        #[derive(Clone, Debug)]
         pub struct $ca {}
 
         impl DaftDataType for $ca {
@@ -201,6 +202,7 @@ pub trait NumericNative:
     + Bounded
     + FromPrimitive
     + ToPrimitive
+    + Serialize
 {
     type DAFTTYPE: DaftNumericType;
 }
