@@ -1,6 +1,11 @@
 use std::ops::{Add, Div, Mul, Rem, Sub};
 
-use pyo3::{exceptions::PyValueError, prelude::*, pyclass::CompareOp, types::PyList};
+use pyo3::{
+    exceptions::PyValueError,
+    prelude::*,
+    pyclass::CompareOp,
+    types::{PyBytes, PyList},
+};
 
 use crate::{
     array::{ops::DaftLogical, pseudo_arrow::PseudoArrowArray, DataArray},
@@ -247,6 +252,10 @@ impl PySeries {
         Ok(self.series.utf8_contains(&pattern.series)?.into())
     }
 
+    pub fn utf8_split(&self, pattern: &Self) -> PyResult<Self> {
+        Ok(self.series.utf8_split(&pattern.series)?.into())
+    }
+
     pub fn utf8_length(&self) -> PyResult<Self> {
         Ok(self.series.utf8_length()?.into())
     }
@@ -263,6 +272,10 @@ impl PySeries {
         Ok(self.series.dt_day()?.into())
     }
 
+    pub fn dt_hour(&self) -> PyResult<Self> {
+        Ok(self.series.dt_hour()?.into())
+    }
+
     pub fn dt_month(&self) -> PyResult<Self> {
         Ok(self.series.dt_month()?.into())
     }
@@ -275,10 +288,42 @@ impl PySeries {
         Ok(self.series.dt_day_of_week()?.into())
     }
 
+    pub fn partitioning_days(&self) -> PyResult<Self> {
+        Ok(self.series.partitioning_days()?.into())
+    }
+
+    pub fn partitioning_hours(&self) -> PyResult<Self> {
+        Ok(self.series.partitioning_hours()?.into())
+    }
+
+    pub fn partitioning_months(&self) -> PyResult<Self> {
+        Ok(self.series.partitioning_months()?.into())
+    }
+
+    pub fn partitioning_years(&self) -> PyResult<Self> {
+        Ok(self.series.partitioning_years()?.into())
+    }
+
+    pub fn partitioning_iceberg_bucket(&self, n: i32) -> PyResult<Self> {
+        Ok(self.series.partitioning_iceberg_bucket(n)?.into())
+    }
+
+    pub fn partitioning_iceberg_truncate(&self, w: i64) -> PyResult<Self> {
+        Ok(self.series.partitioning_iceberg_truncate(w)?.into())
+    }
+
+    pub fn murmur3_32(&self) -> PyResult<Self> {
+        Ok(self.series.murmur3_32()?.into_series().into())
+    }
+
     pub fn list_lengths(&self) -> PyResult<Self> {
         Ok(self.series.list_lengths()?.into_series().into())
     }
 
+    pub fn list_get(&self, idx: &Self, default: &Self) -> PyResult<Self> {
+        Ok(self.series.list_get(&idx.series, &default.series)?.into())
+    }
+
     pub fn image_decode(&self) -> PyResult<Self> {
         Ok(self.series.image_decode()?.into())
     }
@@ -312,6 +357,21 @@ impl PySeries {
     pub fn is_null(&self) -> PyResult<Self> {
         Ok(self.series.is_null()?.into())
     }
+
+    pub fn not_null(&self) -> PyResult<Self> {
+        Ok(self.series.not_null()?.into())
+    }
+
+    pub fn _debug_bincode_serialize(&self, py: Python) -> PyResult<PyObject> {
+        let values = bincode::serialize(&self.series).unwrap();
+        Ok(PyBytes::new(py, &values).to_object(py))
+    }
+
+    #[staticmethod]
+    pub fn _debug_bincode_deserialize(bytes: &PyBytes) -> PyResult<Self> {
+        let values = bincode::deserialize::<Series>(bytes.as_bytes()).unwrap();
+        Ok(Self { series: values })
+    }
 }
 
 impl From<series::Series> for PySeries {
