@@ -57,6 +57,17 @@ macro_rules! impl_series_like_for_data_array {
                 self
             }
 
+            fn with_validity(
+                &self,
+                validity: Option<arrow2::bitmap::Bitmap>,
+            ) -> DaftResult<Series> {
+                Ok(self.0.with_validity(validity)?.into_series())
+            }
+
+            fn validity(&self) -> Option<&arrow2::bitmap::Bitmap> {
+                self.0.validity()
+            }
+
             fn broadcast(&self, num: usize) -> DaftResult<Series> {
                 Ok(self.0.broadcast(num)?.into_series())
             }
@@ -90,6 +101,13 @@ macro_rules! impl_series_like_for_data_array {
 
                 Ok(DaftIsNull::is_null(&self.0)?.into_series())
             }
+
+            fn not_null(&self) -> DaftResult<Series> {
+                use crate::array::ops::DaftNotNull;
+
+                Ok(DaftNotNull::not_null(&self.0)?.into_series())
+            }
+
             fn len(&self) -> usize {
                 self.0.len()
             }
