@@ -39,6 +39,17 @@ macro_rules! impl_series_like_for_logical_array {
                 self
             }
 
+            fn with_validity(
+                &self,
+                validity: Option<arrow2::bitmap::Bitmap>,
+            ) -> DaftResult<Series> {
+                Ok(self.0.physical.with_validity(validity)?.into_series())
+            }
+
+            fn validity(&self) -> Option<&arrow2::bitmap::Bitmap> {
+                self.0.physical.validity()
+            }
+
             fn broadcast(&self, num: usize) -> DaftResult<Series> {
                 use crate::array::ops::broadcast::Broadcastable;
                 let data_array = self.0.physical.broadcast(num)?;
@@ -80,6 +91,12 @@ macro_rules! impl_series_like_for_logical_array {
                 Ok(DaftIsNull::is_null(&self.0.physical)?.into_series())
             }
 
+            fn not_null(&self) -> DaftResult<Series> {
+                use crate::array::ops::DaftNotNull;
+
+                Ok(DaftNotNull::not_null(&self.0.physical)?.into_series())
+            }
+
             fn len(&self) -> usize {
                 self.0.physical.len()
             }
