@@ -1,16 +1,18 @@
 mod array_impl;
 mod from;
 mod ops;
+mod serdes;
 mod series_like;
-
 use std::{
+    borrow::Cow,
     fmt::{Display, Formatter, Result},
     sync::Arc,
 };
 
 use crate::{
     array::ops::{from_arrow::FromArrow, full::FullNull},
-    datatypes::{DataType, Field},
+    datatypes::{DataType, Field, FieldRef},
+    utils::display_table::make_comfy_table,
     with_match_daft_types,
 };
 use common_error::DaftResult;
@@ -19,7 +21,7 @@ pub use array_impl::IntoSeries;
 
 pub(crate) use self::series_like::SeriesLike;
 
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct Series {
     pub inner: Arc<dyn SeriesLike>,
 }
@@ -33,7 +35,10 @@ impl Series {
     ///
     /// This function will check the provided [`Field`] (and all its associated potentially nested fields/dtypes) against
     /// the provided [`arrow2::array::Array`] for compatibility, and returns an error if they do not match.
-    pub fn from_arrow(field: &Field, arrow_arr: Box<dyn arrow2::array::Array>) -> DaftResult<Self> {
+    pub fn from_arrow(
+        field: FieldRef,
+        arrow_arr: Box<dyn arrow2::array::Array>,
+    ) -> DaftResult<Self> {
         with_match_daft_types!(field.dtype, |$T| {
             Ok(<<$T as DaftDataType>::ArrayType as FromArrow>::from_arrow(field, arrow_arr)?.into_series())
         })
@@ -77,48 +82,28 @@ impl Series {
         }
     }
 
-    pub fn to_prettytable(&self) -> prettytable::Table {
-        let mut table = prettytable::Table::new();
+    pub fn to_comfy_table(&self) -> comfy_table::Table {
+        make_comfy_table(
+            vec![Cow::Borrowed(self.field())].as_slice(),
+            Some([self].as_slice()),
+            Some(80),
+        )
+    }
 
-        let header =
-            prettytable::Cell::new(format!("{}\n{}", self.name(), self.data_type()).as_str())
-                .with_style(prettytable::Attr::Bold);
-        table.add_row(prettytable::Row::new(vec![header]));
+    pub fn with_validity(&self, validity: Option<arrow2::bitmap::Bitmap>) -> DaftResult<Series> {
+        self.inner.with_validity(validity)
+    }
 
-        let head_rows;
-        let tail_rows;
-
-        if self.len() > 10 {
-            head_rows = 5;
-            tail_rows = 5;
-        } else {
-            head_rows = self.len();
-            tail_rows = 0;
-        }
-
-        for i in 0..head_rows {
-            let row = vec![self.str_value(i).unwrap()];
-            table.add_row(row.into());
-        }
-        if tail_rows != 0 {
-            let row = vec!["..."];
-            table.add_row(row.into());
-        }
-
-        for i in 0..tail_rows {
-            let row = vec![self.str_value(self.len() - tail_rows - 1 + i).unwrap()];
-            table.add_row(row.into());
-        }
-
-        table
+    pub fn validity(&self) -> Option<&arrow2::bitmap::Bitmap> {
+        self.inner.validity()
     }
 }
 
 impl Display for Series {
     // `f` is a buffer, and this method must write the formatted string into it
     fn fmt(&self, f: &mut Formatter) -> Result {
-        let table = self.to_prettytable();
-        write!(f, "{table}")
+        let table = self.to_comfy_table();
+        writeln!(f, "{table}")
     }
 }
 
