@@ -1,5 +1,5 @@
 use crate::array::{FixedSizeListArray, ListArray, StructArray};
-use crate::datatypes::logical::FixedShapeImageArray;
+use crate::datatypes::logical::{DateArray, Decimal128Array, FixedShapeImageArray, TimestampArray};
 use crate::datatypes::*;
 use crate::series::array_impl::ArrayWrapper;
 use crate::series::Series;
@@ -95,6 +95,18 @@ impl Series {
         self.downcast()
     }
 
+    pub fn date(&self) -> DaftResult<&DateArray> {
+        self.downcast()
+    }
+
+    pub fn timestamp(&self) -> DaftResult<&TimestampArray> {
+        self.downcast()
+    }
+
+    pub fn decimal128(&self) -> DaftResult<&Decimal128Array> {
+        self.downcast()
+    }
+
     #[cfg(feature = "python")]
     pub fn python(&self) -> DaftResult<&PythonArray> {
         self.downcast()
