@@ -41,6 +41,7 @@ pub enum Expr {
     },
     Not(ExprRef),
     IsNull(ExprRef),
+    NotNull(ExprRef),
     Literal(lit::LiteralValue),
     IfElse {
         if_true: ExprRef,
@@ -189,13 +190,6 @@ impl AggExpr {
             }
             Min(expr) | Max(expr) => {
                 let field = expr.to_field(schema)?;
-                // TODO: [ISSUE-688] Make Binary type comparable
-                if field.dtype == DataType::Binary {
-                    return Err(DaftError::TypeError(format!(
-                        "Cannot get min/max of Binary type in column \"{}\"",
-                        field.name,
-                    )));
-                }
                 Ok(Field::new(field.name.as_str(), field.dtype))
             }
             List(expr) => expr.to_field(schema)?.to_list_field(),
@@ -290,6 +284,10 @@ impl Expr {
         Expr::IsNull(self.clone().into())
     }
 
+    pub fn not_null(&self) -> Self {
+        Expr::NotNull(self.clone().into())
+    }
+
     pub fn eq(&self, other: &Self) -> Self {
         binary_op(Operator::Eq, self, other)
     }
@@ -345,6 +343,10 @@ impl Expr {
                 let child_id = expr.semantic_id(schema);
                 FieldID::new(format!("{child_id}.is_null()"))
             }
+            NotNull(expr) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.not_null()"))
+            }
             Function { func, inputs } => {
                 let inputs = inputs
                     .iter()
@@ -388,7 +390,9 @@ impl Expr {
             Literal(..) => vec![],
 
             // One child.
-            Not(expr) | IsNull(expr) | Cast(expr, ..) | Alias(expr, ..) => vec![expr.clone()],
+            Not(expr) | IsNull(expr) | NotNull(expr) | Cast(expr, ..) | Alias(expr, ..) => {
+                vec![expr.clone()]
+            }
             Agg(agg_expr) => vec![agg_expr.child()],
 
             // Multiple children.
@@ -423,6 +427,7 @@ impl Expr {
                 }
             }
             IsNull(expr) => Ok(Field::new(expr.name()?, DataType::Boolean)),
+            NotNull(expr) => Ok(Field::new(expr.name()?, DataType::Boolean)),
             Literal(value) => Ok(Field::new("literal", value.get_type())),
             Function { func, inputs } => func.to_field(inputs.as_slice(), schema, self),
             BinaryOp { op, left, right } => {
@@ -443,7 +448,7 @@ impl Expr {
                     | Operator::NotEq
                     | Operator::LtEq
                     | Operator::GtEq => {
-                        let (result_type, _comp_type) =
+                        let (result_type, _intermediate, _comp_type) =
                             left_field.dtype.comparison_op(&right_field.dtype)?;
                         Ok(Field::new(left_field.name.as_str(), result_type))
                     }
@@ -504,6 +509,7 @@ impl Expr {
             Column(name) => Ok(name.as_ref()),
             Not(expr) => expr.name(),
             IsNull(expr) => expr.name(),
+            NotNull(expr) => expr.name(),
             Literal(..) => Ok("literal"),
             Function { func: _, inputs } => inputs.first().unwrap().name(),
             BinaryOp {
@@ -556,6 +562,7 @@ impl Display for Expr {
             Column(name) => write!(f, "col({name})"),
             Not(expr) => write!(f, "not({expr})"),
             IsNull(expr) => write!(f, "is_null({expr})"),
+            NotNull(expr) => write!(f, "not_null({expr})"),
             Literal(val) => write!(f, "lit({val})"),
             Function { func, inputs } => {
                 write!(f, "{}(", func.fn_name())?;
