@@ -1,7 +1,7 @@
 use std::ops::Range;
 use std::sync::Arc;
 
-use futures::StreamExt;
+use futures::stream::BoxStream;
 use futures::TryStreamExt;
 use google_cloud_storage::client::ClientConfig;
 
@@ -19,10 +19,15 @@ use crate::object_io::FileMetadata;
 use crate::object_io::FileType;
 use crate::object_io::LSResult;
 use crate::object_io::ObjectSource;
-use crate::s3_like;
+use crate::stats::IOStatsRef;
+use crate::stream_utils::io_stats_on_bytestream;
 use crate::GetResult;
 use common_io_config::GCSConfig;
 
+const GCS_DELIMITER: &str = "/";
+const GCS_SCHEME: &str = "gs";
+const DEFAULT_GLOB_FANOUT_LIMIT: usize = 1024;
+
 #[derive(Debug, Snafu)]
 enum Error {
     #[snafu(display("Unable to open {}: {}", path, source))]
@@ -103,10 +108,7 @@ impl From<Error> for super::Error {
     }
 }
 
-enum GCSClientWrapper {
-    Native(Client),
-    S3Compat(Arc<s3_like::S3LikeSource>),
-}
+struct GCSClientWrapper(Client);
 
 fn parse_uri(uri: &url::Url) -> super::Result<(&str, &str)> {
     let bucket = match uri.host_str() {
@@ -117,86 +119,89 @@ fn parse_uri(uri: &url::Url) -> super::Result<(&str, &str)> {
         }),
     }?;
     let key = uri.path();
-    let key = key.strip_prefix('/').unwrap_or(key);
+    let key = key.strip_prefix(GCS_DELIMITER).unwrap_or(key);
     Ok((bucket, key))
 }
 
 impl GCSClientWrapper {
-    async fn get(&self, uri: &str, range: Option<Range<usize>>) -> super::Result<GetResult> {
+    async fn get(
+        &self,
+        uri: &str,
+        range: Option<Range<usize>>,
+        io_stats: Option<IOStatsRef>,
+    ) -> super::Result<GetResult> {
         let uri = url::Url::parse(uri).with_context(|_| InvalidUrlSnafu { path: uri })?;
         let (bucket, key) = parse_uri(&uri)?;
-        match self {
-            GCSClientWrapper::Native(client) => {
-                let req = GetObjectRequest {
-                    bucket: bucket.into(),
-                    object: key.into(),
-                    ..Default::default()
-                };
-                use google_cloud_storage::http::objects::download::Range as GRange;
-                let (grange, size) = if let Some(range) = range {
-                    (
-                        GRange(Some(range.start as u64), Some(range.end as u64)),
-                        Some(range.len()),
-                    )
-                } else {
-                    (GRange::default(), None)
-                };
-                let owned_uri = uri.to_string();
-                let response = client
-                    .download_streamed_object(&req, &grange)
-                    .await
-                    .context(UnableToOpenFileSnafu {
-                        path: uri.to_string(),
-                    })?;
-                let response = response.map_err(move |e| {
-                    UnableToReadBytesSnafu::<String> {
-                        path: owned_uri.clone(),
-                    }
-                    .into_error(e)
-                    .into()
-                });
-                Ok(GetResult::Stream(response.boxed(), size, None))
-            }
-            GCSClientWrapper::S3Compat(client) => {
-                let uri = format!("s3://{}/{}", bucket, key);
-                client.get(&uri, range).await
+        let client = &self.0;
+        let req = GetObjectRequest {
+            bucket: bucket.into(),
+            object: key.into(),
+            ..Default::default()
+        };
+        use google_cloud_storage::http::objects::download::Range as GRange;
+        let (grange, size) = if let Some(range) = range {
+            (
+                GRange(Some(range.start as u64), Some(range.end as u64)),
+                Some(range.len()),
+            )
+        } else {
+            (GRange::default(), None)
+        };
+        let owned_uri = uri.to_string();
+        let response = client
+            .download_streamed_object(&req, &grange)
+            .await
+            .context(UnableToOpenFileSnafu {
+                path: uri.to_string(),
+            })?;
+        let response = response.map_err(move |e| {
+            UnableToReadBytesSnafu::<String> {
+                path: owned_uri.clone(),
             }
+            .into_error(e)
+            .into()
+        });
+        if let Some(is) = io_stats.as_ref() {
+            is.mark_get_requests(1)
         }
+        Ok(GetResult::Stream(
+            io_stats_on_bytestream(response, io_stats),
+            size,
+            None,
+        ))
     }
 
-    async fn get_size(&self, uri: &str) -> super::Result<usize> {
+    async fn get_size(&self, uri: &str, io_stats: Option<IOStatsRef>) -> super::Result<usize> {
         let uri = url::Url::parse(uri).with_context(|_| InvalidUrlSnafu { path: uri })?;
         let (bucket, key) = parse_uri(&uri)?;
-        match self {
-            GCSClientWrapper::Native(client) => {
-                let req = GetObjectRequest {
-                    bucket: bucket.into(),
-                    object: key.into(),
-                    ..Default::default()
-                };
+        let client = &self.0;
+        let req = GetObjectRequest {
+            bucket: bucket.into(),
+            object: key.into(),
+            ..Default::default()
+        };
 
-                let response = client
-                    .get_object(&req)
-                    .await
-                    .context(UnableToOpenFileSnafu {
-                        path: uri.to_string(),
-                    })?;
-                Ok(response.size as usize)
-            }
-            GCSClientWrapper::S3Compat(client) => {
-                let uri = format!("s3://{}/{}", bucket, key);
-                client.get_size(&uri).await
-            }
+        let response = client
+            .get_object(&req)
+            .await
+            .context(UnableToOpenFileSnafu {
+                path: uri.to_string(),
+            })?;
+        if let Some(is) = io_stats.as_ref() {
+            is.mark_head_requests(1)
         }
+        Ok(response.size as usize)
     }
-
+    #[allow(clippy::too_many_arguments)]
     async fn _ls_impl(
         &self,
         client: &Client,
         bucket: &str,
         key: &str,
-        delimiter: &str,
+        delimiter: Option<&str>,
         continuation_token: Option<&str>,
+        page_size: Option<i32>,
+        io_stats: Option<&IOStatsRef>,
     ) -> super::Result<LSResult> {
         let req = ListObjectsRequest {
             bucket: bucket.to_string(),
@@ -204,8 +209,8 @@ impl GCSClientWrapper {
             end_offset: None,
             start_offset: None,
             page_token: continuation_token.map(|s| s.to_string()),
-            delimiter: Some(delimiter.to_string()), // returns results in "directory mode"
-            max_results: Some(1000),                // Recommended value from API docs
+            delimiter: delimiter.map(|d| d.to_string()), // returns results in "directory mode" if delimiter is provided
+            max_results: page_size,
             include_trailing_delimiter: Some(false), // This will not populate "directories" in the response's .item[]
             projection: None,
             versions: None,
@@ -214,17 +219,21 @@ impl GCSClientWrapper {
             .list_objects(&req)
             .await
             .context(UnableToListObjectsSnafu {
-                path: format!("gs://{}/{}", bucket, key),
+                path: format!("{GCS_SCHEME}://{}/{}", bucket, key),
             })?;
+        if let Some(is) = io_stats.as_ref() {
+            is.mark_list_requests(1)
+        }
+
         let response_items = ls_response.items.unwrap_or_default();
         let response_prefixes = ls_response.prefixes.unwrap_or_default();
         let files = response_items.iter().map(|obj| FileMetadata {
-            filepath: format!("gs://{}/{}", bucket, obj.name),
+            filepath: format!("{GCS_SCHEME}://{}/{}", bucket, obj.name),
             size: Some(obj.size as u64),
             filetype: FileType::File,
         });
         let dirs = response_prefixes.iter().map(|pref| FileMetadata {
-            filepath: format!("gs://{}/{}", bucket, pref),
+            filepath: format!("{GCS_SCHEME}://{}/{}", bucket, pref),
             size: None,
             filetype: FileType::Directory,
         });
@@ -237,50 +246,76 @@ impl GCSClientWrapper {
     async fn ls(
         &self,
         path: &str,
-        delimiter: Option<&str>,
+        posix: bool,
         continuation_token: Option<&str>,
+        page_size: Option<i32>,
+        io_stats: Option<IOStatsRef>,
     ) -> super::Result<LSResult> {
         let uri = url::Url::parse(path).with_context(|_| InvalidUrlSnafu { path })?;
         let (bucket, key) = parse_uri(&uri)?;
-        let delimiter = delimiter.unwrap_or("/");
-        match self {
-            GCSClientWrapper::Native(client) => {
-                // Attempt to forcefully ls the key as a directory (by ensuring a "/" suffix)
-                let forced_directory_key =
-                    format!("{}{delimiter}", key.trim_end_matches(delimiter));
-                let forced_directory_ls_result = self
+        let client = &self.0;
+
+        if posix {
+            // Attempt to forcefully ls the key as a directory (by ensuring a "/" suffix)
+            let forced_directory_key = if key.is_empty() {
+                "".to_string()
+            } else {
+                format!("{}{GCS_DELIMITER}", key.trim_end_matches(GCS_DELIMITER))
+            };
+            let forced_directory_ls_result = self
+                ._ls_impl(
+                    client,
+                    bucket,
+                    forced_directory_key.as_str(),
+                    Some(GCS_DELIMITER),
+                    continuation_token,
+                    page_size,
+                    io_stats.as_ref(),
+                )
+                .await?;
+
+            // If no items were obtained, then this is actually a file and we perform a second ls to obtain just the file's
+            // details as the one-and-only-one entry
+            if forced_directory_ls_result.files.is_empty() {
+                let mut file_result = self
                     ._ls_impl(
                         client,
                         bucket,
-                        forced_directory_key.as_str(),
-                        delimiter,
+                        key,
+                        Some(GCS_DELIMITER),
                         continuation_token,
+                        page_size,
+                        io_stats.as_ref(),
                     )
                     .await?;
 
-                // If no items were obtained, then this is actually a file and we perform a second ls to obtain just the file's
-                // details as the one-and-only-one entry
-                if forced_directory_ls_result.files.is_empty() {
-                    let file_result = self
-                        ._ls_impl(client, bucket, key, delimiter, continuation_token)
-                        .await?;
+                // Only retain exact matches (since the API does prefix lists by default)
+                let target_path = format!("{GCS_SCHEME}://{bucket}/{key}");
+                file_result.files.retain(|fm| fm.filepath == target_path);
 
-                    // Not dir and not file, so it is missing
-                    if file_result.files.is_empty() {
-                        return Err(Error::NotFound {
-                            path: path.to_string(),
-                        }
-                        .into());
+                // Not dir and not file, so it is missing
+                if file_result.files.is_empty() {
+                    return Err(Error::NotFound {
+                        path: path.to_string(),
                     }
-
-                    Ok(file_result)
-                } else {
-                    Ok(forced_directory_ls_result)
+                    .into());
                 }
+
+                Ok(file_result)
+            } else {
+                Ok(forced_directory_ls_result)
             }
-            GCSClientWrapper::S3Compat(client) => {
-                client.ls(path, Some(delimiter), continuation_token).await
-            }
+        } else {
+            self._ls_impl(
+                client,
+                bucket,
+                key,
+                None, // Force a prefix-listing
+                continuation_token,
+                page_size,
+                io_stats.as_ref(),
+            )
+            .await
         }
     }
 }
@@ -290,59 +325,81 @@ pub(crate) struct GCSSource {
 }
 
 impl GCSSource {
-    async fn build_s3_compat_client() -> super::Result<Arc<Self>> {
-        let s3_config = common_io_config::S3Config {
-            anonymous: true,
-            endpoint_url: Some("https://storage.googleapis.com".to_string()),
-            ..Default::default()
-        };
-        let s3_client = s3_like::S3LikeSource::get_client(&s3_config).await?;
-        Ok(GCSSource {
-            client: GCSClientWrapper::S3Compat(s3_client),
-        }
-        .into())
-    }
     pub async fn get_client(config: &GCSConfig) -> super::Result<Arc<Self>> {
-        if config.anonymous {
-            GCSSource::build_s3_compat_client().await
-        } else {
-            let config = ClientConfig::default()
+        let config = if !config.anonymous {
+            let attempted = ClientConfig::default()
                 .with_auth()
                 .await
                 .context(UnableToLoadCredentialsSnafu {});
-            match config {
-                Ok(config) => {
-                    let client = Client::new(config);
-                    Ok(GCSSource {
-                        client: GCSClientWrapper::Native(client),
-                    }
-                    .into())
-                }
+
+            match attempted {
+                Ok(attempt) => attempt,
                 Err(err) => {
                     log::warn!("Google Cloud Storage Credentials not provided or found when making client. Reverting to Anonymous mode.\nDetails\n{err}");
-                    GCSSource::build_s3_compat_client().await
+                    ClientConfig::default().anonymous()
                 }
             }
+        } else {
+            ClientConfig::default().anonymous()
+        };
+
+        let client = Client::new(config);
+        Ok(GCSSource {
+            client: GCSClientWrapper(client),
         }
+        .into())
     }
 }
 
 #[async_trait]
 impl ObjectSource for GCSSource {
-    async fn get(&self, uri: &str, range: Option<Range<usize>>) -> super::Result<GetResult> {
-        self.client.get(uri, range).await
+    async fn get(
+        &self,
+        uri: &str,
+        range: Option<Range<usize>>,
+        io_stats: Option<IOStatsRef>,
+    ) -> super::Result<GetResult> {
+        self.client.get(uri, range, io_stats).await
     }
 
-    async fn get_size(&self, uri: &str) -> super::Result<usize> {
-        self.client.get_size(uri).await
+    async fn get_size(&self, uri: &str, io_stats: Option<IOStatsRef>) -> super::Result<usize> {
+        self.client.get_size(uri, io_stats).await
+    }
+
+    async fn glob(
+        self: Arc<Self>,
+        glob_path: &str,
+        fanout_limit: Option<usize>,
+        page_size: Option<i32>,
+        limit: Option<usize>,
+        io_stats: Option<IOStatsRef>,
+    ) -> super::Result<BoxStream<'static, super::Result<FileMetadata>>> {
+        use crate::object_store_glob::glob;
+
+        // Ensure fanout_limit is not None to prevent runaway concurrency
+        let fanout_limit = fanout_limit.or(Some(DEFAULT_GLOB_FANOUT_LIMIT));
+
+        glob(
+            self,
+            glob_path,
+            fanout_limit,
+            page_size.or(Some(1000)),
+            limit,
+            io_stats,
+        )
+        .await
     }
 
     async fn ls(
         &self,
         path: &str,
-        delimiter: Option<&str>,
+        posix: bool,
         continuation_token: Option<&str>,
+        page_size: Option<i32>,
+        io_stats: Option<IOStatsRef>,
     ) -> super::Result<LSResult> {
-        self.client.ls(path, delimiter, continuation_token).await
+        self.client
+            .ls(path, posix, continuation_token, page_size, io_stats)
+            .await
     }
 }
