@@ -1,16 +1,20 @@
 use async_trait::async_trait;
+use aws_config::meta::credentials::CredentialsProviderChain;
 use aws_config::retry::RetryMode;
 use aws_config::timeout::TimeoutConfig;
 use aws_smithy_async::rt::sleep::TokioSleep;
+use futures::stream::BoxStream;
 use reqwest::StatusCode;
 use s3::operation::head_object::HeadObjectError;
 use s3::operation::list_objects_v2::ListObjectsV2Error;
 use tokio::sync::{OwnedSemaphorePermit, SemaphorePermit};
 
 use crate::object_io::{FileMetadata, FileType, LSResult};
-use crate::{InvalidArgumentSnafu, SourceType};
+use crate::stats::IOStatsRef;
+use crate::stream_utils::io_stats_on_bytestream;
+use crate::{get_io_pool_num_threads, InvalidArgumentSnafu, SourceType};
 use aws_config::SdkConfig;
-use aws_credential_types::cache::ProvideCachedCredentials;
+use aws_credential_types::cache::{ProvideCachedCredentials, SharedCredentialsCache};
 use aws_credential_types::provider::error::CredentialsError;
 use aws_sig_auth::signer::SigningRequirements;
 use common_io_config::S3Config;
@@ -20,7 +24,7 @@ use s3::config::{Credentials, Region};
 use s3::error::{DisplayErrorContext, SdkError};
 use s3::operation::get_object::GetObjectError;
 use snafu::{ensure, IntoError, ResultExt, Snafu};
-use url::ParseError;
+use url::{ParseError, Position};
 
 use super::object_io::{GetResult, ObjectSource};
 use async_recursion::async_recursion;
@@ -33,6 +37,9 @@ use std::ops::Range;
 use std::string::FromUtf8Error;
 use std::sync::Arc;
 use std::time::Duration;
+
+const S3_DELIMITER: &str = "/";
+const DEFAULT_GLOB_FANOUT_LIMIT: usize = 1024;
 pub(crate) struct S3LikeSource {
     region_to_client_map: tokio::sync::RwLock<HashMap<Region, Arc<s3::Client>>>,
     connection_pool_sema: Arc<tokio::sync::Semaphore>,
@@ -178,6 +185,28 @@ impl From<Error> for super::Error {
     }
 }
 
+/// Helper to parse S3 URLs, returning (scheme, bucket, key)
+fn parse_url(uri: &str) -> super::Result<(String, String, String)> {
+    let parsed = url::Url::parse(uri).with_context(|_| InvalidUrlSnafu { path: uri })?;
+    let bucket = match parsed.host_str() {
+        Some(s) => Ok(s),
+        None => Err(Error::InvalidUrl {
+            path: uri.into(),
+            source: ParseError::EmptyHost,
+        }),
+    }?;
+
+    // Use raw `uri` for object key: URI special character escaping might mangle key
+    let bucket_scheme_prefix_len = parsed[..Position::AfterHost].len();
+    let key = uri[bucket_scheme_prefix_len..].trim_start_matches(S3_DELIMITER);
+
+    Ok((
+        parsed.scheme().to_string(),
+        bucket.to_string(),
+        key.to_string(),
+    ))
+}
+
 fn handle_https_client_settings(
     builder: aws_sdk_s3::config::Builder,
     config: &S3Config,
@@ -207,7 +236,10 @@ fn handle_https_client_settings(
     Ok(builder)
 }
 
-async fn build_s3_client(config: &S3Config) -> super::Result<(bool, s3::Client)> {
+async fn build_s3_client(
+    config: &S3Config,
+    credentials_cache: Option<SharedCredentialsCache>,
+) -> super::Result<(bool, s3::Client)> {
     const DEFAULT_REGION: Region = Region::from_static("us-east-1");
 
     let mut anonymous = config.anonymous;
@@ -224,8 +256,6 @@ async fn build_s3_client(config: &S3Config) -> super::Result<(bool, s3::Client)>
     };
     let builder = if let Some(region) = &config.region_name {
         builder.region(Region::new(region.to_owned()))
-    } else if conf.region().is_none() && config.region_name.is_none() {
-        builder.region(DEFAULT_REGION)
     } else {
         builder
     };
@@ -264,7 +294,19 @@ async fn build_s3_client(config: &S3Config) -> super::Result<(bool, s3::Client)>
         .build();
     let builder = builder.timeout_config(timeout_config);
 
-    let builder = if config.access_key.is_some() && config.key_id.is_some() {
+    let cached_creds = if let Some(credentials_cache) = credentials_cache {
+        let creds = credentials_cache.provide_cached_credentials().await;
+        creds.ok()
+    } else {
+        None
+    };
+
+    let builder = if let Some(cached_creds) = cached_creds {
+        let provider = CredentialsProviderChain::first_try("different_region_cache", cached_creds)
+            .or_default_provider()
+            .await;
+        builder.credentials_provider(provider)
+    } else if config.access_key.is_some() && config.key_id.is_some() {
         let creds = Credentials::from_keys(
             config.key_id.clone().unwrap(),
             config.access_key.clone().unwrap(),
@@ -279,35 +321,70 @@ async fn build_s3_client(config: &S3Config) -> super::Result<(bool, s3::Client)>
         builder
     };
 
+    let builder_copy = builder.clone();
     let s3_conf = builder.build();
-    if !config.anonymous {
+    const CRED_TRIES: u64 = 4;
+    const JITTER_MS: u64 = 2_500;
+    const MAX_BACKOFF_MS: u64 = 20_000;
+    const MAX_WAITTIME_MS: u64 = 45_000;
+    let check_creds = async || -> super::Result<bool> {
+        use rand::Rng;
         use CredentialsError::*;
-        match s3_conf
-            .credentials_cache()
-            .provide_cached_credentials()
-            .await {
-            Ok(_) => Ok(()),
-            Err(err @ CredentialsNotLoaded(..)) => {
-                log::warn!("S3 Credentials not provided or found when making client for {}! Reverting to Anonymous mode. {err}", s3_conf.region().unwrap_or(&DEFAULT_REGION));
-                anonymous = true;
-                Ok(())
-            },
-            Err(err) => Err(err),
-        }.with_context(|_| UnableToLoadCredentialsSnafu {})?;
+        let mut attempt = 0;
+        let first_attempt_time = std::time::Instant::now();
+        loop {
+            let creds = s3_conf
+                .credentials_cache()
+                .provide_cached_credentials()
+                .await;
+            attempt += 1;
+            match creds {
+                Ok(_) => return Ok(false),
+                Err(err @  ProviderTimedOut(..)) => {
+                    let total_time_waited_ms: u64 = first_attempt_time.elapsed().as_millis().try_into().unwrap();
+                    if attempt < CRED_TRIES && (total_time_waited_ms < MAX_WAITTIME_MS) {
+                        let jitter = rand::thread_rng().gen_range(0..((1<<attempt) * JITTER_MS)) as u64;
+                        let jitter = jitter.min(MAX_BACKOFF_MS);
+                        log::warn!("S3 Credentials Provider timed out when making client for {}! Attempt {attempt} out of {CRED_TRIES} tries. Trying again in {jitter}ms. {err}", s3_conf.region().unwrap_or(&DEFAULT_REGION));
+                        tokio::time::sleep(Duration::from_millis(jitter)).await;
+                        continue;
+                    } else {
+                        Err(err)
+                    }
+                }
+                Err(err @ CredentialsNotLoaded(..)) => {
+                    log::warn!("S3 Credentials not provided or found when making client for {}! Reverting to Anonymous mode. {err}", s3_conf.region().unwrap_or(&DEFAULT_REGION));
+                    return Ok(true)
+                },
+                Err(err) => Err(err),
+            }.with_context(|_| UnableToLoadCredentialsSnafu {})?;
+        }
     };
 
+    if !config.anonymous {
+        anonymous = check_creds().await?;
+    };
+
+    let s3_conf = if s3_conf.region().is_none() {
+        builder_copy.region(DEFAULT_REGION).build()
+    } else {
+        s3_conf
+    };
     Ok((anonymous, s3::Client::from_conf(s3_conf)))
 }
 
 async fn build_client(config: &S3Config) -> super::Result<S3LikeSource> {
-    let (anonymous, client) = build_s3_client(config).await?;
+    let (anonymous, client) = build_s3_client(config, None).await?;
     let mut client_map = HashMap::new();
     let default_region = client.conf().region().unwrap().clone();
     client_map.insert(default_region.clone(), client.into());
     Ok(S3LikeSource {
         region_to_client_map: tokio::sync::RwLock::new(client_map),
         connection_pool_sema: Arc::new(tokio::sync::Semaphore::new(
-            config.max_connections as usize,
+            (config.max_connections_per_io_thread as usize)
+                * get_io_pool_num_threads()
+                    .await
+                    .expect("Should be running in tokio pool"),
         )),
         s3_config: config.clone(),
         default_region,
@@ -336,7 +413,12 @@ impl S3LikeSource {
 
         let mut new_config = self.s3_config.clone();
         new_config.region_name = Some(region.to_string());
-        let (_, new_client) = build_s3_client(&new_config).await?;
+
+        let creds_cache = w_handle
+            .get(&self.default_region)
+            .map(|current_client| current_client.conf().credentials_cache());
+
+        let (_, new_client) = build_s3_client(&new_config, creds_cache).await?;
 
         if w_handle.get(region).is_none() {
             w_handle.insert(region.clone(), new_client.into());
@@ -353,16 +435,11 @@ impl S3LikeSource {
         region: &Region,
     ) -> super::Result<GetResult> {
         log::debug!("S3 get at {uri}, range: {range:?}, in region: {region}");
-        let parsed = url::Url::parse(uri).with_context(|_| InvalidUrlSnafu { path: uri })?;
-        let bucket = match parsed.host_str() {
-            Some(s) => Ok(s),
-            None => Err(Error::InvalidUrl {
-                path: uri.into(),
-                source: ParseError::EmptyHost,
-            }),
-        }?;
-        let key = parsed.path();
-        if let Some(key) = key.strip_prefix('/') {
+        let (_scheme, bucket, key) = parse_url(uri)?;
+
+        if key.is_empty() {
+            Err(Error::NotAFile { path: uri.into() }.into())
+        } else {
             log::debug!("S3 get parsed uri: {uri} into Bucket: {bucket}, Key: {key}");
             let request = self
                 .get_s3_client(region)
@@ -451,8 +528,6 @@ impl S3LikeSource {
                 }
                 Err(err) => Err(UnableToOpenFileSnafu { path: uri }.into_error(err).into()),
             }
-        } else {
-            Err(Error::NotAFile { path: uri.into() }.into())
         }
     }
 
@@ -464,17 +539,11 @@ impl S3LikeSource {
         region: &Region,
     ) -> super::Result<usize> {
         log::debug!("S3 head at {uri} in region: {region}");
-        let parsed = url::Url::parse(uri).with_context(|_| InvalidUrlSnafu { path: uri })?;
+        let (_scheme, bucket, key) = parse_url(uri)?;
 
-        let bucket = match parsed.host_str() {
-            Some(s) => Ok(s),
-            None => Err(Error::InvalidUrl {
-                path: uri.into(),
-                source: ParseError::EmptyHost,
-            }),
-        }?;
-        let key = parsed.path();
-        if let Some(key) = key.strip_prefix('/') {
+        if key.is_empty() {
+            Err(Error::NotAFile { path: uri.into() }.into())
+        } else {
             log::debug!("S3 head parsed uri: {uri} into Bucket: {bucket}, Key: {key}");
             let request = self
                 .get_s3_client(region)
@@ -536,8 +605,6 @@ impl S3LikeSource {
                 }
                 Err(err) => Err(UnableToHeadFileSnafu { path: uri }.into_error(err).into()),
             }
-        } else {
-            Err(Error::NotAFile { path: uri.into() }.into())
         }
     }
 
@@ -549,9 +616,10 @@ impl S3LikeSource {
         scheme: &str,
         bucket: &str,
         key: &str,
-        delimiter: String,
+        delimiter: Option<String>,
         continuation_token: Option<String>,
         region: &Region,
+        page_size: Option<i32>,
     ) -> super::Result<LSResult> {
         log::debug!("S3 list_objects: Bucket: {bucket}, Key: {key}, continuation_token: {continuation_token:?} in region: {region}");
         let request = self
@@ -559,13 +627,22 @@ impl S3LikeSource {
             .await?
             .list_objects_v2()
             .bucket(bucket)
-            .delimiter(&delimiter)
             .prefix(key);
+        let request = if let Some(delimiter) = delimiter.as_ref() {
+            request.delimiter(delimiter)
+        } else {
+            request
+        };
         let request = if let Some(ref continuation_token) = continuation_token {
             request.continuation_token(continuation_token)
         } else {
             request
         };
+        let request = if let Some(page_size) = page_size {
+            request.max_keys(page_size)
+        } else {
+            request
+        };
         let response = if self.anonymous {
             request
                 .customize_middleware()
@@ -660,6 +737,7 @@ impl S3LikeSource {
                             delimiter,
                             continuation_token.clone(),
                             &new_region,
+                            page_size,
                         )
                         .await
                     }
@@ -677,99 +755,177 @@ impl S3LikeSource {
 
 #[async_trait]
 impl ObjectSource for S3LikeSource {
-    async fn get(&self, uri: &str, range: Option<Range<usize>>) -> super::Result<GetResult> {
+    async fn get(
+        &self,
+        uri: &str,
+        range: Option<Range<usize>>,
+        io_stats: Option<IOStatsRef>,
+    ) -> super::Result<GetResult> {
         let permit = self
             .connection_pool_sema
             .clone()
             .acquire_owned()
             .await
             .context(UnableToGrabSemaphoreSnafu)?;
-        self._get_impl(permit, uri, range, &self.default_region)
-            .await
+        let get_result = self
+            ._get_impl(permit, uri, range, &self.default_region)
+            .await?;
+
+        if io_stats.is_some() {
+            if let GetResult::Stream(stream, num_bytes, permit) = get_result {
+                if let Some(is) = io_stats.as_ref() {
+                    is.mark_get_requests(1)
+                }
+                Ok(GetResult::Stream(
+                    io_stats_on_bytestream(stream, io_stats),
+                    num_bytes,
+                    permit,
+                ))
+            } else {
+                panic!("This should always be a stream");
+            }
+        } else {
+            Ok(get_result)
+        }
     }
 
-    async fn get_size(&self, uri: &str) -> super::Result<usize> {
+    async fn get_size(&self, uri: &str, io_stats: Option<IOStatsRef>) -> super::Result<usize> {
         let permit = self
             .connection_pool_sema
             .acquire()
             .await
             .context(UnableToGrabSemaphoreSnafu)?;
-        self._head_impl(permit, uri, &self.default_region).await
+        let head_result = self._head_impl(permit, uri, &self.default_region).await?;
+        if let Some(is) = io_stats.as_ref() {
+            is.mark_head_requests(1)
+        }
+        Ok(head_result)
     }
+
+    async fn glob(
+        self: Arc<Self>,
+        glob_path: &str,
+        fanout_limit: Option<usize>,
+        page_size: Option<i32>,
+        limit: Option<usize>,
+        io_stats: Option<IOStatsRef>,
+    ) -> super::Result<BoxStream<'static, super::Result<FileMetadata>>> {
+        use crate::object_store_glob::glob;
+
+        // Ensure fanout_limit is not None to prevent runaway concurrency
+        let fanout_limit = fanout_limit.or(Some(DEFAULT_GLOB_FANOUT_LIMIT));
+
+        glob(
+            self,
+            glob_path,
+            fanout_limit,
+            page_size.or(Some(1000)),
+            limit,
+            io_stats,
+        )
+        .await
+    }
+
     async fn ls(
         &self,
         path: &str,
-        delimiter: Option<&str>,
+        posix: bool,
         continuation_token: Option<&str>,
+        page_size: Option<i32>,
+        io_stats: Option<IOStatsRef>,
     ) -> super::Result<LSResult> {
-        let parsed = url::Url::parse(path).with_context(|_| InvalidUrlSnafu { path })?;
-        let scheme = parsed.scheme();
-        let delimiter = delimiter.unwrap_or("/");
+        let (scheme, bucket, key) = parse_url(path)?;
 
-        let bucket = match parsed.host_str() {
-            Some(s) => Ok(s),
-            None => Err(Error::InvalidUrl {
-                path: path.into(),
-                source: ParseError::EmptyHost,
-            }),
-        }?;
-        let key = parsed.path();
-        let key = key
-            .trim_start_matches(delimiter)
-            .trim_end_matches(delimiter);
-        let key = if key.is_empty() {
-            "".to_string()
-        } else {
-            format!("{key}{delimiter}")
-        };
+        if posix {
+            // Perform a directory-based list of entries in the next level
+            // assume its a directory first
+            let key = if key.is_empty() {
+                "".to_string()
+            } else {
+                format!("{}{S3_DELIMITER}", key.trim_end_matches(S3_DELIMITER))
+            };
+            let lsr = {
+                let permit = self
+                    .connection_pool_sema
+                    .acquire()
+                    .await
+                    .context(UnableToGrabSemaphoreSnafu)?;
 
-        // assume its a directory first
-        let lsr = {
-            let permit = self
-                .connection_pool_sema
-                .acquire()
-                .await
-                .context(UnableToGrabSemaphoreSnafu)?;
-
-            self._list_impl(
-                permit,
-                scheme,
-                bucket,
-                &key,
-                delimiter.into(),
-                continuation_token.map(String::from),
-                &self.default_region,
-            )
-            .await?
-        };
-        if lsr.files.is_empty() && key.contains(delimiter) {
-            let permit = self
-                .connection_pool_sema
-                .acquire()
-                .await
-                .context(UnableToGrabSemaphoreSnafu)?;
-            // Might be a File
-            let key = key.trim_end_matches(delimiter);
-            let mut lsr = self
-                ._list_impl(
+                self._list_impl(
                     permit,
-                    scheme,
-                    bucket,
-                    key,
-                    delimiter.into(),
+                    scheme.as_str(),
+                    bucket.as_str(),
+                    &key,
+                    Some(S3_DELIMITER.into()),
                     continuation_token.map(String::from),
                     &self.default_region,
+                    page_size,
                 )
-                .await?;
-            let target_path = format!("{scheme}://{bucket}/{key}");
-            lsr.files.retain(|f| f.filepath == target_path);
+                .await?
+            };
+            if let Some(is) = io_stats.as_ref() {
+                is.mark_list_requests(1)
+            }
 
-            if lsr.files.is_empty() {
-                // Isnt a file or a directory
-                return Err(Error::NotFound { path: path.into() }.into());
+            if lsr.files.is_empty() && key.contains(S3_DELIMITER) {
+                let permit = self
+                    .connection_pool_sema
+                    .acquire()
+                    .await
+                    .context(UnableToGrabSemaphoreSnafu)?;
+                // Might be a File
+                let key = key.trim_end_matches(S3_DELIMITER);
+                let mut lsr = self
+                    ._list_impl(
+                        permit,
+                        scheme.as_str(),
+                        bucket.as_str(),
+                        key,
+                        Some(S3_DELIMITER.into()),
+                        continuation_token.map(String::from),
+                        &self.default_region,
+                        page_size,
+                    )
+                    .await?;
+                if let Some(is) = io_stats.as_ref() {
+                    is.mark_list_requests(1)
+                }
+                let target_path = format!("{scheme}://{bucket}/{key}");
+                lsr.files.retain(|f| f.filepath == target_path);
+
+                if lsr.files.is_empty() {
+                    // Isnt a file or a directory
+                    return Err(Error::NotFound { path: path.into() }.into());
+                }
+                Ok(lsr)
+            } else {
+                Ok(lsr)
             }
-            Ok(lsr)
         } else {
+            // Perform a prefix-based list of all entries with this prefix
+            let lsr = {
+                let permit = self
+                    .connection_pool_sema
+                    .acquire()
+                    .await
+                    .context(UnableToGrabSemaphoreSnafu)?;
+
+                self._list_impl(
+                    permit,
+                    scheme.as_str(),
+                    bucket.as_str(),
+                    key.as_str(),
+                    None, // triggers prefix-based list
+                    continuation_token.map(String::from),
+                    &self.default_region,
+                    page_size,
+                )
+                .await?
+            };
+            if let Some(is) = io_stats.as_ref() {
+                is.mark_list_requests(1)
+            }
+
             Ok(lsr)
         }
     }
@@ -793,14 +949,14 @@ mod tests {
             ..Default::default()
         };
         let client = S3LikeSource::get_client(&config).await?;
-        let parquet_file = client.get(parquet_file_path, None).await?;
+        let parquet_file = client.get(parquet_file_path, None, None).await?;
         let bytes = parquet_file.bytes().await?;
         let all_bytes = bytes.as_ref();
         let checksum = format!("{:x}", md5::compute(all_bytes));
         assert_eq!(checksum, parquet_expected_md5);
 
         let first_bytes = client
-            .get_range(parquet_file_path, 0..10)
+            .get_range(parquet_file_path, 0..10, None)
             .await?
             .bytes()
             .await?;
@@ -808,7 +964,7 @@ mod tests {
         assert_eq!(first_bytes.as_ref(), &all_bytes[..10]);
 
         let first_bytes = client
-            .get_range(parquet_file_path, 10..100)
+            .get_range(parquet_file_path, 10..100, None)
             .await?
             .bytes()
             .await?;
@@ -819,6 +975,7 @@ mod tests {
             .get_range(
                 parquet_file_path,
                 (all_bytes.len() - 10)..(all_bytes.len() + 10),
+                None,
             )
             .await?
             .bytes()
@@ -826,7 +983,7 @@ mod tests {
         assert_eq!(last_bytes.len(), 10);
         assert_eq!(last_bytes.as_ref(), &all_bytes[(all_bytes.len() - 10)..]);
 
-        let size_from_get_size = client.get_size(parquet_file_path).await?;
+        let size_from_get_size = client.get_size(parquet_file_path, None).await?;
         assert_eq!(size_from_get_size, all_bytes.len());
 
         Ok(())
@@ -842,7 +999,7 @@ mod tests {
         };
         let client = S3LikeSource::get_client(&config).await?;
 
-        client.ls(file_path, None, None).await?;
+        client.ls(file_path, true, None, None, None).await?;
 
         Ok(())
     }
