@@ -2,8 +2,10 @@ use std::{collections::HashSet, sync::Arc};
 
 use arrow2::io::parquet::read::schema::infer_schema_with_options;
 use common_error::DaftResult;
-use daft_core::{utils::arrow::cast_array_for_daft_if_needed, Series};
-use daft_io::IOClient;
+use daft_core::{schema::Schema, utils::arrow::cast_array_for_daft_if_needed, Series};
+use daft_dsl::ExprRef;
+use daft_io::{IOClient, IOStatsRef};
+use daft_stats::TruthValue;
 use daft_table::Table;
 use futures::{future::try_join_all, StreamExt};
 use parquet2::{
@@ -17,19 +19,21 @@ use crate::{
     metadata::read_parquet_metadata,
     read::ParquetSchemaInferenceOptions,
     read_planner::{CoalescePass, RangesContainer, ReadPlanner, SplitLargeRequestPass},
-    JoinSnafu, OneShotRecvSnafu, UnableToCreateParquetPageStreamSnafu,
-    UnableToParseSchemaFromMetadataSnafu,
+    statistics, JoinSnafu, OneShotRecvSnafu, UnableToConvertRowGroupMetadataToStatsSnafu,
+    UnableToConvertSchemaToDaftSnafu, UnableToCreateParquetPageStreamSnafu,
+    UnableToParseSchemaFromMetadataSnafu, UnableToRunExpressionOnStatsSnafu,
 };
 use arrow2::io::parquet::read::column_iter_to_arrays;
 
 pub(crate) struct ParquetReaderBuilder {
-    uri: String,
-    metadata: parquet2::metadata::FileMetaData,
+    pub uri: String,
+    pub metadata: parquet2::metadata::FileMetaData,
     selected_columns: Option<HashSet<String>>,
     row_start_offset: usize,
-    num_rows: usize,
+    limit: Option<usize>,
     row_groups: Option<Vec<i64>>,
     schema_inference_options: ParquetSchemaInferenceOptions,
+    predicate: Option<ExprRef>,
 }
 use parquet2::read::decompress;
 
@@ -91,20 +95,117 @@ where
     }
 }
 
+pub(crate) fn build_row_ranges(
+    limit: Option<usize>,
+    row_start_offset: usize,
+    row_groups: Option<&[i64]>,
+    predicate: Option<ExprRef>,
+    schema: &Schema,
+    metadata: &parquet2::metadata::FileMetaData,
+    uri: &str,
+) -> super::Result<Vec<RowGroupRange>> {
+    let limit = limit.map(|v| v as i64);
+    let mut row_ranges = vec![];
+    let mut curr_row_index = 0;
+
+    if let Some(row_groups) = row_groups {
+        let mut rows_to_add: i64 = limit.unwrap_or(i64::MAX);
+        for i in row_groups {
+            let i = *i as usize;
+            if !(0..metadata.row_groups.len()).contains(&i) {
+                return Err(super::Error::ParquetRowGroupOutOfIndex {
+                    path: uri.to_string(),
+                    row_group: i as i64,
+                    total_row_groups: metadata.row_groups.len() as i64,
+                });
+            }
+            if rows_to_add <= 0 {
+                break;
+            }
+            let rg = metadata.row_groups.get(i).unwrap();
+            if let Some(ref pred) = predicate {
+                let stats = statistics::row_group_metadata_to_table_stats(rg, schema)
+                    .with_context(|_| UnableToConvertRowGroupMetadataToStatsSnafu {
+                        path: uri.to_string(),
+                    })?;
+
+                let evaled = stats.eval_expression(pred).with_context(|_| {
+                    UnableToRunExpressionOnStatsSnafu {
+                        path: uri.to_string(),
+                    }
+                })?;
+                if evaled.to_truth_value() == TruthValue::False {
+                    continue;
+                }
+            }
+
+            let range_to_add = RowGroupRange {
+                row_group_index: i,
+                start: 0,
+                num_rows: rg.num_rows().min(rows_to_add as usize),
+            };
+            rows_to_add = rows_to_add.saturating_sub((rg.num_rows() as i64).min(rows_to_add));
+            row_ranges.push(range_to_add);
+        }
+    } else {
+        let mut rows_to_add = limit.unwrap_or(metadata.num_rows as i64);
+
+        for (i, rg) in metadata.row_groups.iter().enumerate() {
+            if (curr_row_index + rg.num_rows()) < row_start_offset {
+                curr_row_index += rg.num_rows();
+                continue;
+            } else if rows_to_add > 0 {
+                if let Some(ref pred) = predicate {
+                    let stats = statistics::row_group_metadata_to_table_stats(rg, schema)
+                        .with_context(|_| UnableToConvertRowGroupMetadataToStatsSnafu {
+                            path: uri.to_string(),
+                        })?;
+                    let evaled = stats.eval_expression(pred).with_context(|_| {
+                        UnableToRunExpressionOnStatsSnafu {
+                            path: uri.to_string(),
+                        }
+                    })?;
+                    if evaled.to_truth_value() == TruthValue::False {
+                        curr_row_index += rg.num_rows();
+                        continue;
+                    }
+                }
+                let range_to_add = RowGroupRange {
+                    row_group_index: i,
+                    start: row_start_offset.saturating_sub(curr_row_index),
+                    num_rows: rg.num_rows().min(rows_to_add as usize),
+                };
+                rows_to_add = rows_to_add.saturating_sub((rg.num_rows() as i64).min(rows_to_add));
+                row_ranges.push(range_to_add);
+            } else {
+                break;
+            }
+            curr_row_index += rg.num_rows();
+        }
+    }
+    Ok(row_ranges)
+}
+
 impl ParquetReaderBuilder {
-    pub async fn from_uri(uri: &str, io_client: Arc<daft_io::IOClient>) -> super::Result<Self> {
+    pub async fn from_uri(
+        uri: &str,
+        io_client: Arc<daft_io::IOClient>,
+        io_stats: Option<IOStatsRef>,
+    ) -> super::Result<Self> {
         // TODO(sammy): We actually don't need this since we can do negative offsets when reading the metadata
-        let size = io_client.single_url_get_size(uri.into()).await?;
-        let metadata = read_parquet_metadata(uri, size, io_client).await?;
-        let num_rows = metadata.num_rows;
+        let size = io_client
+            .single_url_get_size(uri.into(), io_stats.clone())
+            .await?;
+        let metadata = read_parquet_metadata(uri, size, io_client, io_stats).await?;
         Ok(ParquetReaderBuilder {
             uri: uri.into(),
             metadata,
             selected_columns: None,
             row_start_offset: 0,
-            num_rows,
+            limit: None,
             row_groups: None,
             schema_inference_options: Default::default(),
+            predicate: None,
         })
     }
 
@@ -116,7 +217,7 @@ impl ParquetReaderBuilder {
         self.metadata().schema()
     }
 
-    pub fn prune_columns(mut self, columns: &[&str]) -> super::Result<Self> {
+    pub fn prune_columns<S: ToString + AsRef<str>>(mut self, columns: &[S]) -> super::Result<Self> {
         let avail_names = self
             .parquet_schema()
             .fields()
@@ -125,7 +226,7 @@ impl ParquetReaderBuilder {
             .collect::<HashSet<_>>();
         let mut names_to_keep = HashSet::new();
         for col_name in columns {
-            if avail_names.contains(col_name) {
+            if avail_names.contains(col_name.as_ref()) {
                 names_to_keep.insert(col_name.to_string());
             } else {
                 return Err(super::Error::FieldNotFound {
@@ -145,9 +246,8 @@ impl ParquetReaderBuilder {
         num_rows: Option<usize>,
     ) -> super::Result<Self> {
         let start_offset = start_offset.unwrap_or(0);
-        let num_rows = num_rows.unwrap_or(self.metadata.num_rows.saturating_sub(start_offset));
         self.row_start_offset = start_offset;
-        self.num_rows = num_rows;
+        self.limit = num_rows;
         Ok(self)
     }
 
@@ -156,81 +256,52 @@ impl ParquetReaderBuilder {
         Ok(self)
     }
 
-    pub fn set_infer_schema_options(mut self, opts: &ParquetSchemaInferenceOptions) -> Self {
-        self.schema_inference_options = opts.clone();
+    pub fn set_infer_schema_options(mut self, opts: ParquetSchemaInferenceOptions) -> Self {
+        self.schema_inference_options = opts;
+        self
+    }
+
+    pub fn set_filter(mut self, predicate: ExprRef) -> Self {
+        assert_eq!(self.limit, None);
+        self.predicate = Some(predicate);
         self
     }
 
     pub fn build(self) -> super::Result<ParquetFileReader> {
-        let mut row_ranges = vec![];
-
-        let mut curr_row_index = 0;
-        let mut rows_to_add = self.num_rows;
-        if let Some(row_groups) = self.row_groups {
-            for i in row_groups {
-                let i = i as usize;
-                if !(0..self.metadata.row_groups.len()).contains(&i) {
-                    return Err(super::Error::ParquetRowGroupOutOfIndex {
-                        path: self.uri,
-                        row_group: i as i64,
-                        total_row_groups: self.metadata.row_groups.len() as i64,
-                    });
-                }
-                let rg = self.metadata.row_groups.get(i).unwrap();
-                let range_to_add = RowGroupRange {
-                    row_group_index: i,
-                    start: 0,
-                    num_rows: rg.num_rows(),
-                };
-                row_ranges.push(range_to_add);
-            }
-        } else {
-            for (i, rg) in self.metadata.row_groups.iter().enumerate() {
-                if (curr_row_index + rg.num_rows()) < self.row_start_offset {
-                    curr_row_index += rg.num_rows();
-                    continue;
-                } else if rows_to_add > 0 {
-                    let range_to_add = RowGroupRange {
-                        row_group_index: i,
-                        start: self.row_start_offset.saturating_sub(curr_row_index),
-                        num_rows: rg.num_rows().min(rows_to_add),
-                    };
-                    rows_to_add = rows_to_add.saturating_sub(rg.num_rows().min(rows_to_add));
-                    row_ranges.push(range_to_add);
-                } else {
-                    break;
-                }
-                curr_row_index += rg.num_rows();
-            }
-        }
-        let mut arrow_schema = infer_schema_with_options(
-            &self.metadata,
-            &Some(arrow2::io::parquet::read::schema::SchemaInferenceOptions {
-                int96_coerce_to_timeunit: self
-                    .schema_inference_options
-                    .coerce_int96_timestamp_unit
-                    .to_arrow(),
-            }),
-        )
-        .context(UnableToParseSchemaFromMetadataSnafu::<String> {
-            path: self.uri.clone(),
-        })?;
+        let mut arrow_schema =
+            infer_schema_with_options(&self.metadata, &Some(self.schema_inference_options.into()))
+                .context(UnableToParseSchemaFromMetadataSnafu::<String> {
+                    path: self.uri.clone(),
+                })?;
 
         if let Some(names_to_keep) = self.selected_columns {
             arrow_schema
                 .fields
                 .retain(|f| names_to_keep.contains(f.name.as_str()));
         }
+        let daft_schema =
+            Schema::try_from(&arrow_schema).with_context(|_| UnableToConvertSchemaToDaftSnafu {
+                path: self.uri.to_string(),
+            })?;
+        let row_ranges = build_row_ranges(
+            self.limit,
+            self.row_start_offset,
+            self.row_groups.as_deref(),
+            self.predicate.clone(),
+            &daft_schema,
+            &self.metadata,
+            &self.uri,
+        )?;
 
         ParquetFileReader::new(self.uri, self.metadata, arrow_schema, row_ranges)
     }
 }
 
 #[derive(Copy, Clone)]
-struct RowGroupRange {
-    row_group_index: usize,
-    start: usize,
-    num_rows: usize,
+pub(crate) struct RowGroupRange {
+    pub row_group_index: usize,
+    pub start: usize,
+    pub num_rows: usize,
 }
 
 pub(crate) struct ParquetFileReader {
@@ -291,7 +362,11 @@ impl ParquetFileReader {
         Ok(read_planner)
     }
 
-    pub fn prebuffer_ranges(&self, io_client: Arc<IOClient>) -> DaftResult<Arc<RangesContainer>> {
+    pub fn prebuffer_ranges(
+        &self,
+        io_client: Arc<IOClient>,
+        io_stats: Option<IOStatsRef>,
+    ) -> DaftResult<Arc<RangesContainer>> {
         let mut read_planner = self.naive_read_plan()?;
         // TODO(sammy) these values should be populated by io_client
         read_planner.add_pass(Box::new(SplitLargeRequestPass {
@@ -305,7 +380,7 @@ impl ParquetFileReader {
         }));
 
         read_planner.run_passes()?;
-        read_planner.collect(io_client)
+        read_planner.collect(io_client, io_stats)
     }
 
     pub async fn read_from_ranges_into_table(
@@ -378,7 +453,10 @@ impl ParquetFileReader {
                                         range_reader,
                                         vec![],
                                         Arc::new(|_, _| true),
-                                        4 * 1024 * 1024,
+                                        // Set to a very high number 256MB to guard against unbounded large
+                                        // downloads from remote storage, which likely indicates corrupted Parquet data
+                                        // See: https://github.com/Eventual-Inc/Daft/issues/1551
+                                        256 * 1024 * 1024,
                                     )
                                     .await
                                     .with_context(|_| {
@@ -440,6 +518,7 @@ impl ParquetFileReader {
                     })
                     .collect::<DaftResult<Vec<_>>>()?;
                 let owned_uri = self.uri.clone();
+                let owned_field = field.clone();
                 let concated_handle = tokio::task::spawn(async move {
                     let series_to_concat =
                         try_join_all(field_handles).await.context(JoinSnafu {
@@ -451,8 +530,14 @@ impl ParquetFileReader {
 
                     let (send, recv) = tokio::sync::oneshot::channel();
                     rayon::spawn(move || {
-                        let concated =
-                            Series::concat(&series_to_concat.iter().flatten().collect::<Vec<_>>());
+                        let concated = if series_to_concat.is_empty() {
+                            Ok(Series::empty(
+                                owned_field.name.as_str(),
+                                &owned_field.data_type().into(),
+                            ))
+                        } else {
+                            Series::concat(&series_to_concat.iter().flatten().collect::<Vec<_>>())
+                        };
                         drop(series_to_concat);
                         let _ = send.send(concated);
                     });
@@ -544,7 +629,10 @@ impl ParquetFileReader {
                                         range_reader,
                                         vec![],
                                         Arc::new(|_, _| true),
-                                        4 * 1024 * 1024,
+                                        // Set to a very high number 256MB to guard against unbounded large
+                                        // downloads from remote storage, which likely indicates corrupted Parquet data
+                                        // See: https://github.com/Eventual-Inc/Daft/issues/1551
+                                        256 * 1024 * 1024,
                                     )
                                     .await
                                     .with_context(|_| {
