@@ -5,7 +5,8 @@ pub mod pylib {
         ffi::field_to_py,
         python::{datatype::PyTimeUnit, schema::PySchema, PySeries},
     };
-    use daft_io::{get_io_client, python::IOConfig};
+    use daft_dsl::python::PyExpr;
+    use daft_io::{get_io_client, python::IOConfig, IOStatsContext};
     use daft_table::python::PyTable;
     use pyo3::{pyfunction, types::PyModule, PyResult, Python};
     use std::{collections::BTreeMap, sync::Arc};
@@ -21,11 +22,14 @@ pub mod pylib {
         start_offset: Option<usize>,
         num_rows: Option<usize>,
         row_groups: Option<Vec<i64>>,
+        predicate: Option<PyExpr>,
         io_config: Option<IOConfig>,
         multithreaded_io: Option<bool>,
         coerce_int96_timestamp_unit: Option<PyTimeUnit>,
     ) -> PyResult<PyTable> {
         py.allow_threads(|| {
+            let io_stats = IOStatsContext::new(format!("read_parquet: for uri {uri}"));
+
             let io_client = get_io_client(
                 multithreaded_io.unwrap_or(true),
                 io_config.unwrap_or_default().config.into(),
@@ -33,17 +37,22 @@ pub mod pylib {
             let schema_infer_options = ParquetSchemaInferenceOptions::new(
                 coerce_int96_timestamp_unit.map(|tu| tu.timeunit),
             );
-            Ok(crate::read::read_parquet(
+            let runtime_handle = daft_io::get_runtime(multithreaded_io.unwrap_or(true))?;
+
+            let result = crate::read::read_parquet(
                 uri,
                 columns.as_deref(),
                 start_offset,
                 num_rows,
-                row_groups.as_deref(),
+                row_groups,
+                predicate.map(|e| e.expr.into()),
                 io_client,
-                multithreaded_io.unwrap_or(true),
-                &schema_infer_options,
+                Some(io_stats.clone()),
+                runtime_handle,
+                schema_infer_options,
             )?
-            .into())
+            .into();
+            Ok(result)
         })
     }
     type PyArrowChunks = Vec<Vec<pyo3::PyObject>>;
@@ -93,19 +102,23 @@ pub mod pylib {
             let schema_infer_options = ParquetSchemaInferenceOptions::new(
                 coerce_int96_timestamp_unit.map(|tu| tu.timeunit),
             );
+
+            let runtime_handle = daft_io::get_runtime(multithreaded_io.unwrap_or(true))?;
+
             crate::read::read_parquet_into_pyarrow(
                 uri,
                 columns.as_deref(),
                 start_offset,
                 num_rows,
-                row_groups.as_deref(),
+                row_groups,
                 io_client,
-                multithreaded_io.unwrap_or(true),
-                &schema_infer_options,
+                None,
+                runtime_handle,
+                schema_infer_options,
             )
         })?;
         let (schema, all_arrays) = read_parquet_result;
-        let pyarrow = py.import("pyarrow")?;
+        let pyarrow = py.import(pyo3::intern!(py, "pyarrow"))?;
         convert_pyarrow_parquet_read_result_into_py(py, schema, all_arrays, pyarrow)
     }
     #[allow(clippy::too_many_arguments)]
@@ -116,13 +129,16 @@ pub mod pylib {
         columns: Option<Vec<&str>>,
         start_offset: Option<usize>,
         num_rows: Option<usize>,
-        row_groups: Option<Vec<Vec<i64>>>,
+        row_groups: Option<Vec<Option<Vec<i64>>>>,
+        predicate: Option<PyExpr>,
         io_config: Option<IOConfig>,
         num_parallel_tasks: Option<i64>,
         multithreaded_io: Option<bool>,
         coerce_int96_timestamp_unit: Option<PyTimeUnit>,
     ) -> PyResult<Vec<PyTable>> {
         py.allow_threads(|| {
+            let io_stats = IOStatsContext::new("read_parquet_bulk");
+
             let io_client = get_io_client(
                 multithreaded_io.unwrap_or(true),
                 io_config.unwrap_or_default().config.into(),
@@ -130,6 +146,7 @@ pub mod pylib {
             let schema_infer_options = ParquetSchemaInferenceOptions::new(
                 coerce_int96_timestamp_unit.map(|tu| tu.timeunit),
             );
+            let runtime_handle = daft_io::get_runtime(multithreaded_io.unwrap_or(true))?;
 
             Ok(crate::read::read_parquet_bulk(
                 uris.as_ref(),
@@ -137,9 +154,11 @@ pub mod pylib {
                 start_offset,
                 num_rows,
                 row_groups,
+                predicate.map(|e| e.expr.into()),
                 io_client,
+                Some(io_stats),
                 num_parallel_tasks.unwrap_or(128) as usize,
-                multithreaded_io.unwrap_or(true),
+                runtime_handle,
                 &schema_infer_options,
             )?
             .into_iter()
@@ -156,7 +175,7 @@ pub mod pylib {
         columns: Option<Vec<&str>>,
         start_offset: Option<usize>,
         num_rows: Option<usize>,
-        row_groups: Option<Vec<Vec<i64>>>,
+        row_groups: Option<Vec<Option<Vec<i64>>>>,
         io_config: Option<IOConfig>,
         num_parallel_tasks: Option<i64>,
         multithreaded_io: Option<bool>,
@@ -178,12 +197,13 @@ pub mod pylib {
                 num_rows,
                 row_groups,
                 io_client,
+                None,
                 num_parallel_tasks.unwrap_or(128) as usize,
                 multithreaded_io.unwrap_or(true),
-                &schema_infer_options,
+                schema_infer_options,
             )
         })?;
-        let pyarrow = py.import("pyarrow")?;
+        let pyarrow = py.import(pyo3::intern!(py, "pyarrow"))?;
         parquet_read_results
             .into_iter()
             .map(|(s, all_arrays)| {
@@ -201,6 +221,8 @@ pub mod pylib {
         coerce_int96_timestamp_unit: Option<PyTimeUnit>,
     ) -> PyResult<PySchema> {
         py.allow_threads(|| {
+            let io_stats = IOStatsContext::new(format!("read_parquet_schema: for uri {uri}"));
+
             let schema_infer_options = ParquetSchemaInferenceOptions::new(
                 coerce_int96_timestamp_unit.map(|tu| tu.timeunit),
             );
@@ -211,7 +233,8 @@ pub mod pylib {
             Ok(Arc::new(crate::read::read_parquet_schema(
                 uri,
                 io_client,
-                &schema_infer_options,
+                Some(io_stats),
+                schema_infer_options,
             )?)
             .into())
         })
@@ -225,11 +248,16 @@ pub mod pylib {
         multithreaded_io: Option<bool>,
     ) -> PyResult<PyTable> {
         py.allow_threads(|| {
+            let io_stats = IOStatsContext::new("read_parquet_statistics");
+
             let io_client = get_io_client(
                 multithreaded_io.unwrap_or(true),
                 io_config.unwrap_or_default().config.into(),
             )?;
-            Ok(crate::read::read_parquet_statistics(&uris.series, io_client)?.into())
+            Ok(
+                crate::read::read_parquet_statistics(&uris.series, io_client, Some(io_stats))?
+                    .into(),
+            )
         })
     }
 }
