@@ -7,14 +7,21 @@ use daft_core::{
     schema::Schema,
     DataType, IntoSeries, Series,
 };
-use daft_io::{get_runtime, IOClient};
+use daft_dsl::{optimization::get_required_columns, ExprRef};
+use daft_io::{get_runtime, parse_url, IOClient, IOStatsRef, SourceType};
 use daft_table::Table;
-use futures::{future::join_all, StreamExt, TryStreamExt};
+use futures::{
+    future::{join_all, try_join_all},
+    StreamExt, TryStreamExt,
+};
+use itertools::Itertools;
 use snafu::ResultExt;
+use tokio::runtime::Runtime;
 
 use crate::{file::ParquetReaderBuilder, JoinSnafu};
+use serde::{Deserialize, Serialize};
 
-#[derive(Clone)]
+#[derive(Clone, Copy, Serialize, Deserialize)]
 pub struct ParquetSchemaInferenceOptions {
     pub coerce_int96_timestamp_unit: TimeUnit,
 }
@@ -38,51 +45,116 @@ impl Default for ParquetSchemaInferenceOptions {
     }
 }
 
+impl From<ParquetSchemaInferenceOptions>
+    for arrow2::io::parquet::read::schema::SchemaInferenceOptions
+{
+    fn from(value: ParquetSchemaInferenceOptions) -> Self {
+        arrow2::io::parquet::read::schema::SchemaInferenceOptions {
+            int96_coerce_to_timeunit: value.coerce_int96_timestamp_unit.to_arrow(),
+        }
+    }
+}
+
+#[allow(clippy::too_many_arguments)]
 async fn read_parquet_single(
     uri: &str,
     columns: Option<&[&str]>,
     start_offset: Option<usize>,
     num_rows: Option<usize>,
-    row_groups: Option<&[i64]>,
+    row_groups: Option<Vec<i64>>,
+    predicate: Option<ExprRef>,
     io_client: Arc<IOClient>,
-    schema_infer_options: &ParquetSchemaInferenceOptions,
+    io_stats: Option<IOStatsRef>,
+    schema_infer_options: ParquetSchemaInferenceOptions,
 ) -> DaftResult<Table> {
-    let builder = ParquetReaderBuilder::from_uri(uri, io_client.clone()).await?;
-    let builder = builder.set_infer_schema_options(schema_infer_options);
-
-    let builder = if let Some(columns) = columns {
-        builder.prune_columns(columns)?
-    } else {
-        builder
-    };
-
-    if row_groups.is_some() && (num_rows.is_some() || start_offset.is_some()) {
-        return Err(common_error::DaftError::ValueError("Both `row_groups` and `num_rows` or `start_offset` is set at the same time. We only support setting one set or the other.".to_string()));
+    let original_columns = columns;
+    let original_num_rows = num_rows;
+    let mut num_rows = num_rows;
+    let mut columns = columns.map(|s| s.iter().map(|s| s.to_string()).collect_vec());
+    let requested_columns = columns.as_ref().map(|v| v.len());
+    if let Some(ref pred) = predicate {
+        if num_rows.is_some() {
+            num_rows = None;
+        }
+        if let Some(req_columns) = columns.as_mut() {
+            let needed_columns = get_required_columns(pred);
+            for c in needed_columns {
+                if !req_columns.contains(&c) {
+                    req_columns.push(c);
+                }
+            }
+        }
     }
-    let builder = builder.limit(start_offset, num_rows)?;
+    let (source_type, fixed_uri) = parse_url(uri)?;
 
-    let rows_per_row_groups = builder
-        .metadata()
+    let (metadata, mut table) = if matches!(source_type, SourceType::File) {
+        crate::stream_reader::local_parquet_read_async(
+            fixed_uri.as_ref(),
+            columns,
+            start_offset,
+            num_rows,
+            row_groups.clone(),
+            predicate.clone(),
+            schema_infer_options,
+        )
+        .await
+    } else {
+        let builder =
+            ParquetReaderBuilder::from_uri(uri, io_client.clone(), io_stats.clone()).await?;
+        let builder = builder.set_infer_schema_options(schema_infer_options);
+
+        let builder = if let Some(columns) = columns.as_ref() {
+            builder.prune_columns(columns.as_slice())?
+        } else {
+            builder
+        };
+
+        if row_groups.is_some() && (num_rows.is_some() || start_offset.is_some()) {
+            return Err(common_error::DaftError::ValueError("Both `row_groups` and `num_rows` or `start_offset` is set at the same time. We only support setting one set or the other.".to_string()));
+        }
+        let builder = builder.limit(start_offset, num_rows)?;
+        let metadata = builder.metadata().clone();
+
+        let builder = if let Some(ref row_groups) = row_groups {
+            builder.set_row_groups(row_groups)?
+        } else {
+            builder
+        };
+
+        let builder = if let Some(ref predicate) = predicate {
+            builder.set_filter(predicate.clone())
+        } else {
+            builder
+        };
+
+        let parquet_reader = builder.build()?;
+        let ranges = parquet_reader.prebuffer_ranges(io_client, io_stats)?;
+        Ok((
+            metadata,
+            parquet_reader.read_from_ranges_into_table(ranges).await?,
+        ))
+    }?;
+
+    let rows_per_row_groups = metadata
         .row_groups
-        .clone()
         .iter()
         .map(|m| m.num_rows())
         .collect::<Vec<_>>();
-    let builder = if let Some(row_groups) = row_groups {
-        builder.set_row_groups(row_groups)?
-    } else {
-        builder
-    };
 
-    let metadata_num_rows = builder.metadata().num_rows;
+    let metadata_num_rows = metadata.num_rows;
 
-    let metadata_num_columns = builder.parquet_schema().fields().len();
+    let metadata_num_columns = metadata.schema().fields().len();
 
-    let parquet_reader = builder.build()?;
-    let ranges = parquet_reader.prebuffer_ranges(io_client)?;
-    let table = parquet_reader.read_from_ranges_into_table(ranges).await?;
-
-    if let Some(row_groups) = row_groups {
+    if let Some(predicate) = predicate {
+        // TODO ideally pipeline this with IO and before concating, rather than after
+        table = table.filter(&[predicate])?;
+        if let Some(oc) = original_columns {
+            table = table.get_columns(oc)?;
+        }
+        if let Some(nr) = original_num_rows {
+            table = table.head(nr)?;
+        }
+    } else if let Some(row_groups) = row_groups {
         let expected_rows: usize = row_groups
             .iter()
             .map(|i| rows_per_row_groups.get(*i as usize).unwrap())
@@ -118,10 +190,10 @@ async fn read_parquet_single(
             }),
             _ => Ok(()),
         }?;
-    };
+    }
 
-    let expected_num_columns = if let Some(columns) = columns {
-        columns.len()
+    let expected_num_columns = if let Some(columns) = requested_columns {
+        columns
     } else {
         metadata_num_columns
     };
@@ -138,56 +210,73 @@ async fn read_parquet_single(
     Ok(table)
 }
 
+#[allow(clippy::too_many_arguments)]
 async fn read_parquet_single_into_arrow(
     uri: &str,
     columns: Option<&[&str]>,
     start_offset: Option<usize>,
     num_rows: Option<usize>,
-    row_groups: Option<&[i64]>,
+    row_groups: Option<Vec<i64>>,
     io_client: Arc<IOClient>,
-    schema_infer_options: &ParquetSchemaInferenceOptions,
-) -> DaftResult<(
-    arrow2::datatypes::SchemaRef,
-    Vec<Vec<Box<dyn arrow2::array::Array>>>,
-)> {
-    let builder = ParquetReaderBuilder::from_uri(uri, io_client.clone()).await?;
-    let builder = builder.set_infer_schema_options(schema_infer_options);
-
-    let builder = if let Some(columns) = columns {
-        builder.prune_columns(columns)?
+    io_stats: Option<IOStatsRef>,
+    schema_infer_options: ParquetSchemaInferenceOptions,
+) -> DaftResult<(arrow2::datatypes::SchemaRef, Vec<ArrowChunk>)> {
+    let (source_type, fixed_uri) = parse_url(uri)?;
+    let (metadata, schema, all_arrays) = if matches!(source_type, SourceType::File) {
+        let (metadata, schema, all_arrays) =
+            crate::stream_reader::local_parquet_read_into_arrow_async(
+                fixed_uri.as_ref(),
+                columns.map(|s| s.iter().map(|s| s.to_string()).collect_vec()),
+                start_offset,
+                num_rows,
+                row_groups.clone(),
+                None,
+                schema_infer_options,
+            )
+            .await?;
+        (metadata, Arc::new(schema), all_arrays)
     } else {
-        builder
+        let builder =
+            ParquetReaderBuilder::from_uri(uri, io_client.clone(), io_stats.clone()).await?;
+        let builder = builder.set_infer_schema_options(schema_infer_options);
+
+        let builder = if let Some(columns) = columns {
+            builder.prune_columns(columns)?
+        } else {
+            builder
+        };
+
+        if row_groups.is_some() && (num_rows.is_some() || start_offset.is_some()) {
+            return Err(common_error::DaftError::ValueError("Both `row_groups` and `num_rows` or `start_offset` is set at the same time. We only support setting one set or the other.".to_string()));
+        }
+        let builder = builder.limit(start_offset, num_rows)?;
+        let metadata = builder.metadata().clone();
+
+        let builder = if let Some(ref row_groups) = row_groups {
+            builder.set_row_groups(row_groups)?
+        } else {
+            builder
+        };
+
+        let parquet_reader = builder.build()?;
+
+        let schema = parquet_reader.arrow_schema().clone();
+        let ranges = parquet_reader.prebuffer_ranges(io_client, io_stats)?;
+        let all_arrays = parquet_reader
+            .read_from_ranges_into_arrow_arrays(ranges)
+            .await?;
+        (metadata, schema, all_arrays)
     };
 
-    if row_groups.is_some() && (num_rows.is_some() || start_offset.is_some()) {
-        return Err(common_error::DaftError::ValueError("Both `row_groups` and `num_rows` or `start_offset` is set at the same time. We only support setting one set or the other.".to_string()));
-    }
-    let builder = builder.limit(start_offset, num_rows)?;
-
-    let rows_per_row_groups = builder
-        .metadata()
+    let rows_per_row_groups = metadata
         .row_groups
-        .clone()
         .iter()
         .map(|m| m.num_rows())
         .collect::<Vec<_>>();
-    let builder = if let Some(row_groups) = row_groups {
-        builder.set_row_groups(row_groups)?
-    } else {
-        builder
-    };
 
-    let metadata_num_rows = builder.metadata().num_rows;
+    let metadata_num_rows = metadata.num_rows;
 
-    let metadata_num_columns = builder.parquet_schema().fields().len();
-
-    let parquet_reader = builder.build()?;
-
-    let schema = parquet_reader.arrow_schema().clone();
-    let ranges = parquet_reader.prebuffer_ranges(io_client)?;
-    let all_arrays = parquet_reader
-        .read_from_ranges_into_arrow_arrays(ranges)
-        .await?;
+    let metadata_num_columns = metadata.schema().fields().len();
 
     let len_per_col = all_arrays
         .iter()
@@ -202,7 +291,7 @@ async fn read_parquet_single_into_arrow(
 
     let table_ncol = all_arrays.len();
 
-    if let Some(row_groups) = row_groups {
+    if let Some(row_groups) = &row_groups {
         let expected_rows: usize = row_groups
             .iter()
             .map(|i| rows_per_row_groups.get(*i as usize).unwrap())
@@ -264,12 +353,13 @@ pub fn read_parquet(
     columns: Option<&[&str]>,
     start_offset: Option<usize>,
     num_rows: Option<usize>,
-    row_groups: Option<&[i64]>,
+    row_groups: Option<Vec<i64>>,
+    predicate: Option<ExprRef>,
     io_client: Arc<IOClient>,
-    multithreaded_io: bool,
-    schema_infer_options: &ParquetSchemaInferenceOptions,
+    io_stats: Option<IOStatsRef>,
+    runtime_handle: Arc<Runtime>,
+    schema_infer_options: ParquetSchemaInferenceOptions,
 ) -> DaftResult<Table> {
-    let runtime_handle = get_runtime(multithreaded_io)?;
     let _rt_guard = runtime_handle.enter();
     runtime_handle.block_on(async {
         read_parquet_single(
@@ -278,7 +368,9 @@ pub fn read_parquet(
             start_offset,
             num_rows,
             row_groups,
+            predicate,
             io_client,
+            io_stats,
             schema_infer_options,
         )
         .await
@@ -292,12 +384,12 @@ pub fn read_parquet_into_pyarrow(
     columns: Option<&[&str]>,
     start_offset: Option<usize>,
     num_rows: Option<usize>,
-    row_groups: Option<&[i64]>,
+    row_groups: Option<Vec<i64>>,
     io_client: Arc<IOClient>,
-    multithreaded_io: bool,
-    schema_infer_options: &ParquetSchemaInferenceOptions,
+    io_stats: Option<IOStatsRef>,
+    runtime_handle: Arc<Runtime>,
+    schema_infer_options: ParquetSchemaInferenceOptions,
 ) -> DaftResult<ParquetPyarrowChunk> {
-    let runtime_handle = get_runtime(multithreaded_io)?;
     let _rt_guard = runtime_handle.enter();
     runtime_handle.block_on(async {
         read_parquet_single_into_arrow(
@@ -307,6 +399,7 @@ pub fn read_parquet_into_pyarrow(
             num_rows,
             row_groups,
             io_client,
+            io_stats,
             schema_infer_options,
         )
         .await
@@ -319,13 +412,14 @@ pub fn read_parquet_bulk(
     columns: Option<&[&str]>,
     start_offset: Option<usize>,
     num_rows: Option<usize>,
-    row_groups: Option<Vec<Vec<i64>>>,
+    row_groups: Option<Vec<Option<Vec<i64>>>>,
+    predicate: Option<ExprRef>,
     io_client: Arc<IOClient>,
+    io_stats: Option<IOStatsRef>,
     num_parallel_tasks: usize,
-    multithreaded_io: bool,
+    runtime_handle: Arc<Runtime>,
     schema_infer_options: &ParquetSchemaInferenceOptions,
 ) -> DaftResult<Vec<Table>> {
-    let runtime_handle = get_runtime(multithreaded_io)?;
     let _rt_guard = runtime_handle.enter();
     let owned_columns = columns.map(|s| s.iter().map(|v| String::from(*v)).collect::<Vec<_>>());
     if let Some(ref row_groups) = row_groups {
@@ -342,13 +436,12 @@ pub fn read_parquet_bulk(
             let task_stream = futures::stream::iter(uris.iter().enumerate().map(|(i, uri)| {
                 let uri = uri.to_string();
                 let owned_columns = owned_columns.clone();
-                let owned_row_group = match &row_groups {
-                    None => None,
-                    Some(v) => v.get(i).cloned(),
-                };
+                let owned_row_group = row_groups.as_ref().and_then(|rgs| rgs[i].clone());
+                let owned_predicate = predicate.clone();
 
                 let io_client = io_client.clone();
-                let schema_infer_options = schema_infer_options.clone();
+                let io_stats = io_stats.clone();
+                let schema_infer_options = *schema_infer_options;
                 tokio::task::spawn(async move {
                     let columns = owned_columns
                         .as_ref()
@@ -360,9 +453,11 @@ pub fn read_parquet_bulk(
                             columns.as_deref(),
                             start_offset,
                             num_rows,
-                            owned_row_group.as_deref(),
+                            owned_row_group,
+                            owned_predicate,
                             io_client,
-                            &schema_infer_options,
+                            io_stats,
+                            schema_infer_options,
                         )
                         .await?,
                     ))
@@ -386,11 +481,12 @@ pub fn read_parquet_into_pyarrow_bulk(
     columns: Option<&[&str]>,
     start_offset: Option<usize>,
     num_rows: Option<usize>,
-    row_groups: Option<Vec<Vec<i64>>>,
+    row_groups: Option<Vec<Option<Vec<i64>>>>,
     io_client: Arc<IOClient>,
+    io_stats: Option<IOStatsRef>,
     num_parallel_tasks: usize,
     multithreaded_io: bool,
-    schema_infer_options: &ParquetSchemaInferenceOptions,
+    schema_infer_options: ParquetSchemaInferenceOptions,
 ) -> DaftResult<Vec<ParquetPyarrowChunk>> {
     let runtime_handle = get_runtime(multithreaded_io)?;
     let _rt_guard = runtime_handle.enter();
@@ -409,13 +505,12 @@ pub fn read_parquet_into_pyarrow_bulk(
             futures::stream::iter(uris.iter().enumerate().map(|(i, uri)| {
                 let uri = uri.to_string();
                 let owned_columns = owned_columns.clone();
-                let owned_row_group = match &row_groups {
-                    None => None,
-                    Some(v) => v.get(i).cloned(),
-                };
+                let schema_infer_options = schema_infer_options;
+                let owned_row_group = row_groups.as_ref().and_then(|rgs| rgs[i].clone());
 
                 let io_client = io_client.clone();
-                let schema_infer_options = schema_infer_options.clone();
+                let io_stats = io_stats.clone();
+
                 tokio::task::spawn(async move {
                     let columns = owned_columns
                         .as_ref()
@@ -427,9 +522,10 @@ pub fn read_parquet_into_pyarrow_bulk(
                             columns.as_deref(),
                             start_offset,
                             num_rows,
-                            owned_row_group.as_deref(),
+                            owned_row_group,
                             io_client,
-                            &schema_infer_options,
+                            io_stats,
+                            schema_infer_options,
                         )
                         .await?,
                     ))
@@ -448,18 +544,51 @@ pub fn read_parquet_into_pyarrow_bulk(
 pub fn read_parquet_schema(
     uri: &str,
     io_client: Arc<IOClient>,
-    schema_inference_options: &ParquetSchemaInferenceOptions,
+    io_stats: Option<IOStatsRef>,
+    schema_inference_options: ParquetSchemaInferenceOptions,
 ) -> DaftResult<Schema> {
     let runtime_handle = get_runtime(true)?;
     let _rt_guard = runtime_handle.enter();
-    let builder = runtime_handle
-        .block_on(async { ParquetReaderBuilder::from_uri(uri, io_client.clone()).await })?;
+    let builder = runtime_handle.block_on(async {
+        ParquetReaderBuilder::from_uri(uri, io_client.clone(), io_stats).await
+    })?;
     let builder = builder.set_infer_schema_options(schema_inference_options);
 
     Schema::try_from(builder.build()?.arrow_schema().as_ref())
 }
 
-pub fn read_parquet_statistics(uris: &Series, io_client: Arc<IOClient>) -> DaftResult<Table> {
+pub async fn read_parquet_metadata(
+    uri: &str,
+    io_client: Arc<IOClient>,
+    io_stats: Option<IOStatsRef>,
+) -> DaftResult<parquet2::metadata::FileMetaData> {
+    let builder = ParquetReaderBuilder::from_uri(uri, io_client, io_stats).await?;
+    Ok(builder.metadata)
+}
+pub async fn read_parquet_metadata_bulk(
+    uris: &[&str],
+    io_client: Arc<IOClient>,
+    io_stats: Option<IOStatsRef>,
+) -> DaftResult<Vec<parquet2::metadata::FileMetaData>> {
+    let handles_iter = uris.iter().map(|uri| {
+        let owned_string = uri.to_string();
+        let owned_client = io_client.clone();
+        let owned_io_stats = io_stats.clone();
+        tokio::spawn(async move {
+            read_parquet_metadata(&owned_string, owned_client, owned_io_stats).await
+        })
+    });
+    let all_metadatas = try_join_all(handles_iter)
+        .await
+        .context(JoinSnafu { path: "BULK READ" })?;
+    all_metadatas.into_iter().collect::<DaftResult<Vec<_>>>()
+}
+
+pub fn read_parquet_statistics(
+    uris: &Series,
+    io_client: Arc<IOClient>,
+    io_stats: Option<IOStatsRef>,
+) -> DaftResult<Table> {
     let runtime_handle = get_runtime(true)?;
     let _rt_guard = runtime_handle.enter();
 
@@ -477,12 +606,14 @@ pub fn read_parquet_statistics(uris: &Series, io_client: Arc<IOClient>) -> DaftR
     let handles_iter = values.iter().map(|uri| {
         let owned_string = uri.map(|v| v.to_string());
         let owned_client = io_client.clone();
+        let io_stats = io_stats.clone();
+
         tokio::spawn(async move {
             if let Some(owned_string) = owned_string {
-                let builder = ParquetReaderBuilder::from_uri(&owned_string, owned_client).await?;
-                let num_rows = builder.metadata().num_rows;
-                let num_row_groups = builder.metadata().row_groups.len();
-                let version_num = builder.metadata().version;
+                let metadata = read_parquet_metadata(&owned_string, owned_client, io_stats).await?;
+                let num_rows = metadata.num_rows;
+                let num_row_groups = metadata.row_groups.len();
+                let version_num = metadata.version;
 
                 Ok((Some(num_rows), Some(num_row_groups), Some(version_num)))
             } else {
@@ -547,6 +678,7 @@ mod tests {
         io_config.s3.anonymous = true;
 
         let io_client = Arc::new(IOClient::new(io_config.into())?);
+        let runtime_handle = daft_io::get_runtime(true)?;
 
         let table = read_parquet(
             file,
@@ -554,9 +686,11 @@ mod tests {
             None,
             None,
             None,
+            None,
             io_client,
-            true,
-            &Default::default(),
+            None,
+            runtime_handle,
+            Default::default(),
         )?;
         assert_eq!(table.len(), 100);
 
