@@ -7,24 +7,29 @@ use crate::{
     planner::plan,
     sink_info::{OutputFileInfo, SinkInfo},
     source_info::{
-        ExternalInfo as ExternalSourceInfo, FileFormatConfig, FileInfos as InputFileInfos,
-        PyStorageConfig, SourceInfo, StorageConfig,
+        ExternalInfo as ExternalSourceInfo, FileInfos as InputFileInfos, LegacyExternalInfo,
+        SourceInfo,
     },
-    FileFormat, JoinType, PartitionScheme, PartitionSpec, PhysicalPlanScheduler, ResourceRequest,
+    JoinType, PartitionScheme, PhysicalPlanScheduler, ResourceRequest,
 };
 use common_error::{DaftError, DaftResult};
+use common_io_config::IOConfig;
 use daft_core::schema::SchemaRef;
 use daft_core::{datatypes::Field, schema::Schema, DataType};
 use daft_dsl::Expr;
+use daft_scan::{
+    file_format::{FileFormat, FileFormatConfig},
+    storage_config::{PyStorageConfig, StorageConfig},
+    Pushdowns, ScanExternalInfo, ScanOperatorRef,
+};
 
 #[cfg(feature = "python")]
 use {
-    crate::{
-        physical_plan::PhysicalPlan,
-        source_info::{InMemoryInfo, PyFileFormatConfig},
-    },
+    crate::{physical_plan::PhysicalPlan, source_info::InMemoryInfo},
+    common_daft_config::PyDaftExecutionConfig,
     daft_core::python::schema::PySchema,
     daft_dsl::python::PyExpr,
+    daft_scan::{file_format::PyFileFormatConfig, python::pylib::ScanOperatorHandle},
     pyo3::prelude::*,
 };
 
@@ -51,20 +56,36 @@ impl LogicalPlanBuilder {
         partition_key: &str,
         cache_entry: PyObject,
         schema: Arc<Schema>,
-        partition_spec: PartitionSpec,
+        num_partitions: usize,
+        size_bytes: usize,
     ) -> DaftResult<Self> {
         let source_info = SourceInfo::InMemoryInfo(InMemoryInfo::new(
             schema.clone(),
             partition_key.into(),
             cache_entry,
+            num_partitions,
+            size_bytes,
         ));
-        let logical_plan: LogicalPlan = logical_ops::Source::new(
-            schema.clone(),
-            source_info.into(),
-            partition_spec.clone().into(),
-            None,
-        )
-        .into();
+        let logical_plan: LogicalPlan =
+            logical_ops::Source::new(schema.clone(), source_info.into()).into();
+        Ok(logical_plan.into())
+    }
+
+    pub fn table_scan_with_scan_operator(
+        scan_operator: ScanOperatorRef,
+        pushdowns: Option<Pushdowns>,
+    ) -> DaftResult<Self> {
+        let schema = scan_operator.0.schema();
+        let partitioning_keys = scan_operator.0.partitioning_keys();
+        let source_info =
+            SourceInfo::ExternalInfo(ExternalSourceInfo::Scan(ScanExternalInfo::new(
+                scan_operator.clone(),
+                schema.clone(),
+                partitioning_keys.into(),
+                pushdowns.unwrap_or_default(),
+            )));
+        let logical_plan: LogicalPlan =
+            logical_ops::Source::new(schema.clone(), source_info.into()).into();
         Ok(logical_plan.into())
     }
 
@@ -74,32 +95,32 @@ impl LogicalPlanBuilder {
         file_format_config: Arc<FileFormatConfig>,
         storage_config: Arc<StorageConfig>,
     ) -> DaftResult<Self> {
-        Self::table_scan_with_limit(file_infos, schema, file_format_config, storage_config, None)
+        Self::table_scan_with_pushdowns(
+            file_infos,
+            schema,
+            file_format_config,
+            storage_config,
+            Default::default(),
+        )
     }
 
-    pub fn table_scan_with_limit(
+    pub fn table_scan_with_pushdowns(
         file_infos: InputFileInfos,
         schema: Arc<Schema>,
         file_format_config: Arc<FileFormatConfig>,
         storage_config: Arc<StorageConfig>,
-        limit: Option<usize>,
+        pushdowns: Pushdowns,
     ) -> DaftResult<Self> {
-        let num_partitions = file_infos.len();
-        let source_info = SourceInfo::ExternalInfo(ExternalSourceInfo::new(
-            schema.clone(),
-            file_infos.into(),
-            file_format_config,
-            storage_config,
-        ));
-        let partition_spec =
-            PartitionSpec::new_internal(PartitionScheme::Unknown, num_partitions, None);
-        let logical_plan: LogicalPlan = logical_ops::Source::new(
-            schema.clone(),
-            source_info.into(),
-            partition_spec.into(),
-            limit,
-        )
-        .into();
+        let source_info =
+            SourceInfo::ExternalInfo(ExternalSourceInfo::Legacy(LegacyExternalInfo::new(
+                schema.clone(),
+                file_infos.into(),
+                file_format_config,
+                storage_config,
+                pushdowns,
+            )));
+        let logical_plan: LogicalPlan =
+            logical_ops::Source::new(schema.clone(), source_info.into()).into();
         Ok(logical_plan.into())
     }
 
@@ -119,8 +140,9 @@ impl LogicalPlanBuilder {
         Ok(logical_plan.into())
     }
 
-    pub fn limit(&self, limit: i64) -> DaftResult<Self> {
-        let logical_plan: LogicalPlan = logical_ops::Limit::new(self.plan.clone(), limit).into();
+    pub fn limit(&self, limit: i64, eager: bool) -> DaftResult<Self> {
+        let logical_plan: LogicalPlan =
+            logical_ops::Limit::new(self.plan.clone(), limit, eager).into();
         Ok(logical_plan.into())
     }
 
@@ -138,19 +160,17 @@ impl LogicalPlanBuilder {
 
     pub fn repartition(
         &self,
-        num_partitions: usize,
+        num_partitions: Option<usize>,
         partition_by: Vec<Expr>,
         scheme: PartitionScheme,
     ) -> DaftResult<Self> {
-        let logical_plan: LogicalPlan =
-            logical_ops::Repartition::new(self.plan.clone(), num_partitions, partition_by, scheme)
-                .into();
-        Ok(logical_plan.into())
-    }
-
-    pub fn coalesce(&self, num_partitions: usize) -> DaftResult<Self> {
-        let logical_plan: LogicalPlan =
-            logical_ops::Coalesce::new(self.plan.clone(), num_partitions).into();
+        let logical_plan: LogicalPlan = logical_ops::Repartition::try_new(
+            self.plan.clone(),
+            num_partitions,
+            partition_by,
+            scheme,
+        )?
+        .into();
         Ok(logical_plan.into())
     }
 
@@ -159,6 +179,17 @@ impl LogicalPlanBuilder {
         Ok(logical_plan.into())
     }
 
+    pub fn sample(
+        &self,
+        fraction: f64,
+        with_replacement: bool,
+        seed: Option<u64>,
+    ) -> DaftResult<Self> {
+        let logical_plan: LogicalPlan =
+            logical_ops::Sample::new(self.plan.clone(), fraction, with_replacement, seed).into();
+        Ok(logical_plan.into())
+    }
+
     pub fn aggregate(&self, agg_exprs: Vec<Expr>, groupby_exprs: Vec<Expr>) -> DaftResult<Self> {
         let agg_exprs = agg_exprs
             .iter()
@@ -205,12 +236,14 @@ impl LogicalPlanBuilder {
         file_format: FileFormat,
         partition_cols: Option<Vec<Expr>>,
         compression: Option<String>,
+        io_config: Option<IOConfig>,
     ) -> DaftResult<Self> {
         let sink_info = SinkInfo::OutputFileInfo(OutputFileInfo::new(
             root_dir.into(),
             file_format,
             partition_cols,
             compression,
+            io_config,
         ));
         let fields = vec![Field::new("path", DataType::Utf8)];
         let logical_plan: LogicalPlan = logical_ops::Sink::new(
@@ -230,10 +263,6 @@ impl LogicalPlanBuilder {
         self.plan.schema()
     }
 
-    pub fn partition_spec(&self) -> PartitionSpec {
-        self.plan.partition_spec().as_ref().clone()
-    }
-
     pub fn repr_ascii(&self, simple: bool) -> String {
         self.plan.repr_ascii(simple)
     }
@@ -271,17 +300,24 @@ impl PyLogicalPlanBuilder {
         partition_key: &str,
         cache_entry: &PyAny,
         schema: PySchema,
-        partition_spec: PartitionSpec,
+        num_partitions: usize,
+        size_bytes: usize,
     ) -> PyResult<Self> {
         Ok(LogicalPlanBuilder::in_memory_scan(
             partition_key,
             cache_entry.to_object(cache_entry.py()),
             schema.into(),
-            partition_spec,
+            num_partitions,
+            size_bytes,
         )?
         .into())
     }
 
+    #[staticmethod]
+    pub fn table_scan_with_scan_operator(scan_operator: ScanOperatorHandle) -> PyResult<Self> {
+        Ok(LogicalPlanBuilder::table_scan_with_scan_operator(scan_operator.into(), None)?.into())
+    }
+
     #[staticmethod]
     pub fn table_scan(
         file_infos: InputFileInfos,
@@ -317,8 +353,8 @@ impl PyLogicalPlanBuilder {
         Ok(self.builder.filter(predicate.expr)?.into())
     }
 
-    pub fn limit(&self, limit: i64) -> PyResult<Self> {
-        Ok(self.builder.limit(limit)?.into())
+    pub fn limit(&self, limit: i64, eager: bool) -> PyResult<Self> {
+        Ok(self.builder.limit(limit, eager)?.into())
     }
 
     pub fn explode(&self, to_explode: Vec<PyExpr>) -> PyResult<Self> {
@@ -336,9 +372,9 @@ impl PyLogicalPlanBuilder {
 
     pub fn repartition(
         &self,
-        num_partitions: usize,
         partition_by: Vec<PyExpr>,
         scheme: PartitionScheme,
+        num_partitions: Option<usize>,
     ) -> PyResult<Self> {
         let partition_by_exprs: Vec<Expr> = partition_by
             .iter()
@@ -350,14 +386,22 @@ impl PyLogicalPlanBuilder {
             .into())
     }
 
-    pub fn coalesce(&self, num_partitions: usize) -> PyResult<Self> {
-        Ok(self.builder.coalesce(num_partitions)?.into())
-    }
-
     pub fn distinct(&self) -> PyResult<Self> {
         Ok(self.builder.distinct()?.into())
     }
 
+    pub fn sample(
+        &self,
+        fraction: f64,
+        with_replacement: bool,
+        seed: Option<u64>,
+    ) -> PyResult<Self> {
+        Ok(self
+            .builder
+            .sample(fraction, with_replacement, seed)?
+            .into())
+    }
+
     pub fn aggregate(&self, agg_exprs: Vec<PyExpr>, groupby_exprs: Vec<PyExpr>) -> PyResult<Self> {
         let agg_exprs = agg_exprs
             .iter()
@@ -401,12 +445,19 @@ impl PyLogicalPlanBuilder {
         file_format: FileFormat,
         partition_cols: Option<Vec<PyExpr>>,
         compression: Option<String>,
+        io_config: Option<common_io_config::python::IOConfig>,
     ) -> PyResult<Self> {
         let partition_cols =
             partition_cols.map(|cols| cols.iter().map(|e| e.clone().into()).collect::<Vec<Expr>>());
         Ok(self
             .builder
-            .table_write(root_dir, file_format, partition_cols, compression)?
+            .table_write(
+                root_dir,
+                file_format,
+                partition_cols,
+                compression,
+                io_config.map(|cfg| cfg.config),
+            )?
             .into())
     }
 
@@ -414,10 +465,6 @@ impl PyLogicalPlanBuilder {
         Ok(self.builder.schema().into())
     }
 
-    pub fn partition_spec(&self) -> PyResult<PartitionSpec> {
-        Ok(self.builder.partition_spec())
-    }
-
     /// Optimize the underlying logical plan, returning a new plan builder containing the optimized plan.
     pub fn optimize(&self) -> PyResult<Self> {
         let optimizer = Optimizer::new(Default::default());
@@ -450,10 +497,17 @@ impl PyLogicalPlanBuilder {
     /// Finalize the logical plan, translate the logical plan to a physical plan, and return
     /// a physical plan scheduler that's capable of launching the work necessary to compute the output
     /// of the physical plan.
-    pub fn to_physical_plan_scheduler(&self) -> PyResult<PhysicalPlanScheduler> {
-        let logical_plan = self.builder.build();
-        let physical_plan: Arc<PhysicalPlan> = plan(logical_plan.as_ref())?.into();
-        Ok(physical_plan.into())
+    pub fn to_physical_plan_scheduler(
+        &self,
+        py: Python,
+        cfg: PyDaftExecutionConfig,
+    ) -> PyResult<PhysicalPlanScheduler> {
+        py.allow_threads(|| {
+            let logical_plan = self.builder.build();
+            let physical_plan: Arc<PhysicalPlan> =
+                plan(logical_plan.as_ref(), cfg.config.clone())?.into();
+            Ok(physical_plan.into())
+        })
     }
 
     pub fn repr_ascii(&self, simple: bool) -> PyResult<String> {
