@@ -1,5 +1,4 @@
 mod agg;
-mod coalesce;
 mod concat;
 mod distinct;
 mod explode;
@@ -8,12 +7,12 @@ mod join;
 mod limit;
 mod project;
 mod repartition;
+mod sample;
 mod sink;
 mod sort;
 mod source;
 
 pub use agg::Aggregate;
-pub use coalesce::Coalesce;
 pub use concat::Concat;
 pub use distinct::Distinct;
 pub use explode::Explode;
@@ -22,6 +21,7 @@ pub use join::Join;
 pub use limit::Limit;
 pub use project::Project;
 pub use repartition::Repartition;
+pub use sample::Sample;
 pub use sink::Sink;
 pub use sort::Sort;
 pub use source::Source;
