@@ -8,6 +8,7 @@ use indexmap::IndexSet;
 
 use crate::{
     logical_ops::{Aggregate, Project, Source},
+    source_info::SourceInfo,
     LogicalPlan, ResourceRequest,
 };
 
@@ -144,31 +145,38 @@ impl PushDownProjection {
                 let [required_columns] = &plan.required_columns()[..] else {
                     panic!()
                 };
-                if required_columns.len() < upstream_schema.names().len() {
-                    let pruned_upstream_schema = upstream_schema
-                        .fields
-                        .iter()
-                        .filter_map(|(name, field)| {
-                            required_columns.contains(name).then(|| field.clone())
-                        })
-                        .collect::<Vec<_>>();
-                    let schema = Schema::new(pruned_upstream_schema)?;
-                    let new_source: LogicalPlan = Source::new(
-                        schema.into(),
-                        source.source_info.clone(),
-                        source.partition_spec.clone(),
-                        source.limit,
-                    )
-                    .into();
-
-                    let new_plan = plan.with_new_children(&[new_source.into()]);
-                    // Retry optimization now that the upstream node is different.
-                    let new_plan = self
-                        .try_optimize(new_plan.clone())?
-                        .or(Transformed::Yes(new_plan));
-                    Ok(new_plan)
-                } else {
-                    Ok(Transformed::No(plan))
+                match source.source_info.as_ref() {
+                    SourceInfo::ExternalInfo(external_info) => {
+                        if required_columns.len() < upstream_schema.names().len() {
+                            let pruned_upstream_schema = upstream_schema
+                                .fields
+                                .iter()
+                                .filter_map(|(name, field)| {
+                                    required_columns.contains(name).then(|| field.clone())
+                                })
+                                .collect::<Vec<_>>();
+                            let schema = Schema::new(pruned_upstream_schema)?;
+                            let new_source: LogicalPlan = Source::new(
+                                schema.into(),
+                                Arc::new(SourceInfo::ExternalInfo(external_info.with_pushdowns(
+                                    external_info.pushdowns().with_columns(Some(Arc::new(
+                                        required_columns.iter().cloned().collect(),
+                                    ))),
+                                ))),
+                            )
+                            .into();
+                            let new_plan = plan.with_new_children(&[new_source.into()]);
+                            // Retry optimization now that the upstream node is different.
+                            let new_plan = self
+                                .try_optimize(new_plan.clone())?
+                                .or(Transformed::Yes(new_plan));
+                            Ok(new_plan)
+                        } else {
+                            Ok(Transformed::No(plan))
+                        }
+                    }
+                    #[cfg(feature = "python")]
+                    SourceInfo::InMemoryInfo(_) => Ok(Transformed::No(plan)),
                 }
             }
             LogicalPlan::Project(upstream_projection) => {
@@ -235,9 +243,9 @@ impl PushDownProjection {
             }
             LogicalPlan::Sort(..)
             | LogicalPlan::Repartition(..)
-            | LogicalPlan::Coalesce(..)
             | LogicalPlan::Limit(..)
             | LogicalPlan::Filter(..)
+            | LogicalPlan::Sample(..)
             | LogicalPlan::Explode(..) => {
                 // Get required columns from projection and upstream.
                 let combined_dependencies = plan
@@ -494,7 +502,7 @@ mod tests {
 
     use common_error::DaftResult;
     use daft_core::{datatypes::Field, DataType};
-    use daft_dsl::{binary_op, col, lit, Operator};
+    use daft_dsl::{col, lit};
 
     use crate::{
         optimization::{
@@ -565,8 +573,8 @@ mod tests {
         .build();
 
         let expected = "\
-        Project: [col(a) + lit(1)] + lit(3), col(b) + lit(2), col(a) + lit(4), Partition spec = PartitionSpec { scheme: Unknown, num_partitions: 1, by: None }\
-        \n  Source: Json, File paths = [/foo], File schema = a (Int64), b (Int64), Format-specific config = Json(JsonSourceConfig), Storage config = Native(NativeStorageConfig { io_config: None }), Output schema = a (Int64), b (Int64)";
+        Project: [col(a) + lit(1)] + lit(3), col(b) + lit(2), col(a) + lit(4)\
+        \n  Source: Json, File paths = [/foo], File schema = a (Int64), b (Int64), Format-specific config = Json(JsonSourceConfig { buffer_size: None, chunk_size: None }), Storage config = Native(NativeStorageConfig { io_config: None, multithreaded_io: true }), Output schema = a (Int64), b (Int64)";
         assert_optimized_plan_eq(unoptimized, expected)?;
         Ok(())
     }
@@ -582,7 +590,7 @@ mod tests {
         .build();
 
         let expected = "\
-        Source: Json, File paths = [/foo], File schema = a (Int64), b (Int64), Format-specific config = Json(JsonSourceConfig), Storage config = Native(NativeStorageConfig { io_config: None }), Output schema = a (Int64), b (Int64)";
+        Source: Json, File paths = [/foo], File schema = a (Int64), b (Int64), Format-specific config = Json(JsonSourceConfig { buffer_size: None, chunk_size: None }), Storage config = Native(NativeStorageConfig { io_config: None, multithreaded_io: true }), Output schema = a (Int64), b (Int64)";
         assert_optimized_plan_eq(unoptimized, expected)?;
 
         Ok(())
@@ -598,8 +606,8 @@ mod tests {
         .build();
 
         let expected = "\
-        Project: col(b), col(a), Partition spec = PartitionSpec { scheme: Unknown, num_partitions: 1, by: None }\
-        \n  Source: Json, File paths = [/foo], File schema = a (Int64), b (Int64), Format-specific config = Json(JsonSourceConfig), Storage config = Native(NativeStorageConfig { io_config: None }), Output schema = a (Int64), b (Int64)";
+        Project: col(b), col(a)\
+        \n  Source: Json, File paths = [/foo], File schema = a (Int64), b (Int64), Format-specific config = Json(JsonSourceConfig { buffer_size: None, chunk_size: None }), Storage config = Native(NativeStorageConfig { io_config: None, multithreaded_io: true }), Output schema = a (Int64), b (Int64)";
         assert_optimized_plan_eq(unoptimized, expected)?;
 
         Ok(())
@@ -616,8 +624,8 @@ mod tests {
         .build();
 
         let expected = "\
-        Project: col(b) + lit(3), Partition spec = PartitionSpec { scheme: Unknown, num_partitions: 1, by: None }\
-        \n  Source: Json, File paths = [/foo], File schema = a (Int64), b (Int64), Format-specific config = Json(JsonSourceConfig), Storage config = Native(NativeStorageConfig { io_config: None }), Output schema = b (Int64)";
+        Project: col(b) + lit(3)\
+        \n  Source: Json, File paths = [/foo], File schema = a (Int64), b (Int64), Format-specific config = Json(JsonSourceConfig { buffer_size: None, chunk_size: None }), Storage config = Native(NativeStorageConfig { io_config: None, multithreaded_io: true }), Projection pushdown = [b], Output schema = b (Int64)";
         assert_optimized_plan_eq(unoptimized, expected)?;
 
         Ok(())
@@ -641,9 +649,9 @@ mod tests {
         .build();
 
         let expected = "\
-        Project: col(a), col(b), col(b) AS c, Partition spec = PartitionSpec { scheme: Unknown, num_partitions: 1, by: None }\
-        \n  Project: col(b) + lit(3), col(a), Partition spec = PartitionSpec { scheme: Unknown, num_partitions: 1, by: None }\
-        \n    Source: Json, File paths = [/foo], File schema = a (Int64), b (Int64), Format-specific config = Json(JsonSourceConfig), Storage config = Native(NativeStorageConfig { io_config: None }), Output schema = a (Int64), b (Int64)";
+        Project: col(a), col(b), col(b) AS c\
+        \n  Project: col(b) + lit(3), col(a)\
+        \n    Source: Json, File paths = [/foo], File schema = a (Int64), b (Int64), Format-specific config = Json(JsonSourceConfig { buffer_size: None, chunk_size: None }), Storage config = Native(NativeStorageConfig { io_config: None, multithreaded_io: true }), Output schema = a (Int64), b (Int64)";
         assert_optimized_plan_eq(unoptimized, expected)?;
 
         Ok(())
@@ -662,9 +670,9 @@ mod tests {
         .build();
 
         let expected = "\
-        Project: col(a), Partition spec = PartitionSpec { scheme: Unknown, num_partitions: 1, by: None }\
+        Project: col(a)\
         \n  Aggregation: mean(col(a)), Group by = col(c), Output schema = c (Int64), a (Float64)\
-        \n    Source: Json, File paths = [/foo], File schema = a (Int64), b (Int64), c (Int64), Format-specific config = Json(JsonSourceConfig), Storage config = Native(NativeStorageConfig { io_config: None }), Output schema = a (Int64), c (Int64)";
+        \n    Source: Json, File paths = [/foo], File schema = a (Int64), b (Int64), c (Int64), Format-specific config = Json(JsonSourceConfig { buffer_size: None, chunk_size: None }), Storage config = Native(NativeStorageConfig { io_config: None, multithreaded_io: true }), Projection pushdown = [a, c], Output schema = a (Int64), c (Int64)";
         assert_optimized_plan_eq(unoptimized, expected)?;
 
         Ok(())
@@ -683,9 +691,9 @@ mod tests {
         .build();
 
         let expected = "\
-        Project: col(a), Partition spec = PartitionSpec { scheme: Unknown, num_partitions: 1, by: None }\
+        Project: col(a)\
         \n  Filter: col(b)\
-        \n    Source: Json, File paths = [/foo], File schema = a (Int64), b (Boolean), c (Int64), Format-specific config = Json(JsonSourceConfig), Storage config = Native(NativeStorageConfig { io_config: None }), Output schema = a (Int64), b (Boolean)";
+        \n    Source: Json, File paths = [/foo], File schema = a (Int64), b (Boolean), c (Int64), Format-specific config = Json(JsonSourceConfig { buffer_size: None, chunk_size: None }), Storage config = Native(NativeStorageConfig { io_config: None, multithreaded_io: true }), Projection pushdown = [a, b], Output schema = a (Int64), b (Boolean)";
         assert_optimized_plan_eq(unoptimized, expected)?;
 
         Ok(())
