@@ -1,42 +1,39 @@
 use std::sync::Arc;
 
 use daft_core::schema::SchemaRef;
-use daft_dsl::ExprRef;
+use daft_scan::Pushdowns;
 
 use crate::{
-    physical_plan::PhysicalPlan, sink_info::OutputFileInfo, source_info::ExternalInfo,
+    physical_plan::PhysicalPlan, sink_info::OutputFileInfo, source_info::LegacyExternalInfo,
     PartitionSpec,
 };
 use serde::{Deserialize, Serialize};
 
-#[derive(Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct TabularScanJson {
     pub projection_schema: SchemaRef,
-    pub external_info: ExternalInfo,
+    pub external_info: LegacyExternalInfo,
     pub partition_spec: Arc<PartitionSpec>,
-    pub limit: Option<usize>,
-    pub filters: Vec<ExprRef>,
+    pub pushdowns: Pushdowns,
 }
 
 impl TabularScanJson {
     pub(crate) fn new(
         projection_schema: SchemaRef,
-        external_info: ExternalInfo,
+        external_info: LegacyExternalInfo,
         partition_spec: Arc<PartitionSpec>,
-        limit: Option<usize>,
-        filters: Vec<ExprRef>,
+        pushdowns: Pushdowns,
     ) -> Self {
         Self {
             projection_schema,
             external_info,
             partition_spec,
-            limit,
-            filters,
+            pushdowns,
         }
     }
 }
 
-#[derive(Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct TabularWriteJson {
     pub schema: SchemaRef,
     pub file_info: OutputFileInfo,
