@@ -2,15 +2,19 @@
 use {
     crate::{
         sink_info::OutputFileInfo,
-        source_info::{
-            ExternalInfo, FileFormat, FileFormatConfig, FileInfos, InMemoryInfo,
-            PyFileFormatConfig, PyStorageConfig, StorageConfig,
-        },
+        source_info::{FileInfos, InMemoryInfo, LegacyExternalInfo},
     },
+    common_io_config::IOConfig,
     daft_core::python::schema::PySchema,
     daft_core::schema::SchemaRef,
     daft_dsl::python::PyExpr,
     daft_dsl::Expr,
+    daft_scan::{
+        file_format::{FileFormat, FileFormatConfig, PyFileFormatConfig},
+        python::pylib::PyScanTask,
+        storage_config::{PyStorageConfig, StorageConfig},
+        Pushdowns,
+    },
     pyo3::{
         pyclass, pymethods, types::PyBytes, PyObject, PyRef, PyRefMut, PyResult, PyTypeInfo,
         Python, ToPyObject,
@@ -20,24 +24,28 @@ use {
 
 use daft_core::impl_bincode_py_state_serialization;
 use serde::{Deserialize, Serialize};
-use std::sync::Arc;
+use std::{cmp::max, sync::Arc};
 
-use crate::physical_ops::*;
+use crate::{physical_ops::*, PartitionScheme, PartitionSpec};
 
 /// Physical plan for a Daft query.
-#[derive(Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, Serialize, Deserialize)]
 pub enum PhysicalPlan {
     #[cfg(feature = "python")]
     InMemoryScan(InMemoryScan),
     TabularScanParquet(TabularScanParquet),
     TabularScanCsv(TabularScanCsv),
     TabularScanJson(TabularScanJson),
+    TabularScan(TabularScan),
+    EmptyScan(EmptyScan),
     Project(Project),
     Filter(Filter),
     Limit(Limit),
     Explode(Explode),
     Sort(Sort),
     Split(Split),
+    Sample(Sample),
+    Coalesce(Coalesce),
     Flatten(Flatten),
     FanoutRandom(FanoutRandom),
     FanoutByHash(FanoutByHash),
@@ -45,14 +53,195 @@ pub enum PhysicalPlan {
     FanoutByRange(FanoutByRange),
     ReduceMerge(ReduceMerge),
     Aggregate(Aggregate),
-    Coalesce(Coalesce),
     Concat(Concat),
-    Join(Join),
+    HashJoin(HashJoin),
+    BroadcastJoin(BroadcastJoin),
     TabularWriteParquet(TabularWriteParquet),
     TabularWriteJson(TabularWriteJson),
     TabularWriteCsv(TabularWriteCsv),
 }
 
+impl PhysicalPlan {
+    pub fn partition_spec(&self) -> Arc<PartitionSpec> {
+        match self {
+            #[cfg(feature = "python")]
+            Self::InMemoryScan(InMemoryScan { partition_spec, .. }) => partition_spec.clone(),
+            Self::TabularScan(TabularScan { partition_spec, .. }) => partition_spec.clone(),
+            Self::EmptyScan(EmptyScan { partition_spec, .. }) => partition_spec.clone(),
+            Self::TabularScanParquet(TabularScanParquet { partition_spec, .. }) => {
+                partition_spec.clone()
+            }
+            Self::TabularScanCsv(TabularScanCsv { partition_spec, .. }) => partition_spec.clone(),
+            Self::TabularScanJson(TabularScanJson { partition_spec, .. }) => partition_spec.clone(),
+            Self::Project(Project { partition_spec, .. }) => partition_spec.clone(),
+            Self::Filter(Filter { input, .. }) => input.partition_spec(),
+            Self::Limit(Limit { input, .. }) => input.partition_spec(),
+            Self::Explode(Explode { partition_spec, .. }) => partition_spec.clone(),
+            Self::Sample(Sample { input, .. }) => input.partition_spec(),
+            Self::Sort(Sort { input, sort_by, .. }) => PartitionSpec::new_internal(
+                PartitionScheme::Range,
+                input.partition_spec().num_partitions,
+                Some(sort_by.clone()),
+            )
+            .into(),
+            Self::Split(Split {
+                output_num_partitions,
+                ..
+            }) => {
+                PartitionSpec::new_internal(PartitionScheme::Unknown, *output_num_partitions, None)
+                    .into()
+            }
+            Self::Coalesce(Coalesce { num_to, .. }) => {
+                PartitionSpec::new_internal(PartitionScheme::Unknown, *num_to, None).into()
+            }
+            Self::Flatten(Flatten { input }) => input.partition_spec(),
+            Self::FanoutRandom(FanoutRandom { num_partitions, .. }) => {
+                PartitionSpec::new_internal(PartitionScheme::Random, *num_partitions, None).into()
+            }
+            Self::FanoutByHash(FanoutByHash {
+                num_partitions,
+                partition_by,
+                ..
+            }) => PartitionSpec::new_internal(
+                PartitionScheme::Hash,
+                *num_partitions,
+                Some(partition_by.clone()),
+            )
+            .into(),
+            Self::FanoutByRange(FanoutByRange {
+                num_partitions,
+                sort_by,
+                ..
+            }) => PartitionSpec::new_internal(
+                PartitionScheme::Range,
+                *num_partitions,
+                Some(sort_by.clone()),
+            )
+            .into(),
+            Self::ReduceMerge(ReduceMerge { input }) => input.partition_spec(),
+            Self::Aggregate(Aggregate { input, groupby, .. }) => {
+                let input_partition_spec = input.partition_spec();
+                if input_partition_spec.num_partitions == 1 {
+                    input_partition_spec.clone()
+                } else if groupby.is_empty() {
+                    PartitionSpec::new_internal(PartitionScheme::Unknown, 1, None).into()
+                } else {
+                    PartitionSpec::new_internal(
+                        PartitionScheme::Hash,
+                        input.partition_spec().num_partitions,
+                        Some(groupby.clone()),
+                    )
+                    .into()
+                }
+            }
+            Self::Concat(Concat { input, other }) => PartitionSpec::new_internal(
+                PartitionScheme::Unknown,
+                input.partition_spec().num_partitions + other.partition_spec().num_partitions,
+                None,
+            )
+            .into(),
+            Self::HashJoin(HashJoin {
+                left,
+                right,
+                left_on,
+                ..
+            }) => {
+                let input_partition_spec = left.partition_spec();
+                match max(
+                    input_partition_spec.num_partitions,
+                    right.partition_spec().num_partitions,
+                ) {
+                    // NOTE: This duplicates the repartitioning logic in the planner, where we
+                    // conditionally repartition the left and right tables.
+                    // TODO(Clark): Consolidate this logic with the planner logic when we push the partition spec
+                    // to be an entirely planner-side concept.
+                    1 => input_partition_spec,
+                    num_partitions => PartitionSpec::new_internal(
+                        PartitionScheme::Hash,
+                        num_partitions,
+                        Some(left_on.clone()),
+                    )
+                    .into(),
+                }
+            }
+            Self::BroadcastJoin(BroadcastJoin {
+                receiver: right, ..
+            }) => right.partition_spec(),
+            Self::TabularWriteParquet(TabularWriteParquet { input, .. }) => input.partition_spec(),
+            Self::TabularWriteCsv(TabularWriteCsv { input, .. }) => input.partition_spec(),
+            Self::TabularWriteJson(TabularWriteJson { input, .. }) => input.partition_spec(),
+        }
+    }
+
+    pub fn approximate_size_bytes(&self) -> Option<usize> {
+        match self {
+            #[cfg(feature = "python")]
+            Self::InMemoryScan(InMemoryScan { in_memory_info, .. }) => {
+                Some(in_memory_info.size_bytes)
+            }
+            Self::TabularScan(TabularScan { scan_tasks, .. }) => scan_tasks
+                .iter()
+                .map(|scan_task| scan_task.size_bytes())
+                .sum::<Option<usize>>(),
+            Self::EmptyScan(..) => Some(0),
+            // Assume no row/column pruning in cardinality-affecting operations.
+            // TODO(Clark): Estimate row/column pruning to get a better size approximation.
+            Self::Filter(Filter { input, .. })
+            | Self::Limit(Limit { input, .. })
+            | Self::Project(Project { input, .. }) => input.approximate_size_bytes(),
+            Self::Sample(Sample {
+                input, fraction, ..
+            }) => input
+                .approximate_size_bytes()
+                .map(|size| (size as f64 * fraction) as usize),
+            // Assume ~the same size in bytes for explodes.
+            // TODO(Clark): Improve this estimate.
+            Self::Explode(Explode { input, .. }) => input.approximate_size_bytes(),
+            // Propagate child approximation for operations that don't affect cardinality.
+            Self::Coalesce(Coalesce { input, .. })
+            | Self::FanoutByHash(FanoutByHash { input, .. })
+            | Self::FanoutByRange(FanoutByRange { input, .. })
+            | Self::FanoutRandom(FanoutRandom { input, .. })
+            | Self::Flatten(Flatten { input, .. })
+            | Self::ReduceMerge(ReduceMerge { input, .. })
+            | Self::Sort(Sort { input, .. })
+            | Self::Split(Split { input, .. }) => input.approximate_size_bytes(),
+            Self::Concat(Concat { input, other }) => {
+                input.approximate_size_bytes().and_then(|input_size| {
+                    other
+                        .approximate_size_bytes()
+                        .map(|other_size| input_size + other_size)
+                })
+            }
+            // Assume a simple sum of the sizes of both sides of the join for the post-join size.
+            // TODO(Clark): This will double-count join key columns, we should ensure that these are only counted once.
+            Self::BroadcastJoin(BroadcastJoin {
+                broadcaster: left,
+                receiver: right,
+                ..
+            })
+            | Self::HashJoin(HashJoin { left, right, .. }) => {
+                left.approximate_size_bytes().and_then(|left_size| {
+                    right
+                        .approximate_size_bytes()
+                        .map(|right_size| left_size + right_size)
+                })
+            }
+            // TODO(Clark): Approximate post-aggregation sizes via grouping estimates + aggregation type.
+            Self::Aggregate(_) => None,
+            // No size approximation support for legacy I/O.
+            Self::TabularScanParquet(_) | Self::TabularScanCsv(_) | Self::TabularScanJson(_) => {
+                None
+            }
+            // Post-write DataFrame will contain paths to files that were written.
+            // TODO(Clark): Estimate output size via root directory and estimates for # of partitions given partitioning column.
+            Self::TabularWriteParquet(_) | Self::TabularWriteCsv(_) | Self::TabularWriteJson(_) => {
+                None
+            }
+        }
+    }
+}
+
 /// A work scheduler for physical plans.
 #[cfg_attr(feature = "python", pyclass(module = "daft.daft"))]
 #[derive(Debug, Serialize, Deserialize)]
@@ -63,6 +252,10 @@ pub struct PhysicalPlanScheduler {
 #[cfg(feature = "python")]
 #[pymethods]
 impl PhysicalPlanScheduler {
+    pub fn num_partitions(&self) -> PyResult<i64> {
+        println!("{:?}", self.plan.partition_spec());
+        self.plan.partition_spec().get_num_partitions()
+    }
     /// Converts the contained physical plan into an iterator of executable partition tasks.
     pub fn to_partition_tasks(
         &self,
@@ -110,15 +303,21 @@ fn tabular_scan(
     file_infos: &Arc<FileInfos>,
     file_format_config: &Arc<FileFormatConfig>,
     storage_config: &Arc<StorageConfig>,
-    limit: &Option<usize>,
+    pushdowns: &Pushdowns,
     is_ray_runner: bool,
 ) -> PyResult<PyObject> {
-    let columns_to_read = projection_schema
-        .fields
-        .iter()
-        .map(|(name, _)| name)
-        .cloned()
-        .collect::<Vec<_>>();
+    let columns_to_read = if projection_schema.names() != source_schema.names() {
+        Some(
+            projection_schema
+                .fields
+                .iter()
+                .map(|(name, _)| name)
+                .cloned()
+                .collect::<Vec<_>>(),
+        )
+    } else {
+        None
+    };
     let py_iter = py
         .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
         .getattr(pyo3::intern!(py, "tabular_scan"))?
@@ -128,12 +327,14 @@ fn tabular_scan(
             file_infos.to_table()?,
             PyFileFormatConfig::from(file_format_config.clone()),
             PyStorageConfig::from(storage_config.clone()),
-            *limit,
+            pushdowns.limit,
             is_ray_runner,
         ))?;
+
     Ok(py_iter.into())
 }
 
+#[allow(clippy::too_many_arguments)]
 #[cfg(feature = "python")]
 fn tabular_write(
     py: Python<'_>,
@@ -143,6 +344,7 @@ fn tabular_write(
     root_dir: &String,
     compression: &Option<String>,
     partition_cols: &Option<Vec<Expr>>,
+    io_config: &Option<IOConfig>,
 ) -> PyResult<PyObject> {
     let part_cols = partition_cols.as_ref().map(|cols| {
         cols.iter()
@@ -159,6 +361,11 @@ fn tabular_write(
             root_dir,
             compression.clone(),
             part_cols,
+            io_config
+                .as_ref()
+                .map(|cfg| common_io_config::python::IOConfig {
+                    config: cfg.clone(),
+                }),
         ))?;
     Ok(py_iter.into())
 }
@@ -186,17 +393,43 @@ impl PhysicalPlan {
                     .call1((partition_iter,))?;
                 Ok(py_iter.into())
             }
+            PhysicalPlan::TabularScan(TabularScan { scan_tasks, .. }) => {
+                let py_iter = py
+                    .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                    .getattr(pyo3::intern!(py, "scan_with_tasks"))?
+                    .call1((scan_tasks
+                        .iter()
+                        .map(|scan_task| PyScanTask(scan_task.clone()))
+                        .collect::<Vec<PyScanTask>>(),))?;
+                Ok(py_iter.into())
+            }
+            PhysicalPlan::EmptyScan(EmptyScan { schema, .. }) => {
+                let schema_mod = py.import(pyo3::intern!(py, "daft.logical.schema"))?;
+                let python_schema = schema_mod
+                    .getattr(pyo3::intern!(py, "Schema"))?
+                    .getattr(pyo3::intern!(py, "_from_pyschema"))?
+                    .call1((PySchema {
+                        schema: schema.clone(),
+                    },))?;
+
+                let py_iter = py
+                    .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                    .getattr(pyo3::intern!(py, "empty_scan"))?
+                    .call1((python_schema,))?;
+                Ok(py_iter.into())
+            }
+
             PhysicalPlan::TabularScanParquet(TabularScanParquet {
                 projection_schema,
                 external_info:
-                    ExternalInfo {
+                    LegacyExternalInfo {
                         source_schema,
                         file_infos,
                         file_format_config,
                         storage_config,
+                        pushdowns,
                         ..
                     },
-                limit,
                 ..
             }) => tabular_scan(
                 py,
@@ -205,20 +438,20 @@ impl PhysicalPlan {
                 file_infos,
                 file_format_config,
                 storage_config,
-                limit,
+                pushdowns,
                 is_ray_runner,
             ),
             PhysicalPlan::TabularScanCsv(TabularScanCsv {
                 projection_schema,
                 external_info:
-                    ExternalInfo {
+                    LegacyExternalInfo {
                         source_schema,
                         file_infos,
                         file_format_config,
                         storage_config,
+                        pushdowns,
                         ..
                     },
-                limit,
                 ..
             }) => tabular_scan(
                 py,
@@ -227,20 +460,20 @@ impl PhysicalPlan {
                 file_infos,
                 file_format_config,
                 storage_config,
-                limit,
+                pushdowns,
                 is_ray_runner,
             ),
             PhysicalPlan::TabularScanJson(TabularScanJson {
                 projection_schema,
                 external_info:
-                    ExternalInfo {
+                    LegacyExternalInfo {
                         source_schema,
                         file_infos,
                         file_format_config,
                         storage_config,
+                        pushdowns,
                         ..
                     },
-                limit,
                 ..
             }) => tabular_scan(
                 py,
@@ -249,13 +482,14 @@ impl PhysicalPlan {
                 file_infos,
                 file_format_config,
                 storage_config,
-                limit,
+                pushdowns,
                 is_ray_runner,
             ),
             PhysicalPlan::Project(Project {
                 input,
                 projection,
                 resource_request,
+                ..
             }) => {
                 let upstream_iter = input.to_partition_tasks(py, psets, is_ray_runner)?;
                 let projection_pyexprs: Vec<PyExpr> = projection
@@ -296,20 +530,20 @@ impl PhysicalPlan {
             PhysicalPlan::Limit(Limit {
                 input,
                 limit,
+                eager,
                 num_partitions,
             }) => {
                 let upstream_iter = input.to_partition_tasks(py, psets, is_ray_runner)?;
                 let py_physical_plan =
                     py.import(pyo3::intern!(py, "daft.execution.physical_plan"))?;
-                let local_limit_iter = py_physical_plan
-                    .getattr(pyo3::intern!(py, "local_limit"))?
-                    .call1((upstream_iter, *limit))?;
                 let global_limit_iter = py_physical_plan
                     .getattr(pyo3::intern!(py, "global_limit"))?
-                    .call1((local_limit_iter, *limit, *num_partitions))?;
+                    .call1((upstream_iter, *limit, *eager, *num_partitions))?;
                 Ok(global_limit_iter.into())
             }
-            PhysicalPlan::Explode(Explode { input, to_explode }) => {
+            PhysicalPlan::Explode(Explode {
+                input, to_explode, ..
+            }) => {
                 let upstream_iter = input.to_partition_tasks(py, psets, is_ray_runner)?;
                 let explode_pyexprs: Vec<PyExpr> = to_explode
                     .iter()
@@ -321,6 +555,19 @@ impl PhysicalPlan {
                     .call1((upstream_iter, explode_pyexprs))?;
                 Ok(py_iter.into())
             }
+            PhysicalPlan::Sample(Sample {
+                input,
+                fraction,
+                with_replacement,
+                seed,
+            }) => {
+                let upstream_iter = input.to_partition_tasks(py, psets, is_ray_runner)?;
+                let py_iter = py
+                    .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                    .getattr(pyo3::intern!(py, "sample"))?
+                    .call1((upstream_iter, *fraction, *with_replacement, *seed))?;
+                Ok(py_iter.into())
+            }
             PhysicalPlan::Sort(Sort {
                 input,
                 sort_by,
@@ -443,7 +690,7 @@ impl PhysicalPlan {
                     .call1((upstream_input_iter, upstream_other_iter))?;
                 Ok(py_iter.into())
             }
-            PhysicalPlan::Join(Join {
+            PhysicalPlan::HashJoin(HashJoin {
                 left,
                 right,
                 left_on,
@@ -463,7 +710,7 @@ impl PhysicalPlan {
                     .collect();
                 let py_iter = py
                     .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
-                    .getattr(pyo3::intern!(py, "join"))?
+                    .getattr(pyo3::intern!(py, "hash_join"))?
                     .call1((
                         upstream_left_iter,
                         upstream_right_iter,
@@ -473,6 +720,37 @@ impl PhysicalPlan {
                     ))?;
                 Ok(py_iter.into())
             }
+            PhysicalPlan::BroadcastJoin(BroadcastJoin {
+                broadcaster: left,
+                receiver: right,
+                left_on,
+                right_on,
+                join_type,
+                is_swapped,
+            }) => {
+                let upstream_left_iter = left.to_partition_tasks(py, psets, is_ray_runner)?;
+                let upstream_right_iter = right.to_partition_tasks(py, psets, is_ray_runner)?;
+                let left_on_pyexprs: Vec<PyExpr> = left_on
+                    .iter()
+                    .map(|expr| PyExpr::from(expr.clone()))
+                    .collect();
+                let right_on_pyexprs: Vec<PyExpr> = right_on
+                    .iter()
+                    .map(|expr| PyExpr::from(expr.clone()))
+                    .collect();
+                let py_iter = py
+                    .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                    .getattr(pyo3::intern!(py, "broadcast_join"))?
+                    .call1((
+                        upstream_left_iter,
+                        upstream_right_iter,
+                        left_on_pyexprs,
+                        right_on_pyexprs,
+                        *join_type,
+                        *is_swapped,
+                    ))?;
+                Ok(py_iter.into())
+            }
             PhysicalPlan::TabularWriteParquet(TabularWriteParquet {
                 schema,
                 file_info:
@@ -481,6 +759,7 @@ impl PhysicalPlan {
                         file_format,
                         partition_cols,
                         compression,
+                        io_config,
                     },
                 input,
             }) => tabular_write(
@@ -491,6 +770,7 @@ impl PhysicalPlan {
                 root_dir,
                 compression,
                 partition_cols,
+                io_config,
             ),
             PhysicalPlan::TabularWriteCsv(TabularWriteCsv {
                 schema,
@@ -500,6 +780,7 @@ impl PhysicalPlan {
                         file_format,
                         partition_cols,
                         compression,
+                        io_config,
                     },
                 input,
             }) => tabular_write(
@@ -510,6 +791,7 @@ impl PhysicalPlan {
                 root_dir,
                 compression,
                 partition_cols,
+                io_config,
             ),
             PhysicalPlan::TabularWriteJson(TabularWriteJson {
                 schema,
@@ -519,6 +801,7 @@ impl PhysicalPlan {
                         file_format,
                         partition_cols,
                         compression,
+                        io_config,
                     },
                 input,
             }) => tabular_write(
@@ -529,6 +812,7 @@ impl PhysicalPlan {
                 root_dir,
                 compression,
                 partition_cols,
+                io_config,
             ),
         }
     }
