@@ -1,20 +1,24 @@
+use std::cmp::Ordering;
 use std::sync::Arc;
 use std::{cmp::max, collections::HashMap};
 
+use common_daft_config::DaftExecutionConfig;
 use common_error::DaftResult;
 use daft_core::count_mode::CountMode;
 use daft_dsl::Expr;
+use daft_scan::file_format::FileFormatConfig;
+use daft_scan::ScanExternalInfo;
 
 use crate::logical_ops::{
-    Aggregate as LogicalAggregate, Coalesce as LogicalCoalesce, Concat as LogicalConcat,
-    Distinct as LogicalDistinct, Explode as LogicalExplode, Filter as LogicalFilter,
-    Join as LogicalJoin, Limit as LogicalLimit, Project as LogicalProject,
-    Repartition as LogicalRepartition, Sink as LogicalSink, Sort as LogicalSort, Source,
+    Aggregate as LogicalAggregate, Concat as LogicalConcat, Distinct as LogicalDistinct,
+    Explode as LogicalExplode, Filter as LogicalFilter, Join as LogicalJoin, Limit as LogicalLimit,
+    Project as LogicalProject, Repartition as LogicalRepartition, Sample as LogicalSample,
+    Sink as LogicalSink, Sort as LogicalSort, Source,
 };
 use crate::logical_plan::LogicalPlan;
 use crate::physical_plan::PhysicalPlan;
 use crate::sink_info::{OutputFileInfo, SinkInfo};
-use crate::source_info::{ExternalInfo as ExternalSourceInfo, FileFormatConfig, SourceInfo};
+use crate::source_info::{ExternalInfo as ExternalSourceInfo, LegacyExternalInfo, SourceInfo};
 use crate::{physical_ops::*, PartitionSpec};
 use crate::{FileFormat, PartitionScheme};
 
@@ -22,64 +26,100 @@ use crate::{FileFormat, PartitionScheme};
 use crate::physical_ops::InMemoryScan;
 
 /// Translate a logical plan to a physical plan.
-pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
+pub fn plan(logical_plan: &LogicalPlan, cfg: Arc<DaftExecutionConfig>) -> DaftResult<PhysicalPlan> {
     match logical_plan {
         LogicalPlan::Source(Source {
             output_schema,
             source_info,
-            partition_spec,
-            limit,
-            filters,
         }) => match source_info.as_ref() {
-            SourceInfo::ExternalInfo(
-                ext_info @ ExternalSourceInfo {
-                    file_format_config, ..
+            SourceInfo::ExternalInfo(ExternalSourceInfo::Legacy(
+                ext_info @ LegacyExternalInfo {
+                    file_format_config,
+                    file_infos,
+                    pushdowns,
+                    ..
                 },
-            ) => match file_format_config.as_ref() {
-                FileFormatConfig::Parquet(_) => {
-                    Ok(PhysicalPlan::TabularScanParquet(TabularScanParquet::new(
-                        output_schema.clone(),
-                        ext_info.clone(),
-                        partition_spec.clone(),
-                        *limit,
-                        filters.to_vec(),
+            )) => {
+                let partition_spec = Arc::new(PartitionSpec::new_internal(
+                    PartitionScheme::Unknown,
+                    file_infos.len(),
+                    None,
+                ));
+                match file_format_config.as_ref() {
+                    FileFormatConfig::Parquet(_) => {
+                        Ok(PhysicalPlan::TabularScanParquet(TabularScanParquet::new(
+                            output_schema.clone(),
+                            ext_info.clone(),
+                            partition_spec,
+                            pushdowns.clone(),
+                        )))
+                    }
+                    FileFormatConfig::Csv(_) => {
+                        Ok(PhysicalPlan::TabularScanCsv(TabularScanCsv::new(
+                            output_schema.clone(),
+                            ext_info.clone(),
+                            partition_spec,
+                            pushdowns.clone(),
+                        )))
+                    }
+                    FileFormatConfig::Json(_) => {
+                        Ok(PhysicalPlan::TabularScanJson(TabularScanJson::new(
+                            output_schema.clone(),
+                            ext_info.clone(),
+                            partition_spec,
+                            pushdowns.clone(),
+                        )))
+                    }
+                }
+            }
+            SourceInfo::ExternalInfo(ExternalSourceInfo::Scan(ScanExternalInfo {
+                pushdowns,
+                scan_op,
+                source_schema,
+                ..
+            })) => {
+                let scan_tasks = scan_op.0.to_scan_tasks(pushdowns.clone())?;
+
+                // Apply transformations on the ScanTasks to optimize
+                let scan_tasks = daft_scan::scan_task_iters::merge_by_sizes(
+                    scan_tasks,
+                    cfg.merge_scan_tasks_min_size_bytes,
+                    cfg.merge_scan_tasks_max_size_bytes,
+                );
+                let scan_tasks = scan_tasks.collect::<DaftResult<Vec<_>>>()?;
+                if scan_tasks.is_empty() {
+                    let partition_spec = Arc::new(PartitionSpec::new_internal(
+                        PartitionScheme::Unknown,
+                        1,
+                        None,
+                    ));
+
+                    Ok(PhysicalPlan::EmptyScan(EmptyScan::new(
+                        source_schema.clone(),
+                        partition_spec,
+                    )))
+                } else {
+                    let partition_spec = Arc::new(PartitionSpec::new_internal(
+                        PartitionScheme::Unknown,
+                        scan_tasks.len(),
+                        None,
+                    ));
+
+                    Ok(PhysicalPlan::TabularScan(TabularScan::new(
+                        scan_tasks,
+                        partition_spec,
                     )))
                 }
-                FileFormatConfig::Csv(_) => Ok(PhysicalPlan::TabularScanCsv(TabularScanCsv::new(
-                    output_schema.clone(),
-                    ext_info.clone(),
-                    partition_spec.clone(),
-                    *limit,
-                    filters.to_vec(),
-                ))),
-                FileFormatConfig::Json(_) => {
-                    Ok(PhysicalPlan::TabularScanJson(TabularScanJson::new(
-                        output_schema.clone(),
-                        ext_info.clone(),
-                        partition_spec.clone(),
-                        *limit,
-                        filters.to_vec(),
-                    )))
-                }
-            },
+            }
             #[cfg(feature = "python")]
             SourceInfo::InMemoryInfo(mem_info) => {
                 let scan = PhysicalPlan::InMemoryScan(InMemoryScan::new(
                     mem_info.source_schema.clone(),
                     mem_info.clone(),
-                    partition_spec.clone(),
+                    PartitionSpec::new(PartitionScheme::Unknown, mem_info.num_partitions, None)
+                        .into(),
                 ));
-                let plan = if output_schema.fields.len() < mem_info.source_schema.fields.len() {
-                    let projection = output_schema
-                        .fields
-                        .iter()
-                        .map(|(name, _)| Expr::Column(name.clone().into()))
-                        .collect::<Vec<_>>();
-                    PhysicalPlan::Project(Project::new(scan.into(), projection, Default::default()))
-                } else {
-                    scan
-                };
-                Ok(plan)
+                Ok(scan)
             }
         },
         LogicalPlan::Project(LogicalProject {
@@ -88,44 +128,52 @@ pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
             resource_request,
             ..
         }) => {
-            let input_physical = plan(input)?;
-            Ok(PhysicalPlan::Project(Project::new(
+            let input_physical = plan(input, cfg)?;
+            let partition_spec = input_physical.partition_spec().clone();
+            Ok(PhysicalPlan::Project(Project::try_new(
                 input_physical.into(),
                 projection.clone(),
                 resource_request.clone(),
-            )))
+                partition_spec,
+            )?))
         }
         LogicalPlan::Filter(LogicalFilter { input, predicate }) => {
-            let input_physical = plan(input)?;
+            let input_physical = plan(input, cfg)?;
             Ok(PhysicalPlan::Filter(Filter::new(
                 input_physical.into(),
                 predicate.clone(),
             )))
         }
-        LogicalPlan::Limit(LogicalLimit { input, limit }) => {
-            let input_physical = plan(input)?;
+        LogicalPlan::Limit(LogicalLimit {
+            input,
+            limit,
+            eager,
+        }) => {
+            let input_physical = plan(input, cfg)?;
+            let num_partitions = input_physical.partition_spec().num_partitions;
             Ok(PhysicalPlan::Limit(Limit::new(
                 input_physical.into(),
                 *limit,
-                logical_plan.partition_spec().num_partitions,
+                *eager,
+                num_partitions,
             )))
         }
         LogicalPlan::Explode(LogicalExplode {
             input, to_explode, ..
         }) => {
-            let input_physical = plan(input)?;
-            Ok(PhysicalPlan::Explode(Explode::new(
+            let input_physical = plan(input, cfg)?;
+            Ok(PhysicalPlan::Explode(Explode::try_new(
                 input_physical.into(),
                 to_explode.clone(),
-            )))
+            )?))
         }
         LogicalPlan::Sort(LogicalSort {
             input,
             sort_by,
             descending,
         }) => {
-            let input_physical = plan(input)?;
-            let num_partitions = logical_plan.partition_spec().num_partitions;
+            let input_physical = plan(input, cfg)?;
+            let num_partitions = input_physical.partition_spec().num_partitions;
             Ok(PhysicalPlan::Sort(Sort::new(
                 input_physical.into(),
                 sort_by.clone(),
@@ -139,44 +187,77 @@ pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
             partition_by,
             scheme,
         }) => {
-            let input_physical = Arc::new(plan(input)?);
-            match scheme {
-                PartitionScheme::Unknown => {
-                    let split_op = PhysicalPlan::Split(Split::new(
-                        input_physical,
-                        input.partition_spec().num_partitions,
-                        *num_partitions,
-                    ));
-                    Ok(PhysicalPlan::Flatten(Flatten::new(split_op.into())))
-                }
+            // Below partition-dropping optimization assumes we are NOT repartitioning using a range partitioning scheme.
+            // A range repartitioning of an existing range-partitioned DataFrame is only redundant if the partition boundaries
+            // are consistent, which is only the case if boundary sampling is deterministic within a query.
+            assert!(!matches!(scheme, PartitionScheme::Range));
+
+            let input_physical = plan(input, cfg)?;
+            let input_partition_spec = input_physical.partition_spec();
+            let input_num_partitions = input_partition_spec.num_partitions;
+            let num_partitions = num_partitions.unwrap_or(input_num_partitions);
+            // Partition spec after repartitioning.
+            let repartitioned_partition_spec = PartitionSpec::new_internal(
+                scheme.clone(),
+                num_partitions,
+                Some(partition_by.clone()),
+            );
+            // Drop the repartition if the output of the repartition would yield the same partitioning as the input.
+            if (input_num_partitions == 1 && num_partitions == 1)
+                // Simple split/coalesce repartition to the same # of partitions is a no-op, no matter the upstream partitioning scheme.
+                || (num_partitions == input_num_partitions && matches!(scheme, PartitionScheme::Unknown))
+                // Repartitioning to the same partition spec as the input is always a no-op.
+                || (&repartitioned_partition_spec == input_partition_spec.as_ref())
+            {
+                return Ok(input_physical);
+            }
+            let input_physical = Arc::new(input_physical);
+            let repartitioned_plan = match scheme {
+                PartitionScheme::Unknown => match num_partitions.cmp(&input_num_partitions) {
+                    Ordering::Greater => {
+                        // Split input partitions into num_partitions.
+                        let split_op = PhysicalPlan::Split(Split::new(
+                            input_physical,
+                            input_num_partitions,
+                            num_partitions,
+                        ));
+                        PhysicalPlan::Flatten(Flatten::new(split_op.into()))
+                    }
+                    Ordering::Less => {
+                        // Coalesce input partitions into num_partitions.
+                        PhysicalPlan::Coalesce(Coalesce::new(
+                            input_physical,
+                            input_num_partitions,
+                            num_partitions,
+                        ))
+                    }
+                    Ordering::Equal => {
+                        // # of output partitions == # of input partitions; this should have already short-circuited with
+                        // a repartition drop above.
+                        unreachable!("Simple repartitioning with same # of output partitions as the input; this should have been dropped.")
+                    }
+                },
                 PartitionScheme::Random => {
                     let split_op = PhysicalPlan::FanoutRandom(FanoutRandom::new(
                         input_physical,
-                        *num_partitions,
+                        num_partitions,
                     ));
-                    Ok(PhysicalPlan::ReduceMerge(ReduceMerge::new(split_op.into())))
+                    PhysicalPlan::ReduceMerge(ReduceMerge::new(split_op.into()))
                 }
                 PartitionScheme::Hash => {
                     let split_op = PhysicalPlan::FanoutByHash(FanoutByHash::new(
                         input_physical,
-                        *num_partitions,
+                        num_partitions,
                         partition_by.clone(),
                     ));
-                    Ok(PhysicalPlan::ReduceMerge(ReduceMerge::new(split_op.into())))
+                    PhysicalPlan::ReduceMerge(ReduceMerge::new(split_op.into()))
                 }
                 PartitionScheme::Range => unreachable!("Repartitioning by range is not supported"),
-            }
-        }
-        LogicalPlan::Coalesce(LogicalCoalesce { input, num_to }) => {
-            let input_physical = plan(input)?;
-            Ok(PhysicalPlan::Coalesce(Coalesce::new(
-                input_physical.into(),
-                input.partition_spec().num_partitions,
-                *num_to,
-            )))
+            };
+            Ok(repartitioned_plan)
         }
         LogicalPlan::Distinct(LogicalDistinct { input }) => {
-            let input_physical = plan(input)?;
+            let input_physical = plan(input, cfg)?;
             let col_exprs = input
                 .schema()
                 .names()
@@ -188,7 +269,7 @@ pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
                 vec![],
                 col_exprs.clone(),
             ));
-            let num_partitions = logical_plan.partition_spec().num_partitions;
+            let num_partitions = agg_op.partition_spec().num_partitions;
             if num_partitions > 1 {
                 let split_op = PhysicalPlan::FanoutByHash(FanoutByHash::new(
                     agg_op.into(),
@@ -205,6 +286,20 @@ pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
                 Ok(agg_op)
             }
         }
+        LogicalPlan::Sample(LogicalSample {
+            input,
+            fraction,
+            with_replacement,
+            seed,
+        }) => {
+            let input_physical = plan(input, cfg)?;
+            Ok(PhysicalPlan::Sample(Sample::new(
+                input_physical.into(),
+                *fraction,
+                *with_replacement,
+                *seed,
+            )))
+        }
         LogicalPlan::Aggregate(LogicalAggregate {
             aggregations,
             groupby,
@@ -213,9 +308,9 @@ pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
         }) => {
             use daft_dsl::AggExpr::{self, *};
             use daft_dsl::Expr::Column;
-            let input_plan = plan(input)?;
+            let input_plan = plan(input, cfg)?;
 
-            let num_input_partitions = input.partition_spec().num_partitions;
+            let num_input_partitions = input_plan.partition_spec().num_partitions;
 
             let result_plan = match num_input_partitions {
                 1 => PhysicalPlan::Aggregate(Aggregate::new(
@@ -390,19 +485,21 @@ pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
                         groupby.clone(),
                     ));
 
-                    PhysicalPlan::Project(Project::new(
+                    let partition_spec = second_stage_agg.partition_spec().clone();
+                    PhysicalPlan::Project(Project::try_new(
                         second_stage_agg.into(),
                         final_exprs,
                         Default::default(),
-                    ))
+                        partition_spec,
+                    )?)
                 }
             };
 
             Ok(result_plan)
         }
         LogicalPlan::Concat(LogicalConcat { input, other }) => {
-            let input_physical = plan(input)?;
-            let other_physical = plan(other)?;
+            let input_physical = plan(input, cfg.clone())?;
+            let other_physical = plan(other, cfg.clone())?;
             Ok(PhysicalPlan::Concat(Concat::new(
                 input_physical.into(),
                 other_physical.into(),
@@ -416,10 +513,11 @@ pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
             join_type,
             ..
         }) => {
-            let mut left_physical = plan(left)?;
-            let mut right_physical = plan(right)?;
-            let left_pspec = left.partition_spec();
-            let right_pspec = right.partition_spec();
+            let mut left_physical = plan(left, cfg.clone())?;
+            let mut right_physical = plan(right, cfg.clone())?;
+
+            let left_pspec = left_physical.partition_spec();
+            let right_pspec = right_physical.partition_spec();
             let num_partitions = max(left_pspec.num_partitions, right_pspec.num_partitions);
             let new_left_pspec = Arc::new(PartitionSpec::new_internal(
                 PartitionScheme::Hash,
@@ -431,8 +529,44 @@ pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
                 num_partitions,
                 Some(right_on.clone()),
             ));
+
+            let is_left_partitioned = left_pspec == new_left_pspec;
+            let is_right_partitioned = right_pspec == new_right_pspec;
+
+            // If either the left or right side of the join are very small tables, perform a broadcast join with the
+            // entire smaller table broadcast to each of the partitions of the larger table.
+
+            // Ensure that the left side of the join is the smaller side.
+            let (smaller_size_bytes, do_swap) = match (
+                left_physical.approximate_size_bytes(),
+                right_physical.approximate_size_bytes(),
+            ) {
+                (Some(left_size_bytes), Some(right_size_bytes)) => {
+                    if right_size_bytes < left_size_bytes {
+                        (Some(right_size_bytes), true)
+                    } else {
+                        (Some(left_size_bytes), false)
+                    }
+                }
+                (Some(left_size_bytes), None) => (Some(left_size_bytes), false),
+                (None, Some(right_size_bytes)) => (Some(right_size_bytes), true),
+                (None, None) => (None, false),
+            };
+            let is_larger_partitioned = if do_swap {
+                is_left_partitioned
+            } else {
+                is_right_partitioned
+            };
+            // If larger table is not already partitioned on the join key AND the smaller table is under broadcast size threshold, use broadcast join.
+            if !is_larger_partitioned && let Some(smaller_size_bytes) = smaller_size_bytes && smaller_size_bytes <= cfg.broadcast_join_size_bytes_threshold {
+                if do_swap {
+                    // These will get swapped back when doing the actual local joins.
+                    (left_physical, right_physical) = (right_physical, left_physical);
+                }
+                return Ok(PhysicalPlan::BroadcastJoin(BroadcastJoin::new(left_physical.into(), right_physical.into(), left_on.clone(), right_on.clone(), *join_type, do_swap)));
+            }
             if (num_partitions > 1 || left_pspec.num_partitions != num_partitions)
-                && left_pspec != new_left_pspec
+                && !is_left_partitioned
             {
                 let split_op = PhysicalPlan::FanoutByHash(FanoutByHash::new(
                     left_physical.into(),
@@ -442,7 +576,7 @@ pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
                 left_physical = PhysicalPlan::ReduceMerge(ReduceMerge::new(split_op.into()));
             }
             if (num_partitions > 1 || right_pspec.num_partitions != num_partitions)
-                && right_pspec != new_right_pspec
+                && !is_right_partitioned
             {
                 let split_op = PhysicalPlan::FanoutByHash(FanoutByHash::new(
                     right_physical.into(),
@@ -451,7 +585,7 @@ pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
                 ));
                 right_physical = PhysicalPlan::ReduceMerge(ReduceMerge::new(split_op.into()));
             }
-            Ok(PhysicalPlan::Join(Join::new(
+            Ok(PhysicalPlan::HashJoin(HashJoin::new(
                 left_physical.into(),
                 right_physical.into(),
                 left_on.clone(),
@@ -464,7 +598,7 @@ pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
             sink_info,
             input,
         }) => {
-            let input_physical = plan(input)?;
+            let input_physical = plan(input, cfg)?;
             match sink_info.as_ref() {
                 SinkInfo::OutputFileInfo(file_info @ OutputFileInfo { file_format, .. }) => {
                     match file_format {
@@ -493,3 +627,113 @@ pub fn plan(logical_plan: &LogicalPlan) -> DaftResult<PhysicalPlan> {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use common_daft_config::DaftExecutionConfig;
+    use common_error::DaftResult;
+    use daft_core::{datatypes::Field, DataType};
+    use daft_dsl::{col, lit, AggExpr, Expr};
+    use std::assert_matches::assert_matches;
+    use std::sync::Arc;
+
+    use crate::physical_plan::PhysicalPlan;
+    use crate::planner::plan;
+    use crate::{test::dummy_scan_node, PartitionScheme};
+
+    /// Tests that planner drops a simple Repartition (e.g. df.into_partitions()) the child already has the desired number of partitions.
+    ///
+    /// Repartition-upstream_op -> upstream_op
+    #[test]
+    fn repartition_dropped_redundant_into_partitions() -> DaftResult<()> {
+        let cfg: Arc<DaftExecutionConfig> = DaftExecutionConfig::default().into();
+        // dummy_scan_node() will create the default PartitionSpec, which only has a single partition.
+        let builder = dummy_scan_node(vec![
+            Field::new("a", DataType::Int64),
+            Field::new("b", DataType::Utf8),
+        ])
+        .repartition(Some(10), vec![], PartitionScheme::Unknown)?
+        .filter(col("a").lt(&lit(2)))?;
+        assert_eq!(
+            plan(builder.build().as_ref(), cfg.clone())?
+                .partition_spec()
+                .num_partitions,
+            10
+        );
+        let logical_plan = builder
+            .repartition(Some(10), vec![], PartitionScheme::Unknown)?
+            .build();
+        let physical_plan = plan(logical_plan.as_ref(), cfg.clone())?;
+        // Check that the last repartition was dropped (the last op should be the filter).
+        assert_matches!(physical_plan, PhysicalPlan::Filter(_));
+        Ok(())
+    }
+
+    /// Tests that planner drops a Repartition if both the Repartition and the child have a single partition.
+    ///
+    /// Repartition-upstream_op -> upstream_op
+    #[test]
+    fn repartition_dropped_single_partition() -> DaftResult<()> {
+        let cfg: Arc<DaftExecutionConfig> = DaftExecutionConfig::default().into();
+        // dummy_scan_node() will create the default PartitionSpec, which only has a single partition.
+        let builder = dummy_scan_node(vec![
+            Field::new("a", DataType::Int64),
+            Field::new("b", DataType::Utf8),
+        ]);
+        assert_eq!(
+            plan(builder.build().as_ref(), cfg.clone())?
+                .partition_spec()
+                .num_partitions,
+            1
+        );
+        let logical_plan = builder
+            .repartition(Some(1), vec![col("a")], PartitionScheme::Hash)?
+            .build();
+        let physical_plan = plan(logical_plan.as_ref(), cfg.clone())?;
+        assert_matches!(physical_plan, PhysicalPlan::TabularScanJson(_));
+        Ok(())
+    }
+
+    /// Tests that planner drops a Repartition if both the Repartition and the child have the same partition spec.
+    ///
+    /// Repartition-upstream_op -> upstream_op
+    #[test]
+    fn repartition_dropped_same_partition_spec() -> DaftResult<()> {
+        let cfg = DaftExecutionConfig::default().into();
+        let logical_plan = dummy_scan_node(vec![
+            Field::new("a", DataType::Int64),
+            Field::new("b", DataType::Utf8),
+        ])
+        .repartition(Some(10), vec![col("a")], PartitionScheme::Hash)?
+        .filter(col("a").lt(&lit(2)))?
+        .repartition(Some(10), vec![col("a")], PartitionScheme::Hash)?
+        .build();
+        let physical_plan = plan(logical_plan.as_ref(), cfg)?;
+        // Check that the last repartition was dropped (the last op should be the filter).
+        assert_matches!(physical_plan, PhysicalPlan::Filter(_));
+        Ok(())
+    }
+
+    /// Tests that planner drops a Repartition if both the Repartition and the upstream Aggregation have the same partition spec.
+    ///
+    /// Repartition-Aggregation -> Aggregation
+    #[test]
+    fn repartition_dropped_same_partition_spec_agg() -> DaftResult<()> {
+        let cfg = DaftExecutionConfig::default().into();
+        let logical_plan = dummy_scan_node(vec![
+            Field::new("a", DataType::Int64),
+            Field::new("b", DataType::Int64),
+        ])
+        .repartition(Some(10), vec![col("a")], PartitionScheme::Hash)?
+        .aggregate(
+            vec![Expr::Agg(AggExpr::Sum(col("a").into()))],
+            vec![col("b")],
+        )?
+        .repartition(Some(10), vec![col("b")], PartitionScheme::Hash)?
+        .build();
+        let physical_plan = plan(logical_plan.as_ref(), cfg)?;
+        // Check that the last repartition was dropped (the last op should be a projection for a multi-partition aggregation).
+        assert_matches!(physical_plan, PhysicalPlan::Project(_));
+        Ok(())
+    }
+}
