@@ -1,3 +1,4 @@
+use common_io_config::IOConfig;
 use daft_dsl::Expr;
 
 use crate::FileFormat;
@@ -14,6 +15,7 @@ pub struct OutputFileInfo {
     pub file_format: FileFormat,
     pub partition_cols: Option<Vec<Expr>>,
     pub compression: Option<String>,
+    pub io_config: Option<IOConfig>,
 }
 
 impl OutputFileInfo {
@@ -22,12 +24,14 @@ impl OutputFileInfo {
         file_format: FileFormat,
         partition_cols: Option<Vec<Expr>>,
         compression: Option<String>,
+        io_config: Option<IOConfig>,
     ) -> Self {
         Self {
             root_dir,
             file_format,
             partition_cols,
             compression,
+            io_config,
         }
     }
 }
