@@ -1,9 +1,11 @@
 #![feature(hash_raw_entry)]
 
+use std::borrow::Cow;
 use std::collections::HashSet;
 use std::fmt::{Display, Formatter, Result};
 
 use daft_core::array::ops::full::FullNull;
+use daft_core::utils::display_table::make_comfy_table;
 use num_traits::ToPrimitive;
 
 use daft_core::array::ops::GroupIndices;
@@ -16,14 +18,16 @@ use daft_core::series::{IntoSeries, Series};
 use daft_dsl::functions::FunctionEvaluator;
 use daft_dsl::{col, null_lit, AggExpr, Expr};
 #[cfg(feature = "python")]
-mod ffi;
+pub mod ffi;
 mod ops;
+
+pub use ops::infer_join_schema;
 #[cfg(feature = "python")]
 pub mod python;
 #[cfg(feature = "python")]
 pub use python::register_modules;
 
-#[derive(Clone)]
+#[derive(Clone, Debug, serde::Serialize, serde::Deserialize)]
 pub struct Table {
     pub schema: SchemaRef,
     columns: Vec<Series>,
@@ -67,6 +71,13 @@ impl Table {
         })
     }
 
+    pub fn new_unchecked<S: Into<SchemaRef>>(schema: S, columns: Vec<Series>) -> Self {
+        Table {
+            schema: schema.into(),
+            columns,
+        }
+    }
+
     pub fn empty(schema: Option<SchemaRef>) -> DaftResult<Self> {
         match schema {
             Some(schema) => {
@@ -126,13 +137,36 @@ impl Table {
         self.slice(0, num)
     }
 
-    pub fn sample(&self, num: usize) -> DaftResult<Self> {
+    pub fn sample_by_fraction(
+        &self,
+        fraction: f64,
+        with_replacement: bool,
+        seed: Option<u64>,
+    ) -> DaftResult<Self> {
+        let num = (fraction * self.len() as f64).ceil() as usize;
+        self.sample(num, with_replacement, seed)
+    }
+
+    pub fn sample(
+        &self,
+        num: usize,
+        with_replacement: bool,
+        seed: Option<u64>,
+    ) -> DaftResult<Self> {
         if num >= self.len() {
             Ok(self.clone())
         } else {
-            use rand::{distributions::Uniform, Rng};
+            use rand::{distributions::Uniform, rngs::StdRng, Rng, SeedableRng};
+            let mut rng = match seed {
+                Some(seed) => StdRng::seed_from_u64(seed),
+                None => StdRng::from_rng(rand::thread_rng()).unwrap(),
+            };
             let range = Uniform::from(0..self.len() as u64);
-            let values: Vec<u64> = rand::thread_rng().sample_iter(&range).take(num).collect();
+            let values: Vec<u64> = if with_replacement {
+                (0..num).map(|_| rng.sample(range)).collect()
+            } else {
+                rng.sample_iter(&range).take(num).collect()
+            };
             let indices: daft_core::array::DataArray<daft_core::datatypes::UInt64Type> =
                 UInt64Array::from(("idx", values));
             self.take(&indices.into_series())
@@ -170,16 +204,20 @@ impl Table {
         Ok(column_sizes?.iter().sum())
     }
 
-    pub fn filter(&self, predicate: &[Expr]) -> DaftResult<Self> {
+    pub fn filter<E: AsRef<Expr>>(&self, predicate: &[E]) -> DaftResult<Self> {
         if predicate.is_empty() {
             Ok(self.clone())
         } else if predicate.len() == 1 {
-            let mask = self.eval_expression(predicate.get(0).unwrap())?;
+            let mask = self.eval_expression(predicate.get(0).unwrap().as_ref())?;
             self.mask_filter(&mask)
         } else {
-            let mut expr = predicate.get(0).unwrap().and(predicate.get(1).unwrap());
+            let mut expr = predicate
+                .get(0)
+                .unwrap()
+                .as_ref()
+                .and(predicate.get(1).unwrap().as_ref());
             for i in 2..predicate.len() {
-                let next = predicate.get(i).unwrap();
+                let next = predicate.get(i).unwrap().as_ref();
                 expr = expr.and(next);
             }
             let mask = self.eval_expression(&expr)?;
@@ -208,19 +246,19 @@ impl Table {
         Ok(Table::new(self.schema.clone(), new_series?).unwrap())
     }
 
-    pub fn concat(tables: &[&Table]) -> DaftResult<Self> {
+    pub fn concat<T: AsRef<Table>>(tables: &[T]) -> DaftResult<Self> {
         if tables.is_empty() {
             return Err(DaftError::ValueError(
-                "Need at least 1 table to perform concat".to_string(),
+                "Need at least 1 Table to perform concat".to_string(),
             ));
         }
         if tables.len() == 1 {
-            return Ok((*tables.first().unwrap()).clone());
+            return Ok((*tables.first().unwrap().as_ref()).clone());
         }
-        let first_table = tables.first().unwrap();
+        let first_table = tables.first().unwrap().as_ref();
 
         let first_schema = first_table.schema.as_ref();
-        for tab in tables.iter().skip(1) {
+        for tab in tables.iter().skip(1).map(|t| t.as_ref()) {
             if tab.schema.as_ref() != first_schema {
                 return Err(DaftError::SchemaMismatch(format!(
                     "Table concat requires all schemas to match, {} vs {}",
@@ -233,7 +271,7 @@ impl Table {
         for i in 0..num_columns {
             let series_to_cat: Vec<&Series> = tables
                 .iter()
-                .map(|s| s.get_column_by_index(i).unwrap())
+                .map(|s| s.as_ref().get_column_by_index(i).unwrap())
                 .collect();
             new_series.push(Series::concat(series_to_cat.as_slice())?);
         }
@@ -287,6 +325,7 @@ impl Table {
             Column(name) => self.get_column(name).cloned(),
             Not(child) => !(self.eval_expression(child)?),
             IsNull(child) => self.eval_expression(child)?.is_null(),
+            NotNull(child) => self.eval_expression(child)?.not_null(),
             BinaryOp { op, left, right } => {
                 let lhs = self.eval_expression(left)?;
                 let rhs = self.eval_expression(right)?;
@@ -466,77 +505,31 @@ impl Table {
         res
     }
 
-    pub fn to_prettytable(&self, max_col_width: Option<usize>) -> prettytable::Table {
-        let mut table = prettytable::Table::new();
-        let header = self
-            .schema
-            .fields
-            .iter()
-            .map(|(name, field)| {
-                prettytable::Cell::new(format!("{}\n{}", name, field.dtype).as_str())
-                    .with_style(prettytable::Attr::Bold)
-            })
-            .collect();
-        table.add_row(header);
-
-        let head_rows;
-        let tail_rows;
-
-        if self.len() > 10 {
-            head_rows = 5;
-            tail_rows = 5;
-        } else {
-            head_rows = self.len();
-            tail_rows = 0;
-        }
-
-        for i in 0..head_rows {
-            let row = self
-                .columns
-                .iter()
-                .map(|s| {
-                    let mut str_val = s.str_value(i).unwrap();
-                    if let Some(max_col_width) = max_col_width {
-                        if str_val.len() > max_col_width {
-                            str_val = format!("{}...", &str_val[..max_col_width - 3]);
-                        }
-                    }
-                    str_val
-                })
-                .collect::<Vec<String>>();
-            table.add_row(row.into());
-        }
-        if tail_rows != 0 {
-            let row: prettytable::Row = (0..self.num_columns()).map(|_| "...").collect();
-            table.add_row(row);
-        }
-
-        for i in (self.len() - tail_rows)..(self.len()) {
-            let row = self
-                .columns
-                .iter()
-                .map(|s| {
-                    let mut str_val = s.str_value(i).unwrap();
-                    if let Some(max_col_width) = max_col_width {
-                        if str_val.len() > max_col_width {
-                            str_val = format!("{}...", &str_val[..max_col_width - 3]);
-                        }
-                    }
-                    str_val
-                })
-                .collect::<Vec<String>>();
-            table.add_row(row.into());
-        }
-
-        table
+    pub fn to_comfy_table(&self, max_col_width: Option<usize>) -> comfy_table::Table {
+        make_comfy_table(
+            self.schema
+                .fields
+                .values()
+                .map(Cow::Borrowed)
+                .collect::<Vec<_>>()
+                .as_slice(),
+            Some(self.columns.iter().collect::<Vec<_>>().as_slice()),
+            max_col_width,
+        )
     }
 }
 
 impl Display for Table {
     // `f` is a buffer, and this method must write the formatted string into it
     fn fmt(&self, f: &mut Formatter) -> Result {
-        let table = self.to_prettytable(Some(32));
-        write!(f, "{table}")
+        let table = self.to_comfy_table(Some(32));
+        writeln!(f, "{table}")
+    }
+}
+
+impl AsRef<Table> for Table {
+    fn as_ref(&self) -> &Table {
+        self
     }
 }
 
