@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import uuid
 from typing import List, Tuple
 
 import pandas as pd
@@ -13,9 +14,20 @@ COLUMNS = ["Unique Key", "Complaint Type", "Borough", "Created Date", "Descripto
 CsvPathAndColumns = Tuple[str, List[str]]
 
 
-@pytest.fixture(scope="function")
-def daft_df():
-    return daft.read_csv(COOKBOOK_DATA_CSV).select(*[col(c) for c in COLUMNS])
+@pytest.fixture(scope="function", params=["parquet", "csv"])
+def daft_df(request, tmp_path):
+    if request.param == "csv":
+        df = daft.read_csv(COOKBOOK_DATA_CSV)
+    elif request.param == "parquet":
+        import pyarrow.csv as pacsv
+        import pyarrow.parquet as papq
+
+        tmp_file = tmp_path / str(uuid.uuid4())
+        papq.write_table(pacsv.read_csv(COOKBOOK_DATA_CSV), str(tmp_file))
+        df = daft.read_parquet(str(tmp_file))
+    else:
+        assert False, f"Can only handle CSV/Parquet formats"
+    return df.select(*[col(c) for c in COLUMNS])
 
 
 @pytest.fixture(scope="function")
