@@ -7,19 +7,16 @@ import json
 import os
 import tempfile
 import uuid
-from unittest.mock import MagicMock, patch
 
 import numpy as np
 import pandas as pd
 import pyarrow as pa
 import pyarrow.parquet as papq
 import pytest
-from fsspec.implementations.local import LocalFileSystem
 from ray.data.extensions import ArrowTensorArray, TensorArray
 
 import daft
 from daft.api_annotations import APITypeError
-from daft.context import get_context
 from daft.dataframe import DataFrame
 from daft.datatype import DataType
 from daft.utils import pyarrow_supports_fixed_shape_tensor
@@ -356,7 +353,8 @@ def create_temp_filename() -> str:
         yield os.path.join(dir, "tempfile")
 
 
-def test_create_dataframe_csv(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_csv(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             header = list(valid_data[0].keys())
@@ -365,7 +363,7 @@ def test_create_dataframe_csv(valid_data: list[dict[str, float]]) -> None:
             writer.writerows([[item[col] for col in header] for item in valid_data])
             f.flush()
 
-        df = daft.read_csv(fname)
+        df = daft.read_csv(fname, use_native_downloader=use_native_downloader)
         assert df.column_names == COL_NAMES
 
         pd_df = df.to_pandas()
@@ -373,7 +371,8 @@ def test_create_dataframe_csv(valid_data: list[dict[str, float]]) -> None:
         assert len(pd_df) == len(valid_data)
 
 
-def test_create_dataframe_multiple_csvs(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_multiple_csvs(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with create_temp_filename() as f1name, create_temp_filename() as f2name:
         with open(f1name, "w") as f1, open(f2name, "w") as f2:
             for f in (f1, f2):
@@ -383,7 +382,7 @@ def test_create_dataframe_multiple_csvs(valid_data: list[dict[str, float]]) -> N
                 writer.writerows([[item[col] for col in header] for item in valid_data])
                 f.flush()
 
-        df = daft.read_csv([f1name, f2name])
+        df = daft.read_csv([f1name, f2name], use_native_downloader=use_native_downloader)
         assert df.column_names == COL_NAMES
 
         pd_df = df.to_pandas()
@@ -391,42 +390,8 @@ def test_create_dataframe_multiple_csvs(valid_data: list[dict[str, float]]) -> N
         assert len(pd_df) == (len(valid_data) * 2)
 
 
-@pytest.mark.skipif(
-    get_context().runner_config.name not in {"py"},
-    reason="requires PyRunner to be in use",
-)
-def test_create_dataframe_csv_custom_fs(valid_data: list[dict[str, float]]) -> None:
-    with create_temp_filename() as fname:
-        with open(fname, "w") as f:
-            header = list(valid_data[0].keys())
-            writer = csv.writer(f)
-            writer.writerow(header)
-            writer.writerows([[item[col] for col in header] for item in valid_data])
-            f.flush()
-
-        # Mark that this filesystem instance shouldn't be automatically reused by fsspec; without this,
-        # fsspec would cache this instance and reuse it for Daft's default construction of filesystems,
-        # which would make this test pass without the passed filesystem being used.
-        fs = LocalFileSystem(skip_instance_cache=True)
-        mock_cache = MagicMock(return_value=None)
-        with patch.object(fs, "info", wraps=fs.info) as mock_info, patch.object(
-            fs, "open", wraps=fs.open
-        ) as mock_open, patch("daft.filesystem._get_fs_from_cache", mock_cache):
-            df = daft.read_csv(fname, fs=fs)
-            # Check that info() is called on the passed filesystem.
-            mock_info.assert_called()
-
-            assert df.column_names == COL_NAMES
-            pd_df = df.to_pandas()
-
-            # Check that open() is called on the passed filesystem.
-            mock_open.assert_called()
-
-        assert list(pd_df.columns) == COL_NAMES
-        assert len(pd_df) == len(valid_data)
-
-
-def test_create_dataframe_csv_generate_headers(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_csv_generate_headers(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             header = list(valid_data[0].keys())
@@ -434,8 +399,12 @@ def test_create_dataframe_csv_generate_headers(valid_data: list[dict[str, float]
             writer.writerows([[item[col] for col in header] for item in valid_data])
             f.flush()
 
-        cnames = [f"f{i}" for i in range(5)]
-        df = daft.read_csv(fname, has_headers=False)
+        cnames = (
+            [f"column_{i}" for i in range(1, 6)]
+            if use_native_downloader or os.environ.get("DAFT_MICROPARTITIONS", "1") == "1"
+            else [f"f{i}" for i in range(5)]
+        )
+        df = daft.read_csv(fname, has_headers=False, use_native_downloader=use_native_downloader)
         assert df.column_names == cnames
 
         pd_df = df.to_pandas()
@@ -443,7 +412,8 @@ def test_create_dataframe_csv_generate_headers(valid_data: list[dict[str, float]
         assert len(pd_df) == len(valid_data)
 
 
-def test_create_dataframe_csv_column_projection(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_csv_column_projection(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             header = list(valid_data[0].keys())
@@ -463,7 +433,8 @@ def test_create_dataframe_csv_column_projection(valid_data: list[dict[str, float
         assert len(pd_df) == len(valid_data)
 
 
-def test_create_dataframe_csv_custom_delimiter(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_csv_custom_delimiter(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             header = list(valid_data[0].keys())
@@ -472,7 +443,7 @@ def test_create_dataframe_csv_custom_delimiter(valid_data: list[dict[str, float]
             writer.writerows([[item[col] for col in header] for item in valid_data])
             f.flush()
 
-        df = daft.read_csv(fname, delimiter="\t")
+        df = daft.read_csv(fname, delimiter="\t", use_native_downloader=use_native_downloader)
         assert df.column_names == COL_NAMES
 
         pd_df = df.to_pandas()
@@ -480,7 +451,8 @@ def test_create_dataframe_csv_custom_delimiter(valid_data: list[dict[str, float]
         assert len(pd_df) == len(valid_data)
 
 
-def test_create_dataframe_csv_specify_schema(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_csv_specify_schema(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             header = list(valid_data[0].keys())
@@ -499,6 +471,7 @@ def test_create_dataframe_csv_specify_schema(valid_data: list[dict[str, float]])
                 "petal_width": DataType.float32(),
                 "variety": DataType.string(),
             },
+            use_native_downloader=use_native_downloader,
         )
         assert df.column_names == COL_NAMES
 
@@ -507,7 +480,10 @@ def test_create_dataframe_csv_specify_schema(valid_data: list[dict[str, float]])
         assert len(pd_df) == len(valid_data)
 
 
-def test_create_dataframe_csv_specify_schema_no_headers(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_csv_specify_schema_no_headers(
+    valid_data: list[dict[str, float]], use_native_downloader
+) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             header = list(valid_data[0].keys())
@@ -515,17 +491,112 @@ def test_create_dataframe_csv_specify_schema_no_headers(valid_data: list[dict[st
             writer.writerows([[item[col] for col in header] for item in valid_data])
             f.flush()
 
+        schema_hints_for_csv_without_headers = {
+            "column_1": DataType.float64(),
+            "column_2": DataType.float64(),
+            "column_3": DataType.float64(),
+            "column_4": DataType.float64(),
+            "column_5": DataType.string(),
+        }
+
+        if use_native_downloader == False and os.environ.get("DAFT_MICROPARTITIONS") == "0":
+            schema_hints_for_csv_without_headers = {
+                "f0": DataType.float64(),
+                "f1": DataType.float64(),
+                "f2": DataType.float64(),
+                "f3": DataType.float64(),
+                "f4": DataType.string(),
+            }
+
+        df = daft.read_csv(
+            fname,
+            delimiter="\t",
+            schema_hints=schema_hints_for_csv_without_headers,
+            has_headers=False,
+            use_native_downloader=use_native_downloader,
+        )
+        assert df.column_names == list(schema_hints_for_csv_without_headers.keys())
+
+        pd_df = df.to_pandas()
+        assert list(pd_df.columns) == list(schema_hints_for_csv_without_headers.keys())
+        assert len(pd_df) == len(valid_data)
+
+
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_csv_schema_hints_partial(valid_data: list[dict[str, float]], use_native_downloader) -> None:
+    with create_temp_filename() as fname:
+        with open(fname, "w") as f:
+            header = list(valid_data[0].keys())
+            writer = csv.writer(f, delimiter="\t")
+            writer.writerow(header)
+            writer.writerows([[item[col] for col in header] for item in valid_data])
+            f.flush()
+
         df = daft.read_csv(
             fname,
             delimiter="\t",
             schema_hints={
                 "sepal_length": DataType.float64(),
                 "sepal_width": DataType.float64(),
-                "petal_length": DataType.float64(),
-                "petal_width": DataType.float64(),
-                "variety": DataType.string(),
             },
-            has_headers=False,
+            use_native_downloader=use_native_downloader,
+        )
+        assert df.column_names == COL_NAMES
+
+        pd_df = df.to_pandas()
+        assert list(pd_df.columns) == COL_NAMES
+        assert len(pd_df) == len(valid_data)
+
+
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_csv_schema_hints_override_types(
+    valid_data: list[dict[str, float]], use_native_downloader
+) -> None:
+    with create_temp_filename() as fname:
+        with open(fname, "w") as f:
+            header = list(valid_data[0].keys())
+            writer = csv.writer(f, delimiter="\t")
+            writer.writerow(header)
+            writer.writerows([[item[col] for col in header] for item in valid_data])
+            f.flush()
+
+        df = daft.read_csv(
+            fname,
+            delimiter="\t",
+            schema_hints={
+                "sepal_length": DataType.string(),  # Override the inferred float64 type to string
+            },
+            use_native_downloader=use_native_downloader,
+        )
+        assert df.column_names == COL_NAMES
+
+        pd_df = df.to_pandas()
+        assert list(pd_df.columns) == COL_NAMES
+        assert len(pd_df) == len(valid_data)
+
+        assert pd_df["sepal_length"].dtype == "object"
+        assert pd_df["sepal_length"][0] == str(valid_data[0]["sepal_length"])
+
+
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_csv_schema_hints_ignore_random_hint(
+    valid_data: list[dict[str, float]], use_native_downloader
+) -> None:
+    with create_temp_filename() as fname:
+        with open(fname, "w") as f:
+            header = list(valid_data[0].keys())
+            writer = csv.writer(f, delimiter="\t")
+            writer.writerow(header)
+            writer.writerows([[item[col] for col in header] for item in valid_data])
+            f.flush()
+
+        df = daft.read_csv(
+            fname,
+            delimiter="\t",
+            schema_hints={
+                "foo": DataType.string(),  # Random column name that is not in the table
+            },
+            use_native_downloader=use_native_downloader,
         )
         assert df.column_names == COL_NAMES
 
@@ -539,7 +610,8 @@ def test_create_dataframe_csv_specify_schema_no_headers(valid_data: list[dict[st
 ###
 
 
-def test_create_dataframe_json(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_json(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             for data in valid_data:
@@ -547,7 +619,7 @@ def test_create_dataframe_json(valid_data: list[dict[str, float]]) -> None:
                 f.write("\n")
             f.flush()
 
-        df = daft.read_json(fname)
+        df = daft.read_json(fname, use_native_downloader=use_native_downloader)
         assert df.column_names == COL_NAMES
 
         pd_df = df.to_pandas()
@@ -555,7 +627,8 @@ def test_create_dataframe_json(valid_data: list[dict[str, float]]) -> None:
         assert len(pd_df) == len(valid_data)
 
 
-def test_create_dataframe_multiple_jsons(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_multiple_jsons(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with create_temp_filename() as f1name, create_temp_filename() as f2name:
         with open(f1name, "w") as f1, open(f2name, "w") as f2:
             for f in (f1, f2):
@@ -564,7 +637,7 @@ def test_create_dataframe_multiple_jsons(valid_data: list[dict[str, float]]) ->
                     f.write("\n")
                 f.flush()
 
-        df = daft.read_json([f1name, f2name])
+        df = daft.read_json([f1name, f2name], use_native_downloader=use_native_downloader)
         assert df.column_names == COL_NAMES
 
         pd_df = df.to_pandas()
@@ -572,42 +645,8 @@ def test_create_dataframe_multiple_jsons(valid_data: list[dict[str, float]]) ->
         assert len(pd_df) == (len(valid_data) * 2)
 
 
-@pytest.mark.skipif(
-    get_context().runner_config.name not in {"py"},
-    reason="requires PyRunner to be in use",
-)
-def test_create_dataframe_json_custom_fs(valid_data: list[dict[str, float]]) -> None:
-    with create_temp_filename() as fname:
-        with open(fname, "w") as f:
-            for data in valid_data:
-                f.write(json.dumps(data))
-                f.write("\n")
-            f.flush()
-
-        # Mark that this filesystem instance shouldn't be automatically reused by fsspec; without this,
-        # fsspec would cache this instance and reuse it for Daft's default construction of filesystems,
-        # which would make this test pass without the passed filesystem being used.
-        fs = LocalFileSystem(skip_instance_cache=True)
-        mock_cache = MagicMock(return_value=None)
-        with patch.object(fs, "info", wraps=fs.info) as mock_info, patch.object(
-            fs, "open", wraps=fs.open
-        ) as mock_open, patch("daft.filesystem._get_fs_from_cache", mock_cache):
-            df = daft.read_json(fname, fs=fs)
-
-            # Check that info() is called on the passed filesystem.
-            mock_info.assert_called()
-
-            assert df.column_names == COL_NAMES
-            pd_df = df.to_pandas()
-
-            # Check that open() is called on the passed filesystem.
-            mock_open.assert_called()
-
-        assert list(pd_df.columns) == COL_NAMES
-        assert len(pd_df) == len(valid_data)
-
-
-def test_create_dataframe_json_column_projection(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_json_column_projection(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             for data in valid_data:
@@ -617,7 +656,7 @@ def test_create_dataframe_json_column_projection(valid_data: list[dict[str, floa
 
         col_subset = COL_NAMES[:3]
 
-        df = daft.read_json(fname)
+        df = daft.read_json(fname, use_native_downloader=use_native_downloader)
         df = df.select(*col_subset)
         assert df.column_names == col_subset
 
@@ -626,14 +665,21 @@ def test_create_dataframe_json_column_projection(valid_data: list[dict[str, floa
         assert len(pd_df) == len(valid_data)
 
 
+# TODO(Clark): Debug why this segfaults for the native downloader and is slow for the Python downloader.
+# @pytest.mark.parametrize("use_native_downloader", [True, False])
+@pytest.mark.skip
 def test_create_dataframe_json_https() -> None:
-    df = daft.read_json("https://github.com/Eventual-Inc/mnist-json/raw/master/mnist_handwritten_test.json.gz")
+    df = daft.read_json(
+        "https://github.com/Eventual-Inc/mnist-json/raw/master/mnist_handwritten_test.json.gz",
+        # use_native_downloader=use_native_downloader,
+    )
     df.collect()
     assert set(df.column_names) == {"label", "image"}
     assert len(df) == 10000
 
 
-def test_create_dataframe_json_specify_schema(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_json_specify_schema(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             for data in valid_data:
@@ -650,6 +696,74 @@ def test_create_dataframe_json_specify_schema(valid_data: list[dict[str, float]]
                 "petal_width": DataType.float32(),
                 "variety": DataType.string(),
             },
+            use_native_downloader=use_native_downloader,
+        )
+        assert df.column_names == COL_NAMES
+
+        pd_df = df.to_pandas()
+        assert list(pd_df.columns) == COL_NAMES
+        assert len(pd_df) == len(valid_data)
+
+
+def test_create_dataframe_json_schema_hints_partial(valid_data: list[dict[str, float]]) -> None:
+    with create_temp_filename() as fname:
+        with open(fname, "w") as f:
+            for data in valid_data:
+                f.write(json.dumps(data))
+                f.write("\n")
+            f.flush()
+
+        df = daft.read_json(
+            fname,
+            schema_hints={
+                "sepal_length": DataType.float64(),
+                "sepal_width": DataType.float64(),
+            },
+        )
+        assert df.column_names == COL_NAMES
+
+        pd_df = df.to_pandas()
+        assert list(pd_df.columns) == COL_NAMES
+        assert len(pd_df) == len(valid_data)
+
+
+def test_create_dataframe_json_schema_hints_override_types(valid_data: list[dict[str, float]]) -> None:
+    with create_temp_filename() as fname:
+        with open(fname, "w") as f:
+            for data in valid_data:
+                f.write(json.dumps(data))
+                f.write("\n")
+            f.flush()
+
+        df = daft.read_json(
+            fname,
+            schema_hints={
+                "sepal_length": DataType.string(),  # Override the inferred float64 type to string
+            },
+        )
+        assert df.column_names == COL_NAMES
+
+        pd_df = df.to_pandas()
+        assert list(pd_df.columns) == COL_NAMES
+        assert len(pd_df) == len(valid_data)
+
+        assert pd_df["sepal_length"].dtype == "object"
+        assert pd_df["sepal_length"][0] == str(valid_data[0]["sepal_length"])
+
+
+def test_create_dataframe_json_schema_hints_ignore_random_hint(valid_data: list[dict[str, float]]) -> None:
+    with create_temp_filename() as fname:
+        with open(fname, "w") as f:
+            for data in valid_data:
+                f.write(json.dumps(data))
+                f.write("\n")
+            f.flush()
+
+        df = daft.read_json(
+            fname,
+            schema_hints={
+                "foo": DataType.string(),  # Random column name that is not in the table
+            },
         )
         assert df.column_names == COL_NAMES
 
@@ -663,15 +777,14 @@ def test_create_dataframe_json_specify_schema(valid_data: list[dict[str, float]]
 ###
 
 
-@pytest.mark.parametrize("use_native_downloader", [True, False])
-def test_create_dataframe_parquet(valid_data: list[dict[str, float]], use_native_downloader) -> None:
+def test_create_dataframe_parquet(valid_data: list[dict[str, float]]) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
             papq.write_table(table, f.name)
             f.flush()
 
-        df = daft.read_parquet(fname, use_native_downloader=use_native_downloader)
+        df = daft.read_parquet(fname)
         assert df.column_names == COL_NAMES
 
         pd_df = df.to_pandas()
@@ -679,15 +792,14 @@ def test_create_dataframe_parquet(valid_data: list[dict[str, float]], use_native
         assert len(pd_df) == len(valid_data)
 
 
-@pytest.mark.parametrize("use_native_downloader", [True, False])
-def test_create_dataframe_parquet_with_filter(valid_data: list[dict[str, float]], use_native_downloader) -> None:
+def test_create_dataframe_parquet_with_filter(valid_data: list[dict[str, float]]) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
             papq.write_table(table, f.name)
             f.flush()
 
-        df = daft.read_parquet(fname, use_native_downloader=use_native_downloader)
+        df = daft.read_parquet(fname)
         assert df.column_names == COL_NAMES
 
         df = df.where(daft.col("sepal_length") > 4.8)
@@ -697,8 +809,7 @@ def test_create_dataframe_parquet_with_filter(valid_data: list[dict[str, float]]
         assert len(pd_df) == len(valid_data) - 1
 
 
-@pytest.mark.parametrize("use_native_downloader", [True, False])
-def test_create_dataframe_multiple_parquets(valid_data: list[dict[str, float]], use_native_downloader) -> None:
+def test_create_dataframe_multiple_parquets(valid_data: list[dict[str, float]]) -> None:
     with create_temp_filename() as f1name, create_temp_filename() as f2name:
         with open(f1name, "w") as f1, open(f2name, "w") as f2:
             for f in (f1, f2):
@@ -706,7 +817,7 @@ def test_create_dataframe_multiple_parquets(valid_data: list[dict[str, float]],
                 papq.write_table(table, f.name)
                 f.flush()
 
-        df = daft.read_parquet([f1name, f2name], use_native_downloader=use_native_downloader)
+        df = daft.read_parquet([f1name, f2name])
         assert df.column_names == COL_NAMES
 
         pd_df = df.to_pandas()
@@ -714,42 +825,7 @@ def test_create_dataframe_multiple_parquets(valid_data: list[dict[str, float]],
         assert len(pd_df) == (len(valid_data) * 2)
 
 
-@pytest.mark.skipif(
-    get_context().runner_config.name not in {"py"},
-    reason="requires PyRunner to be in use",
-)
-def test_create_dataframe_parquet_custom_fs(valid_data: list[dict[str, float]]) -> None:
-    with create_temp_filename() as fname:
-        with open(fname, "w") as f:
-            table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
-            papq.write_table(table, f.name)
-            f.flush()
-
-        # Mark that this filesystem instance shouldn't be automatically reused by fsspec; without this,
-        # fsspec would cache this instance and reuse it for Daft's default construction of filesystems,
-        # which would make this test pass without the passed filesystem being used.
-        fs = LocalFileSystem(skip_instance_cache=True)
-        mock_cache = MagicMock(return_value=None)
-        with patch.object(fs, "info", wraps=fs.info) as mock_info, patch.object(
-            fs, "open", wraps=fs.open
-        ) as mock_open, patch("daft.filesystem._get_fs_from_cache", mock_cache):
-            df = daft.read_parquet(fname, fs=fs)
-
-            # Check that info() is called on the passed filesystem.
-            mock_info.assert_called()
-
-            assert df.column_names == COL_NAMES
-            pd_df = df.to_pandas()
-
-            # Check that open() is called on the passed filesystem.
-            mock_open.assert_called()
-
-        assert list(pd_df.columns) == COL_NAMES
-        assert len(pd_df) == len(valid_data)
-
-
-@pytest.mark.parametrize("use_native_downloader", [True, False])
-def test_create_dataframe_parquet_column_projection(valid_data: list[dict[str, float]], use_native_downloader) -> None:
+def test_create_dataframe_parquet_column_projection(valid_data: list[dict[str, float]]) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
@@ -758,7 +834,7 @@ def test_create_dataframe_parquet_column_projection(valid_data: list[dict[str, f
 
         col_subset = COL_NAMES[:3]
 
-        df = daft.read_parquet(fname, use_native_downloader=use_native_downloader)
+        df = daft.read_parquet(fname)
         df = df.select(*col_subset)
         assert df.column_names == col_subset
 
@@ -767,8 +843,7 @@ def test_create_dataframe_parquet_column_projection(valid_data: list[dict[str, f
         assert len(pd_df) == len(valid_data)
 
 
-@pytest.mark.parametrize("use_native_downloader", [True, False])
-def test_create_dataframe_parquet_specify_schema(valid_data: list[dict[str, float]], use_native_downloader) -> None:
+def test_create_dataframe_parquet_specify_schema(valid_data: list[dict[str, float]]) -> None:
     with create_temp_filename() as fname:
         with open(fname, "w") as f:
             table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
@@ -784,7 +859,70 @@ def test_create_dataframe_parquet_specify_schema(valid_data: list[dict[str, floa
                 "petal_width": DataType.float64(),
                 "variety": DataType.string(),
             },
-            use_native_downloader=use_native_downloader,
+        )
+        assert df.column_names == COL_NAMES
+
+        pd_df = df.to_pandas()
+        assert list(pd_df.columns) == COL_NAMES
+        assert len(pd_df) == len(valid_data)
+
+
+def test_create_dataframe_parquet_schema_hints_partial(valid_data: list[dict[str, float]]) -> None:
+    with create_temp_filename() as fname:
+        with open(fname, "w") as f:
+            table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
+            papq.write_table(table, fname)
+            f.flush()
+
+        df = daft.read_parquet(
+            fname,
+            schema_hints={
+                "sepal_length": DataType.float64(),
+                "sepal_width": DataType.float64(),
+            },
+        )
+        assert df.column_names == COL_NAMES
+
+        pd_df = df.to_pandas()
+        assert list(pd_df.columns) == COL_NAMES
+        assert len(pd_df) == len(valid_data)
+
+
+def test_create_dataframe_parquet_schema_hints_override_types(valid_data: list[dict[str, float]]) -> None:
+    with create_temp_filename() as fname:
+        with open(fname, "w") as f:
+            table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
+            papq.write_table(table, fname)
+            f.flush()
+
+        df = daft.read_parquet(
+            fname,
+            schema_hints={
+                "sepal_length": DataType.string(),  # Override the inferred float64 type to string
+            },
+        )
+        assert df.column_names == COL_NAMES
+
+        pd_df = df.to_pandas()
+        assert list(pd_df.columns) == COL_NAMES
+        assert len(pd_df) == len(valid_data)
+
+        assert pd_df["sepal_length"].dtype == "object"
+        assert pd_df["sepal_length"][0] == str(valid_data[0]["sepal_length"])
+
+
+def test_create_dataframe_parquet_schema_hints_ignore_random_hint(valid_data: list[dict[str, float]]) -> None:
+    with create_temp_filename() as fname:
+        with open(fname, "w") as f:
+            table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
+            papq.write_table(table, fname)
+            f.flush()
+
+        df = daft.read_parquet(
+            fname,
+            schema_hints={
+                "foo": DataType.string(),  # Random column name that is not in the table
+            },
         )
         assert df.column_names == COL_NAMES
 
