@@ -42,7 +42,8 @@ def test_temporal_arithmetic() -> None:
 
 
 @pytest.mark.parametrize("format", ["csv", "parquet"])
-def test_temporal_file_roundtrip(format) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_temporal_file_roundtrip(format, use_native_downloader) -> None:
     data = {
         "date32": pa.array([1], pa.date32()),
         "date64": pa.array([1], pa.date64()),
@@ -69,9 +70,12 @@ def test_temporal_file_roundtrip(format) -> None:
             "timestamp_s": pa.array([1], pa.timestamp("s")),
             "timestamp_ms": pa.array([1], pa.timestamp("ms")),
             "timestamp_us": pa.array([1], pa.timestamp("us")),
-            "timestamp_s_tz": pa.array([1], pa.timestamp("s", tz="UTC")),
-            "timestamp_ms_tz": pa.array([1], pa.timestamp("ms", tz="UTC")),
-            "timestamp_us_tz": pa.array([1], pa.timestamp("us", tz="UTC")),
+            "timestamp_s_utc_tz": pa.array([1], pa.timestamp("s", tz="UTC")),
+            "timestamp_ms_utc_tz": pa.array([1], pa.timestamp("ms", tz="UTC")),
+            "timestamp_us_utc_tz": pa.array([1], pa.timestamp("us", tz="UTC")),
+            "timestamp_s_tz": pa.array([1], pa.timestamp("s", tz="Asia/Singapore")),
+            "timestamp_ms_tz": pa.array([1], pa.timestamp("ms", tz="Asia/Singapore")),
+            "timestamp_us_tz": pa.array([1], pa.timestamp("us", tz="Asia/Singapore")),
         }
 
     pa_table = pa.Table.from_pydict(data)
@@ -81,10 +85,10 @@ def test_temporal_file_roundtrip(format) -> None:
     with tempfile.TemporaryDirectory() as dirname:
         if format == "csv":
             df.write_csv(dirname)
-            df_readback = daft.read_csv(dirname).collect()
+            df_readback = daft.read_csv(dirname, use_native_downloader=use_native_downloader).collect()
         elif format == "parquet":
             df.write_parquet(dirname)
-            df_readback = daft.read_parquet(dirname).collect()
+            df_readback = daft.read_parquet(dirname, use_native_downloader=use_native_downloader).collect()
 
     assert df.to_pydict() == df_readback.to_pydict()
 
