@@ -1,18 +1,16 @@
 from __future__ import annotations
 
-import daft
 
-
-def test_with_column(valid_data: list[dict[str, float]]) -> None:
-    df = daft.from_pylist(valid_data)
+def test_with_column(make_df, valid_data: list[dict[str, float]]) -> None:
+    df = make_df(valid_data)
     expanded_df = df.with_column("bar", df["sepal_width"] + df["petal_length"])
     data = expanded_df.to_pydict()
     assert expanded_df.column_names == df.column_names + ["bar"]
     assert data["bar"] == [sw + pl for sw, pl in zip(data["sepal_width"], data["petal_length"])]
 
 
-def test_stacked_with_columns(valid_data: list[dict[str, float]]):
-    df = daft.from_pylist(valid_data)
+def test_stacked_with_columns(make_df, valid_data: list[dict[str, float]]):
+    df = make_df(valid_data)
     df = df.select(df["sepal_length"])
     df = df.with_column("sepal_length_2", df["sepal_length"])
     df = df.with_column("sepal_length_3", df["sepal_length_2"])
