@@ -4,6 +4,8 @@ import datetime
 import itertools
 import sys
 
+import pytz
+
 if sys.version_info < (3, 8):
     pass
 else:
@@ -16,7 +18,7 @@ import pytest
 from daft.datatype import DataType
 from daft.expressions import Expression, ExpressionsProjection
 from daft.series import Series
-from daft.table import Table
+from daft.table import MicroPartition
 
 ALL_DTYPES = [
     (DataType.int8(), pa.array([1, 2, None], type=pa.int8())),
@@ -33,17 +35,65 @@ ALL_DTYPES = [
     (DataType.bool(), pa.array([True, False, None], type=pa.bool_())),
     (DataType.null(), pa.array([None, None, None], type=pa.null())),
     (DataType.binary(), pa.array([b"1", b"2", None], type=pa.binary())),
-    (DataType.date(), pa.array([datetime.date(2021, 1, 1), datetime.date(2021, 1, 2), None], type=pa.date32())),
-    # TODO(jay): Some of the fixtures are broken/become very complicated when testing against timestamps
-    # (
-    #     DataType.timestamp(TimeUnit.ms()),
-    #     pa.array([datetime.datetime(2021, 1, 1), datetime.datetime(2021, 1, 2), None], type=pa.timestamp("ms")),
-    # ),
 ]
 
 ALL_DATATYPES_BINARY_PAIRS = list(itertools.product(ALL_DTYPES, repeat=2))
 
 
+ALL_TEMPORAL_DTYPES = [
+    (DataType.date(), pa.array([datetime.date(2021, 1, 1), datetime.date(2021, 1, 2), None], type=pa.date32())),
+    *[
+        (
+            DataType.timestamp(unit),
+            pa.array([datetime.datetime(2021, 1, 1), datetime.datetime(2021, 1, 2), None], type=pa.timestamp(unit)),
+        )
+        for unit in ["ns", "us", "ms"]
+    ],
+    *[
+        (
+            DataType.timestamp(unit, "US/Eastern"),
+            pa.array(
+                [
+                    datetime.datetime(2021, 1, 1).astimezone(pytz.timezone("US/Eastern")),
+                    datetime.datetime(2021, 1, 2).astimezone(pytz.timezone("US/Eastern")),
+                    None,
+                ],
+                type=pa.timestamp(unit, "US/Eastern"),
+            ),
+        )
+        for unit in ["ns", "us", "ms"]
+    ],
+    *[
+        (
+            DataType.timestamp(unit, "Africa/Accra"),
+            pa.array(
+                [
+                    datetime.datetime(2021, 1, 1).astimezone(pytz.timezone("Africa/Accra")),
+                    datetime.datetime(2021, 1, 2).astimezone(pytz.timezone("Africa/Accra")),
+                    None,
+                ],
+                type=pa.timestamp(unit, "Africa/Accra"),
+            ),
+        )
+        for unit in ["ns", "us", "ms"]
+    ],
+]
+
+ALL_DTYPES += ALL_TEMPORAL_DTYPES
+
+ALL_TEMPORAL_DATATYPES_BINARY_PAIRS = [
+    ((dt1, data1), (dt2, data2))
+    for (dt1, data1), (dt2, data2) in itertools.product(ALL_TEMPORAL_DTYPES, repeat=2)
+    if not (
+        pa.types.is_timestamp(data1.type)
+        and pa.types.is_timestamp(data2.type)
+        and (data1.type.tz is None) ^ (data2.type.tz is None)
+    )
+]
+
+ALL_DATATYPES_BINARY_PAIRS += ALL_TEMPORAL_DATATYPES_BINARY_PAIRS
+
+
 @pytest.fixture(
     scope="module",
     params=ALL_DATATYPES_BINARY_PAIRS,
@@ -93,11 +143,11 @@ def assert_typing_resolve_vs_runtime_behavior(
 
     Args:
         data: data to test against (generated using one of the provided fixtures, `{unary, binary}_data_fixture`)
-        expr (Expression): Expression used to run the kernel in a Table (use `.name()` of the generated data to refer to columns)
+        expr (Expression): Expression used to run the kernel in a MicroPartition (use `.name()` of the generated data to refer to columns)
         run_kernel (Callable): A lambda that will run the kernel directly on the generated Series' without going through the Expressions API
         resolvable (bool): Whether this kernel should be valid, given the datatypes of the generated Series'
     """
-    table = Table.from_pydict({s.name(): s for s in data})
+    table = MicroPartition.from_pydict({s.name(): s for s in data})
     projection = ExpressionsProjection([expr.alias("result")])
     if resolvable:
         # Check that schema resolution and Series runtime return the same datatype
@@ -138,6 +188,7 @@ def is_comparable(dt: DataType):
         or dt == DataType.bool()
         or dt == DataType.string()
         or dt == DataType.null()
+        or dt == DataType.binary()
         or dt._is_temporal_type()
     )
 
