@@ -1,13 +1,21 @@
 from __future__ import annotations
 
+import fsspec
 import pandas as pd
 import pyarrow as pa
 import pytest
 from pyarrow import parquet as pq
 
 import daft
-from daft.filesystem import get_filesystem_from_path, get_protocol_from_path
-from daft.table import Table
+from daft.filesystem import get_filesystem, get_protocol_from_path
+from daft.table import MicroPartition, Table
+
+
+def get_filesystem_from_path(path: str, **kwargs) -> fsspec.AbstractFileSystem:
+    protocol = get_protocol_from_path(path)
+    fs = get_filesystem(protocol, **kwargs)
+    return fs
+
 
 # Taken from our spreadsheet of files that Daft should be able to handle
 DAFT_CAN_READ_FILES = [
@@ -158,6 +166,10 @@ DAFT_CAN_READ_FILES = [
         "parquet-benchmarking/mvp",
         "s3://daft-public-data/test_fixtures/parquet-dev/mvp.parquet",
     ),
+    (
+        "parquet-benchmarking/s3a-mvp",
+        "s3a://daft-public-data/test_fixtures/parquet-dev/mvp.parquet",
+    ),
     (
         "azure/mvp",
         "az://public-anonymous/mvp.parquet",
@@ -190,7 +202,7 @@ def parquet_file(request) -> tuple[str, str]:
 
 def read_parquet_with_pyarrow(path) -> pa.Table:
     kwargs = {}
-    if get_protocol_from_path(path) == "s3":
+    if get_protocol_from_path(path) == "s3" or get_protocol_from_path(path) == "s3a":
         kwargs["anon"] = True
     if get_protocol_from_path(path) == "az":
         kwargs["account_name"] = "dafttestdata"
@@ -208,8 +220,10 @@ def read_parquet_with_pyarrow(path) -> pa.Table:
 )
 def test_parquet_read_table(parquet_file, public_storage_io_config, multithreaded_io):
     _, url = parquet_file
-    daft_native_read = Table.read_parquet(url, io_config=public_storage_io_config, multithreaded_io=multithreaded_io)
-    pa_read = Table.from_arrow(read_parquet_with_pyarrow(url))
+    daft_native_read = MicroPartition.read_parquet(
+        url, io_config=public_storage_io_config, multithreaded_io=multithreaded_io
+    )
+    pa_read = MicroPartition.from_arrow(read_parquet_with_pyarrow(url))
     assert daft_native_read.schema() == pa_read.schema()
     pd.testing.assert_frame_equal(daft_native_read.to_pandas(), pa_read.to_pandas())
 
@@ -231,29 +245,31 @@ def test_parquet_read_table_into_pyarrow(parquet_file, public_storage_io_config,
 
 
 @pytest.mark.integration()
-@pytest.mark.skipif(
-    daft.context.get_context().use_rust_planner, reason="Custom fsspec filesystems not supported in new query planner"
-)
 @pytest.mark.parametrize(
     "multithreaded_io",
     [False, True],
 )
 def test_parquet_read_table_bulk(parquet_file, public_storage_io_config, multithreaded_io):
     _, url = parquet_file
-    daft_native_reads = Table.read_parquet_bulk(
+    daft_native_reads = MicroPartition.read_parquet_bulk(
         [url] * 2, io_config=public_storage_io_config, multithreaded_io=multithreaded_io
     )
-    pa_read = Table.from_arrow(read_parquet_with_pyarrow(url))
+    pa_read = MicroPartition.from_arrow(read_parquet_with_pyarrow(url))
 
-    for daft_native_read in daft_native_reads:
-        assert daft_native_read.schema() == pa_read.schema()
-        pd.testing.assert_frame_equal(daft_native_read.to_pandas(), pa_read.to_pandas())
+    # Legacy Table returns a list[Table]
+    if MicroPartition == Table:
+        for daft_native_read in daft_native_reads:
+            assert daft_native_read.schema() == pa_read.schema()
+            pd.testing.assert_frame_equal(daft_native_read.to_pandas(), pa_read.to_pandas())
+    # MicroPartitions returns a MicroPartition
+    else:
+        assert daft_native_reads.schema() == pa_read.schema()
+        pd.testing.assert_frame_equal(
+            daft_native_reads.to_pandas(), MicroPartition.concat([pa_read, pa_read]).to_pandas()
+        )
 
 
 @pytest.mark.integration()
-@pytest.mark.skipif(
-    daft.context.get_context().use_rust_planner, reason="Custom fsspec filesystems not supported in new query planner"
-)
 @pytest.mark.parametrize(
     "multithreaded_io",
     [False, True],
@@ -273,16 +289,8 @@ def test_parquet_into_pyarrow_bulk(parquet_file, public_storage_io_config, multi
 @pytest.mark.integration()
 def test_parquet_read_df(parquet_file, public_storage_io_config):
     _, url = parquet_file
-    # This is a hack until we remove `fsspec.info`, `fsspec.glob` and `fsspec.glob` from  `daft.read_parquet`.
-    # We rely on the native downloaders impl for that
-    if url.startswith("az"):
-        import adlfs
-
-        fs = adlfs.AzureBlobFileSystem(account_name="dafttestdata", anon=True)
-    else:
-        fs = None
-    daft_native_read = daft.read_parquet(url, io_config=public_storage_io_config, use_native_downloader=True, fs=fs)
-    pa_read = Table.from_arrow(read_parquet_with_pyarrow(url))
+    daft_native_read = daft.read_parquet(url, io_config=public_storage_io_config)
+    pa_read = MicroPartition.from_arrow(read_parquet_with_pyarrow(url))
     assert daft_native_read.schema() == pa_read.schema()
     pd.testing.assert_frame_equal(daft_native_read.to_pandas(), pa_read.to_pandas())
 
@@ -294,21 +302,21 @@ def test_parquet_read_df(parquet_file, public_storage_io_config):
 )
 def test_row_groups_selection(public_storage_io_config, multithreaded_io):
     url = "s3://daft-public-data/test_fixtures/parquet-dev/mvp.parquet"
-    all_rows = Table.read_parquet(url, io_config=public_storage_io_config, multithreaded_io=multithreaded_io)
+    all_rows = MicroPartition.read_parquet(url, io_config=public_storage_io_config, multithreaded_io=multithreaded_io)
     assert len(all_rows) == 100
-    first = Table.read_parquet(
+    first = MicroPartition.read_parquet(
         url, io_config=public_storage_io_config, multithreaded_io=multithreaded_io, row_groups=[0]
     )
     assert len(first) == 10
     assert all_rows.to_arrow()[:10] == first.to_arrow()
 
-    fifth = Table.read_parquet(
+    fifth = MicroPartition.read_parquet(
         url, io_config=public_storage_io_config, multithreaded_io=multithreaded_io, row_groups=[5]
     )
     assert len(fifth) == 10
     assert all_rows.to_arrow()[50:60] == fifth.to_arrow()
 
-    repeated = Table.read_parquet(
+    repeated = MicroPartition.read_parquet(
         url, io_config=public_storage_io_config, multithreaded_io=multithreaded_io, row_groups=[1, 1, 1]
     )
     assert len(repeated) == 30
@@ -316,7 +324,7 @@ def test_row_groups_selection(public_storage_io_config, multithreaded_io):
     assert all_rows.to_arrow()[10:20] == repeated.to_arrow()[10:20]
     assert all_rows.to_arrow()[10:20] == repeated.to_arrow()[20:]
 
-    out_of_order = Table.read_parquet(
+    out_of_order = MicroPartition.read_parquet(
         url, io_config=public_storage_io_config, multithreaded_io=multithreaded_io, row_groups=[1, 0]
     )
     assert len(out_of_order) == 20
@@ -332,15 +340,24 @@ def test_row_groups_selection(public_storage_io_config, multithreaded_io):
 def test_row_groups_selection_bulk(public_storage_io_config, multithreaded_io):
     url = ["s3://daft-public-data/test_fixtures/parquet-dev/mvp.parquet"] * 11
     row_groups = [list(range(10))] + [[i] for i in range(10)]
-    first, *rest = Table.read_parquet_bulk(
-        url, io_config=public_storage_io_config, multithreaded_io=multithreaded_io, row_groups_per_path=row_groups
-    )
-    assert len(first) == 100
-    assert len(rest) == 10
 
-    for i, t in enumerate(rest):
-        assert len(t) == 10
-        assert first.to_arrow()[i * 10 : (i + 1) * 10] == t.to_arrow()
+    if MicroPartition == Table:
+        first, *rest = MicroPartition.read_parquet_bulk(
+            url, io_config=public_storage_io_config, multithreaded_io=multithreaded_io, row_groups_per_path=row_groups
+        )
+        assert len(first) == 100
+        assert len(rest) == 10
+
+        for i, t in enumerate(rest):
+            assert len(t) == 10
+            assert first.to_arrow()[i * 10 : (i + 1) * 10] == t.to_arrow()
+    else:
+        mp = MicroPartition.read_parquet_bulk(
+            url, io_config=public_storage_io_config, multithreaded_io=multithreaded_io, row_groups_per_path=row_groups
+        )
+        assert len(mp) == 100 + (
+            10 * 10
+        )  # 100 rows in first table (10 rgs), 10 rows each in subsequent tables (1 rg each)
 
 
 @pytest.mark.integration()
