@@ -2,15 +2,18 @@ from __future__ import annotations
 
 import itertools
 import operator
+from datetime import date, datetime
 
 import pyarrow as pa
 import pytest
+import pytz
 
 from daft import DataType, Series
 
 arrow_int_types = [pa.int8(), pa.uint8(), pa.int16(), pa.uint16(), pa.int32(), pa.uint32(), pa.int64(), pa.uint64()]
 arrow_string_types = [pa.string(), pa.large_string()]
 arrow_float_types = [pa.float32(), pa.float64()]
+arrow_binary_types = [pa.binary(), pa.large_binary()]
 
 
 @pytest.mark.parametrize("l_dtype, r_dtype", itertools.product(arrow_int_types + arrow_string_types, repeat=2))
@@ -493,6 +496,115 @@ def test_comparisons_dates() -> None:
     assert gt == [False, False, True, None, None, None]
 
 
+@pytest.mark.parametrize("l_dtype, r_dtype", itertools.product(arrow_binary_types, repeat=2))
+def test_comparisons_binary(l_dtype, r_dtype) -> None:
+    l_arrow = pa.array([b"1", b"22", b"333", None, b"55555", None])
+    r_arrow = pa.array([b"1", b"333", b"1", b"55555", None, None])
+    # eq, lt, gt, None, None, None
+
+    l = Series.from_arrow(l_arrow.cast(l_dtype))
+    r = Series.from_arrow(r_arrow.cast(r_dtype))
+    lt = (l < r).to_pylist()
+    assert lt == [False, True, False, None, None, None]
+
+    le = (l <= r).to_pylist()
+    assert le == [True, True, False, None, None, None]
+
+    eq = (l == r).to_pylist()
+    assert eq == [True, False, False, None, None, None]
+
+    neq = (l != r).to_pylist()
+    assert neq == [False, True, True, None, None, None]
+
+    ge = (l >= r).to_pylist()
+    assert ge == [True, False, True, None, None, None]
+
+    gt = (l > r).to_pylist()
+    assert gt == [False, False, True, None, None, None]
+
+
+@pytest.mark.parametrize("l_dtype, r_dtype", itertools.product(arrow_binary_types, repeat=2))
+def test_comparisons_binary_left_scalar(l_dtype, r_dtype) -> None:
+    l_arrow = pa.array([b"22"])
+    r_arrow = pa.array([b"1", b"22", b"333", None])
+    # gt, eq, lt
+
+    l = Series.from_arrow(l_arrow.cast(l_dtype))
+    r = Series.from_arrow(r_arrow.cast(r_dtype))
+
+    lt = (l < r).to_pylist()
+    assert lt == [False, False, True, None]
+
+    le = (l <= r).to_pylist()
+    assert le == [False, True, True, None]
+
+    eq = (l == r).to_pylist()
+    assert eq == [False, True, False, None]
+
+    neq = (l != r).to_pylist()
+    assert neq == [True, False, True, None]
+
+    ge = (l >= r).to_pylist()
+    assert ge == [True, True, False, None]
+
+    gt = (l > r).to_pylist()
+    assert gt == [True, False, False, None]
+
+
+@pytest.mark.parametrize("l_dtype, r_dtype", itertools.product(arrow_binary_types, repeat=2))
+def test_comparisons_binary_right_scalar(l_dtype, r_dtype) -> None:
+    l_arrow = pa.array([b"1", b"22", b"333", None, b"55555", None])
+    r_arrow = pa.array([b"22"])
+    # lt, eq, gt, None, gt, None
+
+    l = Series.from_arrow(l_arrow.cast(l_dtype))
+    r = Series.from_arrow(r_arrow.cast(r_dtype))
+    lt = (l < r).to_pylist()
+    assert lt == [True, False, False, None, False, None]
+
+    le = (l <= r).to_pylist()
+    assert le == [True, True, False, None, False, None]
+
+    eq = (l == r).to_pylist()
+    assert eq == [False, True, False, None, False, None]
+
+    neq = (l != r).to_pylist()
+    assert neq == [True, False, True, None, True, None]
+
+    ge = (l >= r).to_pylist()
+    assert ge == [False, True, True, None, True, None]
+
+    gt = (l > r).to_pylist()
+    assert gt == [False, False, True, None, True, None]
+
+
+@pytest.mark.parametrize("l_dtype, r_dtype", itertools.product(arrow_int_types + arrow_string_types, repeat=2))
+def test_comparisons_int_and_str_right_null_scalar(l_dtype, r_dtype) -> None:
+    l_arrow = pa.array([1, 2, 3, None, 5, None])
+    r_arrow = pa.array([None], type=r_dtype)
+    # lt, eq, gt, None, gt, None
+
+    l = Series.from_arrow(l_arrow.cast(l_dtype))
+    r = Series.from_arrow(r_arrow)
+    lt = (l < r).to_pylist()
+    assert lt == [None, None, None, None, None, None]
+
+    le = (l <= r).to_pylist()
+    assert le == [None, None, None, None, None, None]
+
+    eq = (l == r).to_pylist()
+    assert eq == [None, None, None, None, None, None]
+
+    neq = (l != r).to_pylist()
+    assert neq == [None, None, None, None, None, None]
+
+    ge = (l >= r).to_pylist()
+    assert ge == [None, None, None, None, None, None]
+
+    gt = (l > r).to_pylist()
+    assert gt == [None, None, None, None, None, None]
+
+
 class CustomZero:
     def __eq__(self, other):
         if isinstance(other, CustomZero):
@@ -572,3 +684,63 @@ def test_logicalops_pyobjects(op, expected, expected_self) -> None:
     assert op(custom_falses, values).datatype() == DataType.bool()
     assert op(custom_falses, values).to_pylist() == expected
     assert op(custom_falses, custom_falses).to_pylist() == expected_self
+
+
+@pytest.mark.parametrize("tu1, tu2", itertools.product(["ns", "us", "ms"], repeat=2))
+def test_compare_timestamps_no_tz(tu1, tu2):
+    tz1 = Series.from_pylist([datetime(2022, 1, 1)])
+    assert (tz1.cast(DataType.timestamp(tu1)) == tz1.cast(DataType.timestamp(tu2))).to_pylist() == [True]
+
+
+def test_compare_timestamps_no_tz_date():
+    tz1 = Series.from_pylist([datetime(2022, 1, 1)])
+    Series.from_pylist([date(2022, 1, 1)])
+    assert (tz1 == tz1).to_pylist() == [True]
+
+
+def test_compare_timestamps_one_tz():
+    tz1 = Series.from_pylist([datetime(2022, 1, 1)])
+    tz2 = Series.from_pylist([datetime(2022, 1, 1, tzinfo=pytz.utc)])
+    with pytest.raises(ValueError, match="Cannot perform comparison on types"):
+        assert (tz1 == tz2).to_pylist() == [True]
+
+
+def test_compare_timestamps_and_int():
+    tz1 = Series.from_pylist([datetime(2022, 1, 1)])
+    tz2 = Series.from_pylist([5])
+    with pytest.raises(ValueError, match="Cannot perform comparison on types"):
+        assert (tz1 == tz2).to_pylist() == [True]
+
+
+def test_compare_timestamps_tz_date():
+    tz1 = Series.from_pylist([datetime(2022, 1, 1, tzinfo=pytz.utc)])
+    tz2 = Series.from_pylist([date(2022, 1, 1)])
+    assert (tz1 == tz2).to_pylist() == [True]
+
+
+def test_compare_lt_timestamps_tz_date():
+    tz1 = Series.from_pylist([datetime(2022, 1, 1, tzinfo=pytz.utc)])
+    tz2 = Series.from_pylist([date(2022, 1, 6)])
+    assert (tz1 < tz2).to_pylist() == [True]
+
+
+def test_compare_lt_timestamps_tz_date_same():
+    tz1 = Series.from_pylist([datetime(2022, 1, 6, tzinfo=pytz.utc)])
+    tz2 = Series.from_pylist([date(2022, 1, 6)])
+    assert (tz1 < tz2).to_pylist() == [False]
+
+
+@pytest.mark.parametrize("tu1, tu2", itertools.product(["ns", "us", "ms"], repeat=2))
+def test_compare_timestamps_same_tz(tu1, tu2):
+    tz1 = Series.from_pylist([datetime(2022, 1, 1, tzinfo=pytz.utc)]).cast(DataType.timestamp(tu1, "UTC"))
+    tz2 = Series.from_pylist([datetime(2022, 1, 1, tzinfo=pytz.utc)]).cast(DataType.timestamp(tu2, "UTC"))
+    assert (tz1 == tz2).to_pylist() == [True]
+
+
+@pytest.mark.parametrize("tu1, tu2", itertools.product(["ns", "us", "ms"], repeat=2))
+def test_compare_timestamps_diff_tz(tu1, tu2):
+    utc = datetime(2022, 1, 1, tzinfo=pytz.utc)
+    eastern = utc.astimezone(pytz.timezone("US/Eastern"))
+    tz1 = Series.from_pylist([utc]).cast(DataType.timestamp(tu1, "UTC"))
+    tz2 = Series.from_pylist([eastern]).cast(DataType.timestamp(tu1, "US/Eastern"))
+    assert (tz1 == tz2).to_pylist() == [True]
